<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 9e94440b0a1e $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-09 17:48 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * LiveUpdateTemplates is a helper function taking care of the view of the
 * update process. The functions here are only called from templates!
 *
 */
class welu_update_templates{

	/**
	 * returns standard html container for output
	 *
	 * @param string $headline
	 * @param string $content
	 * @return string
	 */
	private static function getContainer(string $headline, string $content, string $buttons = ''): string{
		return we_html_element::div(['id' => 'contentDiv'], we_html_element::div(['id' => 'contentHeadlineDiv'], '<b>' . $headline . '</b><hr />') .
				we_html_element::div(['id' => 'contentTextDiv'], $content) .
				($buttons ? we_html_element::div(['id' => 'buttonDiv'], $buttons) : '')
		);
	}

	/**
	 * returns header of template
	 *
	 * @return string
	 */
	private static function getHtmlHead(): string{
		return we_html_tools::metaCtCharset() . LIVEUPDATE_CSS;
	}

	/**
	 * Returns a html page as response
	 *
	 * @param string $headline
	 * @param string $content
	 * @param string $header
	 * @param string $buttons
	 * @return string
	 */
	static function getHtml(string $headline, string $content, string $header = '', string $buttons = ''): string{
		return we_html_tools::getHtmlTop('', self::getHtmlHead() .
				$header, '<body>' .
				self::getContainer($headline, $content, $buttons) .
				'</body>');
	}

	public static function connect(): string{
		$checkButton = we_html_button::create_button(we_html_button::NEXT, welu_update_const::LOCAL_SCRIPT . '?section=connect&update_cmd=checkConnection&clientLng=' . we_base_WE::$Language . '_UTF-8');

		return welu_update_templates::getHtml(g_l('liveUpdate', '[connect][headline]'), we_html_element::div([], g_l('liveUpdate', '[connect][description]') . '
	<br/><br/>' . $checkButton));
	}

	public static function connectError(): string{
		$errorMessage = //(isset($Response) ? str_replace('</body></html>', '', stristr($Response, '<body>') ?: '') : '') .
			we_html_element::div(['id' => 'contentHeadlineDiv', 'style' => 'height: 30px; margin-top:30px;'],
												'<b>' . g_l('liveUpdate', '[connect][connectionInfo]') . '<hr /></b>') . '<br/>
	<li>' . g_l('liveUpdate', '[connect][availableConnectionTypes]') . ':
	<ul>' .
			(ini_get('allow_url_fopen') == "1" ?
			'<li>fopen</li>' : '') .
			(is_callable('curl_exec') ?
			'<li>curl</li>' : '') .
			'</ul>
	<li>' . g_l('liveUpdate', '[connect][connectionType]') . ": ";

		if(!empty($_SESSION['le_proxy_use'])){
			$errorMessage .= 'Proxy (fsockopen)' .
				'<ul>
<li>' . g_l('liveUpdate', '[connect][proxyHost]') . ": " . $_SESSION['le_proxy_host'] . '</li>
<li>' . g_l('liveUpdate', '[connect][proxyPort]') . ": " . $_SESSION['le_proxy_port'] . '</li>';

			if(preg_match('/(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)/', $_SESSION['le_proxy_host'])){
				$errorMessage .= '<li>' . g_l('liveUpdate', '[connect][ipResolutionTest]') . ' (IPv4 only): ';
				$hostName = gethostbyaddr((string) $_SESSION['le_proxy_host']);
				if($hostName != $_SESSION['le_proxy_host']){
					$errorMessage .= g_l('liveUpdate', '[connect][succeeded]') . '.</li>' .
						"<li>" . g_l('liveUpdate', '[connect][hostName]') . ': ' . $hostName . '</li>';
				} else {
					$errorMessage .= g_l('liveUpdate', '[connect][failed]') . '.</li>';
				}
			}
			// gethostbyaddr currently does not support ipv6 address resolution
			else {
				$errorMessage .= '<li>' . g_l('liveUpdate', '[connect][dnsResolutionTest]') . ': ';
				if(($ipAddr = gethostbynamel($_SESSION['le_proxy_host']))){
					$errorMessage .= g_l('liveUpdate', '[connect][succeeded]') . '.</li>' .
						"<li>" . g_l('liveUpdate', '[connect][ipAddresses]') . ': ' . implode(',', $ipAddr) . "</li>";
				} else {
					$errorMessage .= g_l('liveUpdate', '[connect][failed]') . '.</li>';
				}
			}
			$errorMessage .= '</ul>';
		} else {
			$errorMessage .= liveUpdateHttp::getHttpOption();
		}
		$errorMessage .= "</li>
<li>" . g_l('liveUpdate', '[connect][addressResolution]') . ' ' . g_l('liveUpdate', '[connect][updateServer]') . ":</li>
<ul><li>" . g_l('liveUpdate', '[connect][hostName]') . ": " . welu_update_const::SERVER . "</li>";

		$errorMessage .= "<li>" . g_l('liveUpdate', '[connect][dnsResolutionTest]') . ": " .
			(($ipAddr = gethostbynamel(welu_update_const::SERVER)) ?
			g_l('liveUpdate', '[connect][succeeded]') . '.</li><li>' . g_l('liveUpdate', '[connect][ipAddresses]') . ": " . implode(',', $ipAddr) . "</li>" :
			g_l('liveUpdate', '[connect][failed]') . ".</li>"
			) . "</ul>";

		$jsCmd = new we_base_jsCmd();
		$jsCmd->addMsg(g_l('liveUpdate', '[connect][connectionErrorJs]'), we_base_util::WE_MESSAGE_FRONTEND);
		return welu_update_templates::getHtml(g_l('liveUpdate', '[connect][headline]'), we_html_element::div([], g_l('liveUpdate', '[connect][connectionError]')) . $errorMessage, $jsCmd->getCmds());
	}

	public static function languages(welu_update_frames $frame): string{
		$nextButton = we_html_button::create_button(we_html_button::NEXT, welu_update_const::LOCAL_SCRIPT . '?section=languages&update_cmd=languages&detail=selectLanguages');
		$deleteButton = we_html_button::create_button(we_html_button::DELETE, 'javascript:document.we_form.submit()');

		$languages = liveUpdateFunctions::getInstalledLanguages();

		$languagesStr = '';
		foreach($languages as $lng){
			'@phan-var string $lng';
			switch($lng){
				case WE_LANGUAGE:
					$lngBox = we_html_forms::checkbox($lng, false, 'deleteLanguages[]', '<i>' . $lng . ' (' . g_l('liveUpdate', '[languages][systemLanguage]') . ")</i>", false, '', '', true);
					break;
				case we_base_WE::$Language:
					$lngBox = we_html_forms::checkbox($lng, false, 'deleteLanguages[]', '<i>' . $lng . ' (' . g_l('liveUpdate', '[languages][usedLanguage]') . ")</i>", false, '', '', true);
					break;
				default:
					$lngBox = we_html_forms::checkbox($lng, false, 'deleteLanguages[]', $lng, true);
					break;
			}
			$languagesStr .= we_html_element::div([], $lngBox);
		}

		$deletedLngs = $frame->getData('deletedLngs');
		$notDeletedLngs = $frame->getData('notDeletedLngs');
		$jsAlert = '';
		if(!empty($deletedLngs)){
			$jsAlert .= g_l('liveUpdate', '[languages][languagesDeleted]') . '\n';
			foreach($deletedLngs as $lng){
				$jsAlert .= $lng . '\n';
			}
		}
		if(!empty($notDeletedLngs)){
			$jsAlert .= g_l('liveUpdate', '[languages][languagesNotDeleted]') . '\n';
			foreach($notDeletedLngs as $lng){
				$jsAlert .= $lng . '\n';
			}
		}

		$jsCmd = new we_base_jsCmd();
		if($jsAlert){
			$jsCmd->addMsg($jsAlert, we_base_util::WE_MESSAGE_FRONTEND);
		}

		return welu_update_templates::getHtml(g_l('liveUpdate', '[languages][headline]'), we_html_element::form(['name' => we_base_constants::WE_FORM], we_html_element::hidden('section', 'languages') . '
' . g_l('liveUpdate', '[languages][installedLngs]') . '<br/>
' . $languagesStr . '<br/>
<table style="width:100%">
<tr>
	<td>' . g_l('liveUpdate', '[languages][showLanguages]') . '</td>
	<td>' . $nextButton . '</td>
</tr>
<tr>
	<td><br/></td>
</tr>
<tr>
	<td>' . g_l('liveUpdate', '[languages][deleteSelectedLanguages]') . '</td>
	<td>' . $deleteButton . '</td>
</tr>
</table>'), $jsCmd->getCmds());
	}

	public static function tabs(welu_update_frames $frame): string{
		$tabs = new we_gui_tabs();
		foreach([
		'update' => 'fa-recycle',
		'languages' => 'fa-language',
		'updatelog' => 'fa-list-alt',
		'connect' => 'fa-handshake-o',
		//'beta'=> 'fa-unlock-alt',
		] as $name => $icon){
			$tabs->addTab(g_l('liveUpdate', '[tabs][' . $name . ']'), $icon, ($frame->Data['activeTab'] === $name), "'" . $name . "'");
		}

// get output

		return we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
				we_html_element::jsScript(JS_DIR . 'initTabs.js') .
				we_html_element::jsScript(JS_DIR . 'update/updatelog.js'), we_html_element::body([
					'id' => 'eHeaderBody',
					'onload' => 'weTabs.setFrameSize();',
					'onresize' => 'weTabs.setFrameSize()'
					], we_html_element::div(['id' => "main"], $tabs->getHTML()))
		);
	}

	public static function update(welu_update_frames $frame): string{
		if(!isset($_SESSION['weS']['testUpdate'])){
			$_SESSION['weS']['testUpdate'] = false;
		}
		/* if(we_base_request::bool('setBeta')){
		  $_SESSION['weS']['testUpdate'] = we_base_request::bool('setTestUpdate', $_SESSION['weS']['testUpdate'] ?? false);
		  } */
		$alsoBeta = (defined('WE_VERSION_SUPP') && WE_VERSION_SUPP != 'release' ? '&setTestUpdate=1' : '');

		return welu_update_templates::getHtml(g_l('liveUpdate', '[update][headline]'), '
<table style="width:100%">
<tr>
	<td>' . g_l('liveUpdate', '[update][actualVersion]') . '</td>
	<td>' . (!empty(WE_VERSION_NAME) ? WE_VERSION_NAME : WE_VERSION) . ' (' . WE_VERSION . (WE_SVNREV != '0000' ? ', Revision: ' . WE_SVNREV : '') . ')</td>
</tr>
<tr>
	<td>' . g_l('liveUpdate', '[update][lastUpdate]') . '</td>
	<td>' . $frame->Data['lastUpdate'] . '</td>
</tr>
<tr><td style="padding-top:2em;padding-bottom:2em;">' . g_l('liveUpdate', '[beta][lookForUpdate]') . '</td><td><form name="betaform" action="' . welu_update_const::LOCAL_SCRIPT . '?section=update" method="post">' . we_html_forms::checkboxWithHidden($_SESSION['weS']['testUpdate'], 'setTestUpdate', g_l('liveUpdate', '[tabs][beta]'), false, '', 'betaform.submit()') . we_html_element::hidden('setBeta', 1) . '</form></td></tr>
<tr>
	<td>' . g_l('liveUpdate', '[update][lookForUpdate]') . '</td>
	<td>' . (SYSTEM_WE_SESSION_CRYPT === we_base_sessionHandler::CRYPT_COOKIE ? we_html_tools::alertAttentionBox(g_l('liveUpdate', '[session][cryptCookieWarning]'), we_html_tools::TYPE_ALERT, '100%', false) : we_html_button::create_button(we_html_button::SEARCH, welu_update_const::LOCAL_SCRIPT . '?section=update&update_cmd=update&detail=lookForUpdate' . $alsoBeta)) . '</td>
</tr>
</table>');
	}

	public static function updateLog(welu_update_frames $frame): string{

		/*
		 * items found - show them in table
		 */
		$buttons = '';

		if($frame->Data['allEntries']){ // entries exist
			// there are entries available -> show them in table
			$start = $frame->Data['start'];
			$content = '
<form name="' . we_base_constants::WE_FORM . '">' . we_html_element::hiddens([
					'section' => we_base_request::string('section'),
					'log_cmd' => 'dummy',
					'start' => $start
				]) . '
<table style="width:100%">
<tr>
	<td>' . g_l('liveUpdate', '[updatelog][entriesTotal]') . ': ' . $frame->Data['amountEntries'] . '</td>
	<td class="alignRight">' . g_l('liveUpdate', '[updatelog][page]') . ' ' . (($start / $frame->Data['amountPerPage']) + 1) . '/ ' . ((ceil($frame->Data['amountEntries'] / $frame->Data['amountPerPage'])) ?: 1) . '</td>
</tr>
</table>' .
				we_html_element::div([], we_html_forms::checkbox('1', we_base_request::bool('messages'), 'messages', we_html_element::span([
							'class' => "logMessage"], g_l('liveUpdate', '[updatelog][legendMessages]') . ' (' . $frame->Data['amountMessages'] . ")"), false, "small", "document.we_form.submit();") .
					we_html_forms::checkbox('1', we_base_request::bool('notices'), 'notices', we_html_element::span(['class' => "logNotice"], g_l('liveUpdate', '[updatelog][legendNotices]') . ' (' . $frame->Data['amountNotices'] . ")"), false, "small", "document.we_form.submit();") .
					we_html_forms::checkbox('1', we_base_request::bool('errors'), 'errors', we_html_element::span(['class' => "logError"], g_l('liveUpdate', '[updatelog][legendErrors]') . ' (' . $frame->Data['amountErrors'] . ")"), false, "small", "document.we_form.submit();")
				) . we_html_element::br();

			if(($frame->Data['logEntries'])){ // entries match filter
				$content .= '
<table style="width:100%" class="updateContent" id="updateLogTable">
<tr>
	<th>' . g_l('liveUpdate', '[updatelog][date]') . '</th>
	<th>' . g_l('liveUpdate', '[updatelog][action]') . '</th>
	<th>' . g_l('liveUpdate', '[updatelog][version]') . '</th>
</tr>';

				foreach($frame->Data['logEntries'] as $logEntry){
					switch($logEntry['state']){
						case 1:
							$classStr = ' class="logError"';
							break;
						case 2:
							$classStr = ' class="logNotice"';
							break;
						default:
							$classStr = ' class="logMessage"';
					}

					$content .= '<tr' . $classStr . '>
		<td style="vertical-align:top">' . $logEntry['date'] . '</td>
		<td>' . $logEntry['action'] . '</td>
		<td style="vertical-align:top">' . $logEntry['version'] . '</td>
	</tr>';
				}

				/*
				 * Add buttons for next, back and delete
				 */

				$buttons = we_html_button::create_button(we_html_button::DELETE, "javascript:confirmDelete();") .
					($start > 0 ? //	backbutton
					we_html_button::create_button(we_html_button::BACK, "javascript:lastEntries();") :
					we_html_button::create_button(we_html_button::BACK, '', '', '', true)) .
					($frame->Data['amountEntries'] <= $start + $frame->Data['amountPerPage'] ? //	next_button
					we_html_button::create_button(we_html_button::NEXT, '', '', '', true) :
					we_html_button::create_button(we_html_button::NEXT, "javascript:nextEntries();"));

				$content .= '
</table>';
			} else {
				$content .= g_l('liveUpdate', '[updatelog][noEntriesMatchFilter]');
			}
			$content .= '
</form>';
		} else {
			$content = g_l('liveUpdate', '[updatelog][logIsEmpty]');
		}

		return welu_update_templates::getHtml(g_l('liveUpdate', '[updatelog][headline]'), $content, we_html_element::jsScript(JS_DIR . 'update/updatelog.js'), $buttons);
	}

	/* public static function beta(welu_update_frames $frame): string{

	  $_SESSION['weS']['testUpdate'] = we_base_request::bool('setTestUpdate', $_SESSION['weS']['testUpdate'] ?? false);

	  return welu_update_templates::getHtml(g_l('liveUpdate', '[beta][headline]'), '
	  <table style="width:100%">
	  <tr>
	  <td>' . g_l('liveUpdate', '[update][actualVersion]') . '</td>
	  <td>' . WE_VERSION . '</td>
	  </tr>
	  <tr>
	  <td>' . g_l('liveUpdate', '[update][lastUpdate]') . '</td>
	  <td>' . ($frame->Data ? $frame->Data['lastUpdate'] : '') . '</td>
	  </tr>
	  <tr><td colspan="2"><br/><br/></td></tr>
	  <tr>
	  <td></td><td><form name="betaform" action="' . welu_update_const::LOCAL_SCRIPT . '?section=beta" method="post">' . we_html_forms::checkboxWithHidden($_SESSION['weS']['testUpdate'], 'setTestUpdate', g_l('liveUpdate', '[beta][lookForUpdate]'), false, '', 'betaform.submit()') . '</form>
	  <br/>
	  <br/>
	  </td>
	  </tr>
	  <tr><td colspan="2">' . g_l('liveUpdate', '[beta][warning]', false, true) . '<br/><br/></td></tr>
	  </table>');
	  } */
}
