<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
/* * New Format for classes: we_Domain_specific
 * 1. ALL classes start with we_
 * 2. ALL files in one domain
 * 3. all files of one domain reside in the same directory
 * 4. if further division is needed, create a subdomain which does not contain _ !
 * 5. domains are looked up in this class
 * 6. NOTE that files MUST end with .class.php
 */

abstract class we_base_autoloader{
	//fallback classes if local classes do not exist - mostly pear
	private const FALLBACK = [
		'Console_Getopt' => 'we/libraries/pear/Getopt.php',
		'DB_WE' => 'we/classes/database/we_database_mysqli.class.php',
		'Image_IPTC' => 'we/libraries/pear/Image_IPTC.class.php',
		'Less_Parser' => 'we/libraries/Less/Parser.php',
		'PEAR' => 'we/libraries/pear/PEAR.php',
		'PEAR_Error' => 'we/libraries/pear/PEAR.php',
		'PHPGangsta_GoogleAuthenticator' => 'we/libraries/googleAuthenticator/googleAuthenticator.class.php',
		'Services_JSON' => 'we/libraries/pear/Services_JSON.class.php', //remove in cardada
		'Services_JSON_Error' => 'we/libraries/pear/Services_JSON.class.php', //remove in cardada
		'lessc' => 'we/libraries/Less/lessc.inc.php',
		'liveUpdateFrames' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		'paypal_class' => 'we/libraries/paypal/paypal.class.php',
		'we_base_updater' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		'we_imageDocument' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		'we_objectFile' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		'we_updater' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		'we_webEditionDocument' => 'we/classes/update/we_update_deprecated.class.php', //FIXME: remove in cardada
		//add upgrade classes
		//'' => 'we/include/we_classes/update/we_update_upgrade.class.php',
	];

	private static $nameSpaces = [
		'Sabre' => [WE_LIB_DIR . 'sabre', '.php'],
		'Psr' => [WE_LIB_DIR . 'Psr', '.php'],
	];

	/**
	 * default webEdition autoloader
	 * @param string $class_name
	 */
	public static function autoload(string $class_name): void{
		@[$where, $domain] = explode('_', $class_name, 3);
		switch($where){
			case 'we':
				if(file_exists(($file = WE_CLASS_PATH . $domain . '/' . $class_name . '.class.php'))){
					include($file);
					return;
				}
				switch($domain){
					case 'tag':
						if(file_exists(($file = WE_CUSTOM_PATH . '/' . we_weTag_tag::DIR . '/' . $class_name . '.class.php'))){//special handling of custom tags
							include($file);
							return;
						}
						break;
					case 'hook':
						if(file_exists(($file = WE_CUSTOM_PATH . '/' . we_hook_handler::DIR . '/' . $class_name . '.class.php'))){//special handling of custom hooks
							include($file);
							return;
						}
				}
				break;
			case 'welu':
				if(file_exists(($file = WEBEDITION_PATH . 'liveUpdate/classes/' . $domain . '/' . $class_name . '.class.php'))){
					include($file);
					return;
				}
				break;
			case 'Horde':
				include(WE_LIB_PATH . str_replace('_', '/', $class_name) . '.php');
				return;
		}

		//might be a class of fallback
		if(array_key_exists($class_name, self::FALLBACK)){
			include(WEBEDITION_PATH . self::FALLBACK[$class_name]);
			return;
		}
		if(str_contains($class_name, '\\')){
			$ns = explode('\\', $class_name);
			$namespace = array_shift($ns);
			if(array_key_exists($namespace, self::$nameSpaces)){
				include(WE_DOCUMENT_ROOT . self::$nameSpaces[$namespace][0] . '/' . implode('/', $ns) . self::$nameSpaces[$namespace][1]);
				return;
			}
		}

		// add this loader at the end, if class was not yet found
		spl_autoload_register('we_base_autoloader::finalLoad', true);
		//will try next auto-loader, if present - else finalload is called
	}

	public static function registerNameSpace(string $nameSpace, string $trgDir, string $ext): void{
		self::$nameSpaces[$nameSpace] = [$trgDir, $ext];
	}

	/**
	 * Added after last loader to trigger not found classes
	 * @param string $class_name
	 */
	public static function finalLoad(string $class_name): void{
		if(array_key_exists($class_name, self::FALLBACK)){
			include(WEBEDITION_PATH . self::FALLBACK[$class_name]);
			return;
		}
		//t_e('notice', 'we_base_autoloader: class ' . $class_name . ' not found');
	}

}
