<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2f1aeda46386 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-02-13 01:48 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class we_history
 *
 * Provides functions determined to handle a list of last modified files required by
 * the 'personalized desktop'.
 */
abstract class we_base_history{
	private const MAX = 50;

	static function userHasPerms(int $creatorid, string $owners, bool $restricted): bool{
		return (we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || !$restricted || we_users_util::isOwner(explode(',', $owners)) || we_users_util::isOwner([
				$creatorid]));
	}

	static function insertIntoHistory(we_contents_base $object): void{
		if($object instanceof we_folder_base){
			return;
		}
		$db = DB_WE::$inst;
		$uid = !we_base_WE::$schedulerActive ? we_users_user::getID() : 0;
		$tab = stripTblPrefix($object->Table);
		$cnt = fI('SELECT COUNT(1) FROM ' . HISTORY_TABLE . ' WHERE DocumentTable="' . $tab . '" AND UID=' . $uid);
		if($cnt > self::MAX){
			$db->query('DELETE FROM ' . HISTORY_TABLE . ' WHERE DocumentTable="' . $tab . '" AND UID=' . $uid . ' ORDER BY Mod_Date ASC LIMIT ' . ($cnt - self::MAX));
		}

		$db->query('REPLACE INTO ' . HISTORY_TABLE . ' SET ' . we_database_base::arraySetter(['DID' => intval($object->ID),
				'DocumentTable' => $tab,
				'UserName' => (we_base_WE::$schedulerActive ? 'Scheduler' : ($_SESSION['user']['Username'] ?? ($_SESSION['webuser']['Username'] ?? 'Unknown'))),
				'UID' => $uid,
		]));
	}

	/**
	 * Deletes a model from navigation History
	 *
	 * @param int[] $modelIds
	 * @param string $table
	 */
	static function deleteFromHistory(array $modelIds, string $table): void{
		DB_WE::$inst->query('DELETE FROM ' . HISTORY_TABLE . ' WHERE DID IN (' . implode(', ', $modelIds) . ') AND DocumentTable="' . stripTblPrefix($table) . '"');
	}

	public static function deleteByUserID(int $uid): void{
		if($uid){
			$db = DB_WE::$inst;
			$db->query('DELETE FROM ' . HISTORY_TABLE . ' WHERE UID=' . $uid);
		}
	}

}
