<?php
declare(strict_types=1);
/**
 * webEdition CMS
 *
 * $Rev: 773bebbd9b07 $
 * $Author: Lukas Imhof $
 * $Date: 2020-11-10 16:24 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_base_srcEditorConsts{

	public static function process(string $what): string{
		$stat = stat(__FILE__);
		$setting = we_base_request::_(we_base_request::STRING, 'settings', []);
		$etag = md5($stat['size'] . $stat['ctime'] . $stat['mtime'] . serialize($setting));
		$modDate = gmdate('D, d M Y H:i:s', $stat['mtime']) . ' GMT';
		header('Etag: "' . $etag . '"');
		header('Last-Modified: ' . $modDate, true);
		header('Expires: ' . gmdate('D, d M Y H:i:s', time() + we_base_util::SECONDS_DAY) . ' GMT', true);
		header('Cache-Control: max-age=' . we_base_util::SECONDS_DAY . ', must-revalidate', true);
		header_remove('Pragma');

		if((!empty($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] === $etag) ||
			(!empty($_SERVER['HTTP_IF_MODIFIED_SINCE']) && $_SERVER['HTTP_IF_MODIFIED_SINCE'] === $modDate)){
			we_html_tools::setHttpCode(304);
			return '';
		}
		switch($what){
			case 'CMCSS':
				return self::getCMCSS($setting);
			case 'tagCSS':
				we_html_tools::headerCtCharset('text/css');
				return we_contents_template::we_getCodeMirror2Tags(true, we_base_request::_(we_base_request::BOOL, 'settings'));
			case 'tagJS':
				we_html_tools::headerCtCharset('text/javascript');
				return we_contents_template::we_getCodeMirror2Tags(false, we_base_request::_(we_base_request::BOOL, 'settings'));
		}
		t_e('sth. went wrong', $what);
		return '';
	}

	/**
	 *
	 * @param array<string,string[]> $setting
	 * @return string
	 */
	private static function getCMCSS(array $setting): string{
		if(empty($setting)){
			return '';
		}

		we_html_tools::headerCtCharset('text/css');
		return '
.weSelfClose:hover:after,
.cm-weSelfClose:hover:after,
.weOpenTag:hover:after,
.cm-weOpenTag:hover:after,
.weTagAttribute:hover:after,
.cm-weTagAttribute:hover:after {
	font-family: ' . $setting['toolFont']['family'] . ';
	font-size: ' . $setting['toolFont']['size'] . 'px;
}

.CodeMirror,
.CodeMirror.cm-s-ambiance,
.CodeMirror.cm-s-solarized {
	font-family: ' . $setting['font']['family'] . ' !important;
	font-size: ' . $setting['font']['size'] . 'px !important;
}

' . ($setting['tab'] ? '' : '
.CodeMirror .cm-tab {
color: transparent;
}

') .
			(empty($setting['Template']['tid']) ? '' :
			we_contents_template::we_getCSSIds((int) $setting['Template']['tid']));
	}

}
