<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * General Definition of Customer's Prefers
 *
 */
class we_customer_settings{
	/** @var we_database_base */
	private $db;

	/** @var array<string,string> */
	public $properties = [];

	/** @var array<string,array{string,mixed}> */
	const FIELD_TYPES = [
		//name => [dbtype,default]
		'input' => ['varchar(255)', ''],
		'number' => ['int(11)', 0],
		'select' => ['enum', null],
		'multiselect' => ['set', null],
		'textarea' => ['text', ''],
		'dateTime' => ['datetime', null],
		'date' => ['date', null],
		'password' => ['varchar(32)', ''],
		'img' => ['int unsigned', 0],
		'country' => ["enum('AD','AE','AF','AG','AI','AL','AM','AO','AQ','AR','AS','AT','AU','AW','AX','AZ','BA','BB','BD','BE','BF','BG','BH','BI','BJ','BL','BM','BN','BO','BQ','BR','BS','BT','BV','BW','BY','BZ','CA','CC','CD','CF','CG','CH','CI','CK','CL','CM','CN','CO','CR','CU','CV','CW','CX','CY','CZ','DE','DJ','DK','DM','DO','DZ','EC','EE','EG','EH','ER','ES','ET','FI','FJ','FK','FM','FO','FR','GA','GB','GD','GE','GF','GG','GH','GI','GL','GM','GN','GP','GQ','GR','GS','GT','GU','GW','GY','HK','HM','HN','HR','HT','HU','ID','IE','IL','IM','IN','IO','IQ','IR','IS','IT','JE','JM','JO','JP','KE','KG','KH','KI','KM','KN','KP','KR','KW','KY','KZ','LA','LB','LC','LI','LK','LR','LS','LT','LU','LV','LY','MA','MC','MD','ME','MG','MH','MK','ML','MM','MN','MO','MP','MQ','MR','MS','MT','MU','MV','MW','MX','MY','MZ','NA','NC','NE','NF','NG','NI','NL','NO','NP','NR','NU','NZ','OM','PA','PE','PF','PG','PH','PK','PL','PM','PN','PR','PS','PT','PW','PY','QA','RE','RO','RU','RW','SA','SB','SC','SD','SE','SG','SH','SI','SJ','SK','SL','SM','SN','SO','SR','ST','SV','SX','SY','SZ','TC','TD','TF','TG','TH','TJ','TK','TL','TM','TN','TO','TR','TT','TV','TW','TZ','UA','UG','UM','US','UY','UZ','VA','VC','VE','VG','VI','VN','VU','WF','WS','YE','YT','YU','ZA','ZM','ZW')",
			null],
		'language' => ["enum('ab','af','an','ar','as','az','be','bg','bn','bo','br','bs','ca','ce','co','cs','cu','cy','da','de','el','en','eo','es','et','eu','fa','fi','fj','fo','fr','fy','ga','gd','gl','gv','he','hi','hr','ht','hu','hy','id','is','it','ja','jv','ka','kg','ko','ku','kw','ky','la','lb','li','ln','lt','lv','mg','mk','mn','mo','ms','mt','my','nb','ne','nl','nn','no','oc','pl','pt','rm','ro','ru','sc','se','sk','sl','so','sq','sr','sv','sw','tk','tr','ty','uk','ur','uz','vi','vo','yi','zh')",
			null]
	];
	private const SPECIAL_FIELD_TYPES = [
		'select' => 'enum',
		'multiselect' => 'set',
	];
	const ORDER_TABLE = [
		'ASC' => 'ASC',
		'DESC' => 'DESC'
	];
	const FUNCTION_TABLE = [
		'FIRSTCHAR' => 'UPPER(SUBSTRING(%s,1,1))',
		'MINUTE' => 'DATE_FORMAT(%s,\'%%i\')',
		'HOUR' => 'DATE_FORMAT(%s,\'%%H\')',
		'DAY' => 'DAYOFMONTH(%s)',
		'MONTH' => 'MONTH(%s)',
		'YEAR' => 'YEAR(%s)',
		'DAYOFWEEK' => 'DAYOFWEEK(%s)',
		'DAYOFMONTH' => 'DAYOFMONTH(%s)',
		'DAYOFYEAR' => 'DAYOFYEAR(%s)',
		'DAYNAME' => 'DAYNAME(%s)',
		'MONTHNAME' => 'MONTHNAME(%s)',
		'QUARTER' => 'QUARTER(%s)'
	];
	private const TYPE_FUNCTION = [
		'FIRSTCHAR' => 'input,select,textarea,password',
		'MINUTE' => 'date',
		'HOUR' => 'date',
		'DAY' => 'date',
		'MONTH' => 'date',
		'YEAR' => 'date',
		'DAYOFWEEK' => 'date',
		'DAYOFMONTH' => 'date',
		'DAYOFYEAR' => 'date',
		'DAYNAME' => 'date',
		'MONTHNAME' => 'date',
		'QUARTER' => 'date'
	];

	/** @var int */
	public const MaxSearchResults = 99999;

	/** @var array<string,string> */
	private $PropertyTitle = [];

	/** @var array<string,array{type:string,default:string,encrypt:bool}> */
	public $FieldAdds = [];

	/** @var array<string,array<int,array{branch:string,field:string,function:string,order:string}>> */
	public $SortView = [];

	/** @var array<string,string> */
	private $Prefs = [
		'treetext_format' => '#Username (#Forename #Surname)',
		'start_year' => '1900',
		'default_sort_view' => '',
		'default_order' => '',
	];

	/** @var string */
	public $treeTextFormatSQL = '';

	/** @var string[] */
	public $formatFields = [];

	function __construct(){
		$this->db = new DB_WE();

		$this->PropertyTitle = [
			'Username' => g_l('modules_customer', '[username]'),
			'Password' => g_l('modules_customer', '[password]'),
			'Forename' => g_l('modules_customer', '[Forname]'),
			'Surname' => g_l('modules_customer', '[Surname]'),
			'LoginDenied' => g_l('modules_customer', '[login]'),
			'AutoLoginDenied' => g_l('modules_customer', '[autologin]'),
			'AutoLogin' => g_l('modules_customer', '[autologin]'),
			'MemberSince_Date' => g_l('modules_customer', '[MemeberSince]'),
			'LastLogin_Date' => g_l('modules_customer', '[LastLogin]'),
			'LastAccess_Date' => g_l('modules_customer', '[LastAccess]'),
			'ID' => 'ID',
		];
	}

	function load(): void{
		$modified = false;
		$this->properties = $this->db->getAllFirstq('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="customer"', false);
		$this->SortView = (empty($this->properties['SortView']) ? [] : we_unserialize((string) $this->properties['SortView']));

		if(!empty($this->properties['FieldAdds'])){
			$this->FieldAdds = we_unserialize((string) $this->properties['FieldAdds']);
			//check if all fields are set
			$fields = we_customer_customer::getFieldset();
			foreach($fields as $name){
				$name = (string) $name;
				$tmp = we_customer_customer::getFieldDbProperties($name);
				$old = $this->FieldAdds[$name]['type'] ?? '';
				$this->FieldAdds[$name]['default'] = $tmp['Default'] ?: '';

				switch($old){
					case 'input':
					case '':
						$this->FieldAdds[$name]['type'] = 'input';
						$modified = $modified || ($old != $this->FieldAdds[$name]['type']);
						break;
					case 'img':
					case 'number':
						$this->FieldAdds[$name]['default'] = '0';
						break;
					case 'select':
					case 'multiselect':
						$enum_array = [];
						preg_match_all("/'(.*?)'/", $tmp['Type'], $enum_array);
						$this->FieldAdds[$name]['default'] = implode(',', $enum_array[1]);
				}
				if(!isset($this->FieldAdds[$name]['encrypt'])){
					$modified = true;
					$this->FieldAdds[$name]['encrypt'] = false;
				}
			}
		}

		if(!empty($this->properties['Prefs'])){
			$this->Prefs = array_merge($this->Prefs, we_unserialize((string) $this->properties['Prefs']));
		}

		$this->treeTextFormatSQL = 'COALESCE(NULLIF(TRIM(CONCAT("' . $this->Prefs['treetext_format'] . '")),""),Username)';
		$field_names = DB_WE::$inst->metaFieldNames(CUSTOMER_TABLE);

		foreach($field_names as $fieldname){
			if(str_contains($this->treeTextFormatSQL, '#' . $fieldname)){
				$this->formatFields[] = $fieldname;
			}
			$this->treeTextFormatSQL = str_replace('#' . $fieldname, '",`' . $fieldname . '`,"', $this->treeTextFormatSQL);
		}

		if($modified){
			$this->save();
		}
	}

	function save(): bool{
		//FIXME: make Fieldadds more fields in DB
		$tmp = $this->FieldAdds;
		foreach($tmp as &$cur){
			unset($cur['default']);
		}
		$this->properties['FieldAdds'] = we_serialize($tmp, SERIALIZE_JSON);
		$this->properties['SortView'] = we_serialize($this->SortView, SERIALIZE_JSON);
		$this->properties['Prefs'] = we_serialize($this->Prefs, SERIALIZE_JSON);
		foreach($this->properties as $key => $value){
			'@phan-var string $value';
			$this->db->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET tool="customer",pref_value="' . $this->db->escape($value) . '",pref_name="' . ((string) $key) . '"');
		}
		return true;
	}

	function getPref(string $pref_name): string{
		return (string) $this->properties[$pref_name];
	}

	function setPref(string $pref_name, string $pref_value): void{
		$this->properties[$pref_name] = $pref_value;
	}

	function isFunctionForField(string $function, string $field): bool{
		if(str_contains($field, g_l('modules_customer', '[other]'))){
			$field = str_replace(g_l('modules_customer', '[other]') . '_', '', $field);
		}
		$fieldprops = we_customer_customer::getFieldDbProperties($field);

		$fieldtype = $this->getFieldType($fieldprops['Field'], $fieldprops['Type']);
		if($fieldtype != ''){
			foreach(self::TYPE_FUNCTION as $fk => $fv){
				$tmp = explode(',', $fv);
				if(($fk == $function) && (in_array($fieldtype, $tmp))){
					return true;
				}
			}
		}
		return false;
	}

	//returns db field type
	function getDbType(string $field_type, string $fieldname, string $default): string{
		if(array_key_exists($field_type, self::SPECIAL_FIELD_TYPES)){
			if(empty($this->FieldAdds[$fieldname]['default'])){
				$def = self::FIELD_TYPES['input'][1];
				'@phan-var mixed $def';
				return self::FIELD_TYPES['input'][0] . ($def === null ? ' NULL DEFAULT NULL' : ' NOT NULL DEFAULT \'' . $def . '\'');
			}
			return self::SPECIAL_FIELD_TYPES[$field_type] . '(\'' . implode('\',\'', explode(',', $this->FieldAdds[$fieldname]['default'])) . '\') NULL DEFAULT NULL';
		}
		$def = $default ?: self::FIELD_TYPES[$field_type][1];
		return self::FIELD_TYPES[$field_type][0] . ' ' . ($def !== null ? ' NOT NULL DEFAULT \'' . $def . '\'' : ' NULL DEFAULT NULL');
	}

	function getFieldType(string $name, string $dbType): string{
		return (string) ($this->FieldAdds[$name]['type'] ?? array_search($dbType, array_column(self::FIELD_TYPES, '0', -1)) ?: 'input');
	}

	function getPropertyTitle(string $prop): string{
		return ($this->PropertyTitle[$prop] ?? $prop);
	}

	function initCustomerWithDefaults(we_customer_customer $customer): void{
		$meta = $this->db->metadata(CUSTOMER_TABLE, we_database_base::META_FULL);
		'@phan-var array<string,array{table:string,name:string,type:string,len:int,flags:int,default:mixed}> $meta';
		foreach($meta as $field => $vals){
			if(we_customer_customer::isProperty($field) || we_customer_customer::isProtected($field)){
				continue;
			}
			if(empty($vals['default'])){
				continue;
			}
			$customer->{$field} = $vals['default'];
		}
	}

	//field adds operations
	function storeFieldAdd(string $fieldName, string $addName, string $value): void{
		$this->FieldAdds[$fieldName][$addName] = $value;
	}

	function retriveFieldAdd(string $fieldName, string $addName, string $default = ''): string{
		return (string) ($this->FieldAdds[$fieldName][$addName] ?? $default);
	}

	function removeFieldAdd(string $fieldName): void{
		unset($this->FieldAdds[$fieldName]);
	}

	function renameFieldAdds(string $old, string $new): void{
		foreach(array_keys($this->FieldAdds) as $k){
			if($k == $old){
				$tmp = $this->FieldAdds[$k];
				unset($this->FieldAdds[$k]);
				$this->FieldAdds[$new] = $tmp;
			}
		}
	}

	function getSettings(string $settings): string{
		return (string) ($this->Prefs[$settings] ?? '');
	}

	function setSettings(string $settings, string $value): void{
		$this->Prefs[$settings] = $value;
	}

	/**
	 *
	 * @return array<string,string>
	 */
	function getAllSettings(): array{
		return $this->Prefs;
	}

}
