<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//	build table for login screen.

abstract class we_dialog_info{

	private static function getValueLoginMode(string $val): bool{
		$mode = $_COOKIE['we_mode'] ?? we_base_constants::MODE_NORMAL;
		switch($val){
			case we_base_constants::MODE_SEE:
				return ($mode === we_base_constants::MODE_SEE);
			default:
			case we_base_constants::MODE_NORMAL:// start normal mode
				return ($mode != we_base_constants::MODE_SEE);
		}
	}

	public static function getDialog(bool $isLogin = false, bool $showLoginDialog = false): string{
		$we_version = '';

		$extra = '';
		if(!$isLogin){
			$we_version .= ((defined('WE_VERSION_NAME') && WE_VERSION_NAME != '') ? WE_VERSION_NAME : WE_VERSION) . ' (' . WE_VERSION .
				((defined('WE_SVNREV') && WE_SVNREV != '0000') ? ', Revision: ' . WE_SVNREV : '') . (defined('WE_VERSION_HOTFIX_NR') && WE_VERSION_HOTFIX_NR ? ' , h' . WE_VERSION_HOTFIX_NR : '') . ')' .
				((defined('WE_VERSION_SUPP') && WE_VERSION_SUPP != '') ? ' ' . g_l('global', '[' . WE_VERSION_SUPP . ']') : '') .
				((defined('WE_VERSION_SUPP_VERSION') && WE_VERSION_SUPP_VERSION != 0) ? WE_VERSION_SUPP_VERSION : '');

			switch(strtolower(WE_VERSION_SUPP)){
				case 'rc':
					$extra = 'RC';
					break;
				case 'alpha':
					$extra = 'ALPHA';
					break;
				case 'beta':
					$extra = 'BETA';
					break;
				case 'nightly':
				case 'weekly':
				case 'nightly-build':
					$extra = 'NIGHTLY';
					break;
				case 'preview':
				case 'dp':
					$extra = 'PREVIEW';
					break;
				case 'trunk':
				case 'svn':
					$extra = 'VC';
					break;
			}
		}
		$actRow = 0;
		$table = (new we_html_table(['id' => 'mainTable', 'class' => MAINTENANCE_MODE], 8, 1))
			->setCol($actRow++, 0, '<a href="http://www.webedition.org" target="_blank"  title="www.webedition.org"><img src="' . IMAGE_DIR . 'webedition.svg"/></a>' . ($extra ? '<div id="versionSpec">' . $extra . '</div>' : ''), [
			'class' => 'logo']);

		if($we_version){
			$table->setCol($actRow++, 0, 'Version: ' . $we_version, ['class' => 'small row5']);
		}
		if(MAINTENANCE_MODE){
			$table->setCol($actRow++, 0, g_l('prefs', '[maintenance][mode]') . ': ' . MAINTENANCE_MODE, ['class' => 'important']);
		}

		$table->setCol($actRow++, 0, '<div id="credits">' .
			g_l('global', '[developed_further_by]') . ': <a href="http://www.webedition.org/" target="_blank" ><strong>webEdition e.V.</strong></a>', ['class' => 'defaultfont small row5']);

		if(is_readable(WE_CUSTOM_PATH . 'agency.inc.php')){
			$tmp = include_once(WE_CUSTOM_PATH . 'agency.inc.php');
			$table->setCol($actRow++, 0, is_string($tmp) ? $tmp : '', ['class' => 'defaultfont small row10']);
		}

		$loginRow = 0;

		if($showLoginDialog){
			$loginTable = (new we_html_table(['class' => 'plainTable credentials'], 4, 2))
				->setCol($loginRow++, 0, we_html_element::label(['for' => 'username'], g_l('global', '[username]')), ['class' => 'small userPwd_row', 'colspan' => '2'])
				->setCol($loginRow++, 0, we_html_tools::textInput('WE_LOGIN_username', '', 255, ['id' => 'username', 'autofocus' => 'autofocus',
						'placeholder' => g_l('global', '[username]')], 'text') . '<i class="fa fa-user"></i>', ['class' => 'userPwd_row', 'colspan' => '2'])
				->setCol($loginRow++, 0, we_html_element::label(['for' => 'password'], g_l('global', '[password]')), ['class' => 'small row5 userPwd_row', 'colspan' => '2'])
				->setCol($loginRow, 0, we_html_tools::textInput('WE_LOGIN_password', '', we_users_user::MAX_PWD_LEN, ['id' => 'password', 'placeholder' => g_l('global', '[password]'),
						'onkeydown' => "blindPwd('password');"], 'password') . '<i class="fa fa-lock"></i>', ['class' => 'userPwd_row'])
				->setCol($loginRow++, 1, '<button type="button" class="showPwd" onclick="showPwd(\'password\');"><i class="far fa-eye"></i></button>', ['class' => 'userPwd_row'])
				->setCol($loginRow++, 0, we_html_element::label(['for' => 'totp'], g_l('modules_users', '[TOTP_Key]')), ['class' => 'small row5 topt_row', 'style' => 'display:none;',
					'colspan' => '2'])
				->setCol($loginRow++, 0, we_html_tools::textInput('WE_LOGIN_TOPT', '', 10, ['id' => 'totp',
						'placeholder' => '123456',
						'pattern' => '[0-9]{6,}'], 'text') . '<i class="fa fa-qrcode"></i>', ['class' => 'topt_row', 'style' => 'display:none;', 'colspan' => '2'])
				->setCol($loginRow++, 0, '<a href="' . WEBEDITION_DIR . 'resetpwd.php">' . g_l('global', '[pwd][forgotten]') . '</a>', ['colspan' => '2']);

			$table->setCol($actRow++, 0, $loginTable->getHtml() . we_html_element::hiddens(['WE_LOGIN_do' => '1', 'hasJava' => '0']), ['class' => 'spaceTable']);


			//	mode-table
			$modetable = new we_html_table(['class' => 'plainTable modeTable'], 1, 3);

			$loginButton = we_html_button::create_button('fat:login,fa-lg fa-sign-in', we_html_button::WE_FORM . ':loginForm');
			if(!WE_SEEM){ //	deactivate See-Mode
				$modetable
					->setCol(0, 0, '')
					->setCol(0, 1, $loginButton, ['style' => 'text-align:right;vertical-align:bottom;', 'rowspan' => 2]);
			} else { //	normal login
				if(WE_SEEM){
//	15th Mode
					$table->setCol($actRow++, 0, g_l('SEEM', '[start_mode]'), ['class' => 'small']);
				}

				// if button is between these radio boces, they can not be reachable with <tab>
				$modetable
					->setCol(0, 0, '<table class="default">
	<tr><td>' . we_html_forms::radiobutton(we_base_constants::MODE_NORMAL, self::getValueLoginMode(we_base_constants::MODE_NORMAL), 'mode', g_l('SEEM', '[start_mode_normal]'), true, 'small') .
						'</td></tr>
	<tr><td>' . we_html_forms::radiobutton(we_base_constants::MODE_SEE, self::getValueLoginMode(we_base_constants::MODE_SEE), 'mode', '<abbr title="' . g_l('SEEM', '[start_mode_seem_acronym]') . '">' . g_l('SEEM', '[start_mode_seem]') . '</abbr>', true, 'small') .
						'</td></tr>
		</table>')
					->setCol(0, 1, $loginButton, ['style' => 'text-align:right;vertical-align:bottom', 'rowspan' => '3']);
			}

			//	16th
			$table->setCol($actRow++, 0, $modetable->getHtml(), ['class' => 'small']);
		} else if($isLogin){
			$table->addRows(2);

			//	9th Login ok
			$table
				->setCol($actRow++, 0, g_l('global', '[loginok]', false, true), ['class' => 'small spaceTable'])

				//	11th back button
				->setCol($actRow++, 0, we_html_button::create_button('back_to_login', WEBEDITION_DIR . 'index.php'), ['width' => (string) (432 - 30), 'class' => 'small',
					'style' => 'text-align:right;padding-bottom:15px;']);
		}

		return $table->getHtml() . (!$showLoginDialog ? '' :
			we_html_element::div(['id' => 'loading'], '<i class="fa fa-5x fa-spinner fa-pulse"></i>')
			);
	}

	public static function getFullDialog(): void{
		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'loginScreen.css') .
			we_html_element::cssLink(CSS_DIR . 'infoScreen.css'), we_html_element::body([
				'id' => 'infoScreen',
				'onload' => "self.focus();"
				], we_html_tools::dialogLayout(self::getDialog(), '', we_html_button::create_button(we_html_button::CLOSE, 'javascript:WE().util.dialogTop(window).doClose();'))
		));
	}

}
