<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 39e7b777091a $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2019-12-20 15:43 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_xhtmlxtras extends we_dialog_base{
	protected $isTinyDialog = true;
	protected $Plugin = '';

	protected function __construct(bool $noInternals = true){
		parent::__construct($noInternals);
		$this->onOkJsOnly = true;
		$this->noInternals = $noInternals;
		$this->initByHttp();
	}

	protected function initByHttp(): void{
		$this->Plugin = trim(we_base_request::string('we_dialog_args', '', 'plugin'));
		$this->dialogTitle = g_l('wysiwyg', '[' . $this->Plugin . ']');
	}

	protected function getJs(): string{
		return parent::getJs() .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/wexhtmlxtras/js/' . $this->Plugin . '.js') .
			we_html_element::jsScript(WE_JS_TINYMCE_DIR . 'plugins/wexhtmlxtras/js/element_common.js');
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		$fields = '';
		$parts = new we_html_multiIconBoxContent();
		if($this->Plugin !== 'cite'){
			foreach(['datetime', 'cite'] as $field){
				$fields .= $this->getInputElement($field);
			}
			$parts->add((new we_html_multiIconBoxContentLine(g_l('wysiwyg', '[xhtmlxtras][fieldsetGeneral]'), we_html_element::div(['class' => 'xhtmlxtrasContent', 'style' => 'margin: 0 0 18px 0;'], $fields)))->noline());
		}

		$fields = '';
		foreach(['title', 'id', 'class', /* 'style', 'dir', */ 'lang'] as $field){
			$fields .= $this->getInputElement($field);
		}

		$parts->add(new we_html_multiIconBoxContentLine(g_l('wysiwyg', '[xhtmlxtras][fieldsetAttribs]'), we_html_element::div(['class' => 'xhtmlxtrasContent'], $fields)));


		return $parts;
	}

	protected function getInputElement(string $name): string{
		switch($name){
			case 'datetime':
				$input = we_html_element::input(['id' => $name, 'name' => $name, 'class' => 'xhtmlxtras_date wetextinput']) .
					'<a href="javascript:insertDateTime(\'datetime\');" onmousedown="return false;" class="browse" role="button" aria-labelledby="datetimelabel"><span class="datetime" title="' . g_l('wysiwyg', '[xhtmlxtras][insertDate]') . '"><i class="fa fa-calendar" aria-hidden="true"></i></span></a>';
				break;
			case 'class':
				$input = we_html_tools::select($name, ['-1' => 'ohne'], 1, '', false, ['class' => 'defaultfont mceEditableSelect', 'id' => $name]);
				break;
			case 'dir':
				$input = we_html_tools::select($name, ['ltr' => 'LLTTRR', 'rtl' => 'RRTTLL'], 1, '', false, ['class' => 'defaultfont', 'id' => $name]);
				break;
			case 'lang':
				$input = we_html_tools::getLangField($name, '', '', 0);
				break;
			default:
				$input = we_html_element::input(['id' => $name, 'name' => $name, 'class' => 'wetextinput']);
		}

		return we_html_element::div([], we_html_element::label(['for' => $name, 'class' => 'xhtmlxtras_attribLabel'], g_l('wysiwyg', '[xhtmlxtras][' . $name . ']')) . $input);
	}

	protected function getDialogButtons(): string{
		$buttons = we_html_button::create_button(we_html_button::TRASH, "javascript:weTinyDialog_doRemove();", '', '', true, false);

		return we_html_element::div(['class' => 'htmlDialogLayoutButtons'], $buttons . parent::getDialogButtons());
	}

}
