<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_import_site extends we_import_wizard{
	var $step = 0;

	/** @var string */
	var $cmd = '';

	/** @var string */
	var $from = '/';

	/** @var int */
	var $to = 0;

	/** @var bool */
	var $images = true;

	/** @var bool */
	var $htmlPages = true;

	/** @var bool */
	private $createWePages = true;

	/** @var bool */
	var $video = true;

	/** @var bool */
	var $js = true;

	/** @var bool */
	var $css = true;

	/** @var bool */
	var $text = true;

	/** @var bool */
	var $other = true;

	/** @var int */
	var $maxSize = 12; // in Mb

	/** @var string */
	var $sameName = we_exim_ExIm::COLLISION_OVERWRITE;

	/** @var bool */
	var $isSearchable = true;

	/** @var bool */
	private $importMetadata = true;

	/** @var array<int,array{path:string,contentType:string,sourceDir:string,destDirID:int,sameName:string,thumbs:string,width:int,height:int,widthSelect:string,heightSelect:string,keepRatio:bool,quality:int,degrees:int,isSearchable:bool,importMetadata:bool}> */
	public $files = [];

	/** @var int */
	private $depth = 0;
	//private $actualDepth = 0;

	/** @var string */
	var $thumbs = '';

	/** @var int */
	var $width = 0;

	/** @var int */
	var $height = 0;

	/** @var string */
	var $widthSelect = 'pixel';

	/** @var string */
	var $heightSelect = 'pixel';

	/** @var bool */
	var $keepRatio = true;

	/** @var int */
	var $quality = 8;

	/** @var int */
	var $degrees = 0;

	/** @var array */
	private $postProcess = [];

	/** @var we_database_base */
	private static $DB = null;

	/**
	 * Constructor of Class
	 *
	 *
	 */
	public function __construct(string $what, int $step, int $mode){
		parent::__construct($what, $step, $mode);
		$ws = we_users_util::get_def_ws() ?: 0;
		$this->from = we_base_request::_(we_base_request::FILE, 'from', (!empty($_SESSION['prefs']['import_from']) ? $_SESSION['prefs']['import_from'] : $this->from));
		$_SESSION['prefs']['import_from'] = $this->from;
		$this->to = we_base_request::_(we_base_request::FILE, 'to', $ws);
		$this->depth = we_base_request::int('depth', $this->depth) === 1 ? -1 : $this->depth;
		$this->images = we_base_request::bool('images', $this->images);
		$this->htmlPages = we_base_request::bool('htmlPages', $this->htmlPages);
		$this->createWePages = we_base_request::bool('createWePages', $this->createWePages);
		$this->video = we_base_request::bool('video', $this->video);
		$this->js = we_base_request::bool('js', $this->js);
		$this->css = we_base_request::bool('css', $this->css);
		$this->text = we_base_request::bool('text', $this->text);
		$this->other = we_base_request::bool('other', $this->other);
		$this->maxSize = we_base_request::int('maxSize', $this->maxSize);
		$this->step = we_base_request::int('step', $this->step);
		$this->sameName = we_base_request::string('sameName', $this->sameName);
		$this->isSearchable = we_base_request::bool('isSearchable', $this->isSearchable);
		$this->importMetadata = we_base_request::bool('importMetadata', $this->importMetadata);
		$this->thumbs = ($thumbs = we_base_request::intList('thumbs', we_base_request::NOT_VALID_AI)) !== we_base_request::NOT_VALID_AI ? implode(',', $thumbs) : $this->thumbs;
		$this->width = we_base_request::int('width', $this->width);
		$this->height = we_base_request::int('height', $this->height);
		$this->widthSelect = we_base_request::string('widthSelect', $this->widthSelect);
		$this->heightSelect = we_base_request::string('heightSelect', $this->heightSelect);
		$this->keepRatio = we_base_request::bool('keepRatio', $this->keepRatio);
		$this->quality = we_base_request::int('quality', $this->quality);
		$this->degrees = we_base_request::int('degrees', $this->degrees);

		$this->files = [];

		switch(($cmd = we_base_request::string('we_cmd', '', 0))){
			case 'SaveWePageSettings' :
			case 'CreateWePageSettings' :
			case 'updateSiteImportTable' :
				$this->cmd = $cmd;
				break;
			default:
				$this->cmd = we_base_request::string('cmd', $this->cmd);
		}
	}

	/**
	 * returns the right HTML for siteimport depending on $this->cmd
	 *
	 * @return         string
	 */
	public function getHTML(): string{
		switch($this->cmd){
			case 'updateSiteImportTable' :
				return $this->updateSiteImportTable();
			case 'createWePageSettings' :
				return $this->getCreateWePageSettingsHTML();
			case 'saveWePageSettings' :
				return $this->getSaveWePageSettingsHTML();
			case 'content' :
				return $this->getContentHTML();
			case 'buttons' :
				return $this->getWizBusy();
			default:
				return $this->getWizFrameset();
		}
	}

	/**
	 * returns the fields of the template with given $tid (ID of template)
	 *
	 * @param	int	$tid ID of template
	 *
	 * @return	array<string,mixed>
	 */
	private static function getFieldsFromTemplate(int $tid): array{
		$templateCode = fS('SELECT c.Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" AND c.DID=' . $tid . ' AND c.nHash=x\'' . md5(we_contents_template::FULLCODE) . '\'');
		$tp = new we_weTag_parser($templateCode);
		$records = [];
		foreach($tp->getTagsWithAttributes() as $tag){
			if(empty($tag['attribs']['name'])){
				continue;
			}
			$name = $tag['attribs']['name'];
			switch($tag['name']){
				case 'var':
				case 'field':
					continue 2;
				// tags with text content, images, links and hrefs
				case 'img' :
					$records[$name] = 'img';
					break;
				case 'href' :
					$records[$name] = 'href';
					break;
				case 'link' :
					$records[$name] = 'link';
					break;
				case 'textarea' :
					$records[$name] = 'text';
					break;
				case 'input' :
					switch(!empty($tag['attribs']['type']) ? $tag['attribs']['type'] : 'text'){
						case 'text' :
						case 'choice' :
						case 'select' :
							$records[$name] = 'text';
							break;
						case 'date' :
							$records[$name] = 'date';
							break;
					}
					break;
			}
		}
		$records[we_contents_root::TITLE_FIELD] = 'text';
		$records[we_contents_root::DESCRIPTION_FIELD] = 'text';
		$records[we_contents_root::KEYWORD_FIELD] = 'text';
		return $records;
	}

	/**
	 * returns the HTML with JavasScript which updates the HTML of the site import table (its a view function called from getHTML()) via DOM (kind of AJAX)
	 *
	 * @return	string
	 */
	private function updateSiteImportTable(): string{

		$templateFields = self::getFieldsFromTemplate(we_base_request::int("tid"));
		$hasDateFields = false;

		$values = [];

		foreach($templateFields as $name => $type){
			if($type === 'date'){
				$hasDateFields = true;
			}
			switch($name){
				case we_contents_root::TITLE_FIELD:
					$values[] = ['name' => $name,
						'pre' => '<title>',
						'post' => '</title>'
					];
					break;

				case we_contents_root::KEYWORD_FIELD:
					$values[] = ['name' => $name,
						'pre' => '<meta name="keywords" content="',
						'post' => '">'
					];
					break;

				case we_contents_root::DESCRIPTION_FIELD:
					$values[] = ['name' => $name,
						'pre' => '<meta name="description" content="',
						'post' => '">'
					];
					break;

				case 'Charset':
					$values[] = ['name' => $name,
						'pre' => '<meta charset="',
						'post' => '">'
					];
					break;

				default:
					$values[] = ['name' => $name,
						'pre' => '',
						'post' => ''
					];
			}
		}

		return $this->getHtmlPage('', we_html_element::jsElement('
var tableDivObj = parent.document.getElementById("tablediv");
tableDivObj.innerHTML = "' . strtr(addslashes($this->getSiteImportTableHTML($templateFields, $values)), ["\r" => '\r', "\n" => '\n']) . '"
parent.document.getElementById("dateFormatDiv").style.display="' . ($hasDateFields ? "block" : "none") . '";'
		));
	}

	/**
	 * saves the request data in database and session and returns the HTML which closes the window
	 *
	 * @return	string
	 */
	private function getSaveWePageSettingsHTML(): string{
		$ct = we_base_request::string('createType');

		$data = ($ct === 'specify' ?
			['valueCreateType' => $ct,
			'valueTemplateId' => we_base_request::int('templateID', 0),
			'valueUseRegex' => we_base_request::bool('useRegEx'),
			'valueFieldValues' => we_serialize(we_base_request::_(we_base_request::RAW, 'fields', []), SERIALIZE_JSON),
			'valueDateFormat' => we_base_request::string('dateFormat', 'unix'),
			'valueDateFormatField' => we_base_request::_(we_base_request::RAW, 'dateformatField', ''),
			'valueTemplateName' => g_l('siteimport', '[newTemplate]'),
			'valueTemplateParentID' => 0,
			] :
			['valueCreateType' => $ct,
			'valueTemplateId' => 0,
			'valueUseRegex' => false,
			'valueFieldValues' => we_serialize([], SERIALIZE_JSON),
			'valueDateFormat' => 'unix',
			'valueDateFormatField' => '',
			'valueTemplateName' => we_base_request::string('templateName', g_l('siteimport', '[newTemplate]')),
			'valueTemplateParentID' => we_base_request::int('templateParentID', 0),
		]);
		// update session
		$_SESSION['prefs']['siteImportPrefs'] = we_serialize($data);
		// update DB
		DB_WE::$inst->query('REPLACE INTO ' . PREFS_TABLE . ' SET userID=' . we_users_user::getID() . ',`key`="siteImportPrefs",`value`="' . DB_WE::$inst->escape($_SESSION['prefs']['siteImportPrefs']) . '"');
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addCmd('close_dialog');
		return $this->getHtmlPage('', $jsCmd->getCmds());
	}

	/**
	 * returns HTML of Table with fields and start and end mark
	 *
	 * @param	array<string,string>	$fields array with fields
	 * @param	array	$values array with values like it comes from REQUEST
	 *
	 * @return	string
	 */
	private function getSiteImportTableHTML(array $fields, array $values = []): string{
		$headlines = [
			g_l('siteimport', '[fieldName]'),
			g_l('siteimport', '[startMark]'),
			g_l('siteimport', '[endMark]')
		];

		$content = [];
		foreach(array_keys($fields) as $i => $name){
			'@phan-var string $name';
			[$valpre, $valpost] = $this->getIndexOfValues($values, $name);
			$content[] = [
				htmlspecialchars($name) . we_html_element::hidden('fields[' . $i . '][name]', $name),
				'<textarea name="fields[' . $i . '][pre]" style="width:160px;height:80px" wrap="off">' . htmlspecialchars($valpre) . '</textarea>',
				'<textarea name="fields[' . $i . '][post]" style="width:160px;height:80px" wrap="off">' . htmlspecialchars($valpost) . '</textarea>',
			];
		}

		return we_html_tools::dialogBorder(420, $content, $headlines, 'middlefont', 'fields');
	}

	/**
	 * returns index of array which name is the same as $name
	 *
	 * @param	array	$values array with values
	 * @param	string	$name name to compare
	 *
	 * @return array{string,string}
	 */
	private function getIndexOfValues(array $values, string $name): array{
		foreach($values as $cur){
			if($cur['name'] === $name){
				return [$cur['pre'], $cur['post']];
			}
		}
		return ['', ''];
	}

	/**
	 * returns HTML of the "create webEdition page" settings Dialog
	 *
	 * @return	string
	 */
	private function getCreateWePageSettingsHTML(): string{
		$data = (isset($_SESSION['prefs']['siteImportPrefs'])) ? we_unserialize($_SESSION['prefs']['siteImportPrefs']) : [];

		$valueCreateType = $data['valueCreateType'] ?? 'auto';
		$valueTemplateId = $data['valueTemplateId'] ?? 0;
		$valueUseRegex = (bool) ($data['valueUseRegex'] ?? false);
		$valueFieldValues = isset($data['valueFieldValues']) ? we_unserialize($data['valueFieldValues']) : [];
		$valueDateFormat = $data['valueDateFormat'] ?? 'unix';
		$valueDateFormatField = $data['valueDateFormatField'] ?? g_l('siteimport', '[dateFormatString]');
		$valueTemplateName = $data['valueTemplateName'] ?? str_replace(' ', '', g_l('siteimport', '[newTemplate]'));
		$valueTemplateParentID = (int) ($data['valueTemplateParentID'] ?? 0);

		$templateFields = self::getFieldsFromTemplate($valueTemplateId);
		$hasDateFields = false;
		foreach($templateFields as $type){
			if($type === "date"){
				$hasDateFields = true;
				break;
			}
		}
		$date_help_button = we_html_button::create_button('fa:btn_help,fa-question', "javascript:showDateHelp();");
		$dateformatvals = ['unix' => g_l('import', '[uts]'),
			'gmt' => g_l('import', '[gts]'),
			'own' => g_l('import', '[fts]')
		];
		$dateFormatHTML = '<div id="dateFormatDiv" style="display:' . ($hasDateFields ? 'block' : 'none') . ';margin-bottom:10px;"><table style="margin:10px 0 10px 0" class="default"><tr><td style="padding-right:10px" class="defaultfont">' . htmlspecialchars(g_l('siteimport', '[dateFormat]'), ENT_QUOTES) . ':</td><td>' . we_html_tools::select('dateFormat', $dateformatvals, 1, $valueDateFormat, false, [
				'onchange' => "dateFormatChanged(this);"]) . '</td><td id="ownValueInput" style="padding-left:10px;display:' . (($valueDateFormat === "own") ? 'block' : 'none') . '">' . we_html_tools::textInput("dateformatField", $valueDateFormatField) . '</td><td id="ownValueInputHelp" style="padding-bottom:1px;padding-left:10px;display:' . (($valueDateFormat === "own") ? 'block' : 'none') . '">' . $date_help_button . '</td></tr></table></div>';

		$table = '<div style="overflow:auto;height:330px; margin-top:5px;"><div style="width:450px;" id="tablediv">' . $this->getSiteImportTableHTML($templateFields, $valueFieldValues) . '</div></div>';

		$regExCheckbox = we_html_forms::checkboxWithHidden($valueUseRegex, 'useRegEx', htmlspecialchars(g_l('siteimport', '[useRegEx]'), ENT_QUOTES));
		$specifyHTML = $this->getTemplateSelectHTML($valueTemplateId) . '<div id="specifyParam" style="padding-top:10px;display:' . ($valueTemplateId ? 'block' : 'none') . '">' . $dateFormatHTML . $regExCheckbox . $table . '</div>';

		$vals = [
			'auto' => htmlspecialchars(g_l('siteimport', '[cresteAutoTemplate]'), ENT_QUOTES),
			'specify' => htmlspecialchars(g_l('siteimport', '[useSpecifiedTemplate]'), ENT_QUOTES)
		];

		$html = '<table style="margin-bottom:10px" class="default"><tr><td style="padding-right:10px">' . htmlspecialchars(g_l('siteimport', '[importKind]'), ENT_QUOTES) . ':</td><td>' . we_html_tools::select('createType', $vals, 1, $valueCreateType, false, [
				'onchange' => "createTypeChanged(this);"]) . '</td></tr></table><div id="ctauto" style="display:' . (($valueCreateType === "auto") ? 'block' : 'none') . '">' . we_html_tools::alertAttentionBox(g_l('siteimport', '[autoExpl]'), we_html_tools::TYPE_INFO, 450) . self::formPathHTML($valueTemplateName, $valueTemplateParentID) . '</div><div id="ctspecify" style="display:' . (($valueCreateType === "specify") ? 'block' : 'none') . '"><div style="height:4px;"></div>' . $specifyHTML . '</div>';

		$html = '<div style="height:480px">' . $html . '</div>';

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $html));
		$buttons = we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:if(checkForm()){document.we_form.submit();}"), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()"));

		$bodyhtml = '<body class="weDialogBody">
					<iframe style="position:absolute;top:-2000px;width:400px;height:200px;" src="about:blank" id="iloadframe" name="iloadframe"></iframe>
					<form onsubmit="return false;" name="' . we_base_constants::WE_FORM . '" method="post" action="' . WEBEDITION_DIR . 'we_cmd.php" target="iloadframe">' .
			we_html_element::hiddens([
				'we_cmd[0]' => 'SaveWePageSettings',
				'ok' => '1']) .
				(new we_html_multiIconBox($parts))
				->marginLeft()
				->buttons($buttons)
				->headline(g_l('siteimport', '[importSettingsWePages]'))
				->get() .
			'</form></body>';

		return $this->getHtmlPage($bodyhtml, we_html_element::jsScript(JS_DIR . 'import_site.js'));
	}

	/**
	 * returns HTML of the template selector
	 *
	 * @param int $tid  ID of template
	 *
	 * @return	string
	 */
	private function getTemplateSelectHTML(int $tid): string{
		$path = fS('SELECT Path FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . $tid);

		$button = we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements.templateID.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'templateID',targetName:'templateDummy',jsCmd:'displayTable',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})");

		$foo = we_html_tools::textInput('templateDummy', $path, 0, ['readonly' => 'readonly'], "text", '320px');
		return we_html_tools::formElementTable(htmlspecialchars(g_l('siteimport', '[template]'), ENT_QUOTES), $foo, we_html_element::hidden('templateID', $tid), $button);
	}

	/**
	 * returns HTML of the main dialog (contemt)
	 *
	 * @return	string
	 */
	private function getContentHTML(): string{
		// Suorce Directory

		$from_button = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ?
			we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'from',WE().consts.contentTypes.FOLDER_DOCUMENT,document.we_form.elements.from.value)") :
			'';

		$input = we_html_tools::textInput('from', $this->from, 0, ['readonly' => 'readonly'], 'text', '300px');
		$importFrom = we_html_tools::formElementTable(g_l('siteimport', '[importFrom]'), $input, $from_button);

		// Destination Directory
		//$hidden = we_html_element::hidden("to",$this->to);
		//$input = we_html_tools::textInput("toPath",id2path($this->to),'','readonly="readonly"',"text",300);
		//$importTo = we_html_tools::formElementTable($input, g_l('siteimport',"[importTo]"), "defaultfont", $to_button, $hidden, '', '', 0);


		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('DirPath')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('toPath', id2path($this->to))
			->setLabel(g_l('siteimport', '[importTo]'))
			->setMaxResults(10)
			->setRequired(true)
			->setResult('to', (string) $this->to)
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.to.value,table:WE().consts.tables.FILE_TABLE,targetID:'to',targetName:'toPath',rootDirID:0})"));

		$importTo = $weSuggest->getHTML();

		// Checkboxes
		$weoncklick = "if(this.checked && (!this.form.elements.htmlPages.checked)){this.form.elements.htmlPages.checked = true;}";
		$weoncklick .= ((!we_base_permission::hasPerm([we_base_perm::WORK_HTML_NEW])) && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW])) ? "if((!this.checked) && this.form.elements.htmlPages.checked){this.form.elements.htmlPages.checked = false;}" : '';

		$images = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW]) ? $this->images : false, "images", g_l('siteimport', '[importImages]'), false, "", '', !we_base_permission::hasPerm([
					we_base_perm::WORK_IMAGE_NEW]));

		$htmlPages = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_HTML_NEW]) ? $this->htmlPages : ((we_base_permission::hasPerm([
					we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]) && $this->createWePages)), "htmlPages", g_l('siteimport', '[importHtmlPages]'), false, "", "if(this.checked){this.form.elements.check_createWePages.disabled=false;document.getElementById('label__createWePages').style.color='black';}else{this.form.elements.check_createWePages.disabled=true;document.getElementById('label__createWePages').style.color='grey';}", !we_base_permission::hasPerm([
					we_base_perm::WORK_HTML_NEW]));
		$createWePages = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]) ? $this->createWePages : false, "createWePages", g_l('siteimport', '[createWePages]') . "&nbsp;&nbsp;", false, "", $weoncklick, !we_base_permission::hasPerm([
					we_base_perm::WORK_DOCUMENT_WEBEDITION_NEW]));
		$video = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_VIDEO_NEW]) ? $this->video : false, 'video', g_l('siteimport', '[importFlashmovies]'), false, '', '', !we_base_permission::hasPerm([
					we_base_perm::WORK_VIDEO_NEW]));
		$jss = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_JS_NEW]) ? $this->js : false, 'js', g_l('siteimport', '[importJS]'), false, "", '', !we_base_permission::hasPerm([
					we_base_perm::WORK_JS_NEW]));
		$css = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_CSS_NEW]) ? $this->css : false, 'css', g_l('siteimport', '[importCSS]'), false, "", '', !we_base_permission::hasPerm([
					we_base_perm::WORK_CSS_NEW]));
		$text = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_TEXT_NEW]) ? $this->text : false, 'text', g_l('siteimport', '[importText]'), false, "", '', !we_base_permission::hasPerm([
					we_base_perm::WORK_TEXT_NEW]));
		//$htaccess = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_HTACCESS_NEW]) ? $this->text : false, 'htaccess', g_l('siteimport', '[importHTACCESS]'), false, '', '', !we_base_permission::hasPerm([we_base_perm::WORK_HTACCESS_NEW]));
		$others = we_html_forms::checkboxWithHidden(we_base_permission::hasPerm([we_base_perm::WORK_OTHER_NEW]) ? $this->other : false, "other", g_l('siteimport', '[importOther]'), false, "", '', !we_base_permission::hasPerm([
					we_base_perm::WORK_OTHER_NEW]));

		$wePagesOptionButton = we_html_button::create_button('preferences', "javascript:we_cmd('CreateWePageSettings')", '', '', false, true, '', true);
		// Depth

		$depth = we_html_tools::formElementTable('', we_html_forms::checkboxWithHidden($this->depth === 1, 'depth', g_l('siteimport', '[import_recursively]'), false));
		$maxallowed = round(DB_WE::$inst->getMaxAllowedPacket() / (1024 * 1024)) ?: 20;
		$maxarray = [0 => g_l('siteimport', '[nolimit]'), '0.5' => "0.5"];
		for($i = 1; $i <= $maxallowed; $i++){
			$maxarray[$i] = $i;
		}

		// maxSize
		$select = we_html_tools::select("maxSize", $maxarray, 1, $this->maxSize, false, [], true, 150);
		$maxSize = we_html_tools::formElementTable(g_l('siteimport', '[maxSize]'), $select);

		$thumbsarray = DB_WE::$inst->getAllFirstq('SELECT ID,Text FROM ' . THUMBNAILS_TABLE . ' ORDER BY Text', false);
		$select = we_html_tools::select('thumbs[]', $thumbsarray, 1, $this->thumbs, true, ['class' => 'searchSelect',], true, 150);
		$thumbs = we_html_tools::formElementTable(g_l('importFiles', '[thumbnails]'), $select);

		/* Create Main Table */
		$tableObj = (new we_html_table(['class' => 'default'], 5, 3))
			->setCol(0, 0, $images, ['colspan' => '2'])
			->setCol(0, 2, $jss)
			->setCol(1, 0, $video, ['colspan' => '2'])
			->setCol(1, 2, $css)
			->setCol(2, 0, $htmlPages, ['colspan' => '2'])
			->setCol(2, 2, $text)
			->setColAttributes(3, 0, ['style' => 'width:20px;'])
			->setCol(3, 1, $createWePages, ['style' => 'width:200px;'])
			->setCol(3, 2, $others, ['style' => 'width:180px;'])
			->setCol(4, 1, $wePagesOptionButton);
		$table2Obj = (new we_html_table(['class' => 'default'], 1, 2))
			->setCol(0, 0, $depth, ['style' => 'width:220px;'])
			->setCol(0, 1, $maxSize, ['style' => 'width:180px;']);


		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('siteimport', '[dirs_headline]'), $importFrom . $importTo))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('siteimport', '[import]'), $tableObj->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('siteimport', '[limits]'), $table2Obj->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('importFiles', '[sameName_headline]'), we_html_tools::alertAttentionBox(g_l('importFiles', '[sameName_expl]'), we_html_tools::TYPE_INFO, 410) .
					we_html_element::div(['style' => 'margin-top:10px;'], we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_OVERWRITE, $this->sameName, 'sameName', g_l('importFiles', '[sameName_overwrite]')) .
						we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $this->sameName, 'sameName', g_l('importFiles', '[sameName_rename]')) .
						we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $this->sameName, 'sameName', g_l('importFiles', '[sameName_nothing]'))
					)))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('importFiles', '[imgsSearchable]'), we_html_forms::checkboxWithHidden($this->isSearchable, 'isSearchable', g_l('importFiles', '	[searchable_label]'))))
			->space(we_html_multiIconBox::SPACE_MED)
		);

		if(we_base_permission::hasPerm([we_base_perm::WORK_IMAGE_NEW])){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('importFiles', '[metadata]'), we_html_forms::checkboxWithHidden($this->importMetadata, 'importMetadata', g_l('importFiles', '[import_metadata]'))))
					->space(we_html_multiIconBox::SPACE_MED)
			);

			if(we_base_imageEdit::gd_version() > 0){
				$parts->add((new we_html_multiIconBoxContentLine(g_l('importFiles', '[make_thumbs]', false, true), $thumbs))
						->space(we_html_multiIconBox::SPACE_MED)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[resize]'), '<table class=" defaultfont">
				<tr>
					<td>' . g_l('weClass', '[width]') . ':</td>
					<td>' . we_html_tools::textInput('width', (string) $this->width, 0, [], 'text', '60px') . '</td>
					<td><select class="weSelect" name="widthSelect"><option value="pixel"' . (($this->widthSelect === 'pixel') ? ' selected="selected"' : '') . '>' . g_l('weClass', '[pixel]') . '</option><option value="percent"' . (($this->widthSelect === 'percent') ? ' selected="selected"' : '') . '>' . g_l('weClass', '[percent]') . '</option></select></td>
				</tr>
				<tr>
					<td>' . g_l('weClass', '[height]') . ':</td>
					<td>' . we_html_tools::textInput('height', (string) $this->height, 0, [], 'text', '60px') . '</td>
					<td><select class="weSelect" name="heightSelect"><option value="pixel"' . (($this->heightSelect === 'pixel') ? ' selected="selected"' : '') . '>' . g_l('weClass', '[pixel]') . '</option><option value="percent"' . (($this->heightSelect === 'percent') ? ' selected="selected"' : '') . '>' . g_l('weClass', '[percent]') . '</option></select></td>
				</tr>
				<tr>
					<td colspan="3">' . we_html_forms::checkbox('1', $this->keepRatio, 'keepRatio', g_l('thumbnails', '[ratio]')) . '</td>
				</tr>
			</table>'))
						->space(we_html_multiIconBox::SPACE_MED)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[rotate]'), we_html_forms::radiobuttonV('0', (string) $this->degrees, 'degrees', g_l('weClass', '[rotate0]')) .
							we_html_forms::radiobuttonV('180', (string) $this->degrees, 'degrees', g_l('weClass', '[rotate180]')) .
							we_html_forms::radiobuttonV('90', (string) $this->degrees, 'degrees', g_l('weClass', '[rotate90l]')) .
							we_html_forms::radiobuttonV('270', (string) $this->degrees, 'degrees', g_l('weClass', '[rotate90r]'))
						))->space(we_html_multiIconBox::SPACE_MED)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('weClass', '[quality]'), we_base_imageEdit::qualitySelect('quality', $this->quality)))
						->space(we_html_multiIconBox::SPACE_MED)
				);
			} else {
				$parts->add(new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('importFiles', '[add_description_nogdlib]'), we_html_tools::TYPE_INFO, '')));
			}
			$foldAT = 5;
		} else {
			$foldAT = -1;
		}

		$body = we_html_element::body(['class' => 'weDialogBody', 'onunload' => "doUnload();"
				], we_html_element::form([
					'action' => WEBEDITION_DIR . 'we_cmd.php',
					'name' => we_base_constants::WE_FORM,
					'method' => 'post',
					'target' => 'wizcmd'
					], (new we_html_multiIconBox($parts, 'wesiteimport'))
						->marginLeft()
						->fold($foldAT, false, g_l('importFiles', '[image_options_open]'), g_l('importFiles', '[image_options_close]')
						)
						->headline(g_l('siteimport', '[siteimport]'))
						->get() .
					$this->getHiddensHTML()));

		return $this->getHtmlPage($body, we_html_element::jsScript(JS_DIR . 'import_site.js'));
	}

	/**
	 * returns HTML of the buttons frame
	 *
	 * @return	string
	 */
	protected function getWizBusy(): string{
		if($this->step == 1){
			if(!we_base_request::bool('doFragments')){//true if data was already set
				$this->fillFiles();
				if(empty($this->files)){
					$jscmd = new we_base_jsCmd();

					$importDirectory = rtrim(rtrim(WE_DOCUMENT_ROOT, '/') . $this->from, '/');
					$dir = scandir($importDirectory);
					$jscmd->addMsg((!$dir || count($dir) <= 2 ?
							g_l('importFiles', '[emptyDir]') :
							g_l('importFiles', '[noFiles]'))
						, we_base_util::WE_MESSAGE_INFO);

					$jscmd->addCmd('close');
					return $jscmd->getCmds();
				}
			} else {
				$this->files = [];
			}
			$frag = new we_import_siteFrag('siteImport', 10, ['style' => 'margin:10px 15px;'], $this->files);
			$frag->run();
			return '';
		}

		$pb = new we_gui_progressBar(0, 200);
		$pb->addText("&nbsp;", we_gui_progressBar::TOP, "progressTxt");

		$table = (new we_html_table(['class' => 'default', 'width' => '100%'], 1, 2))
			->setCol(0, 0, $pb->getHTML('', 'display:none;'))
			->setCol(0, 1, we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::BACK, "javascript:back();", '', '', false, false) .
				we_html_button::create_button(we_html_button::NEXT, "javascript:next();", '', '', false, false), '', we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose()", '', '', false, false)), [
			'style' => 'text-align:right']);

		return $this->getHtmlPage(we_html_element::body(['class' => "editfooter", 'style' => 'overflow:hidden;'], $table->getHtml()), we_html_element::jsScript(JS_DIR . 'import_site.js') . we_gui_progressBar::getJSCode());
	}

	/**
	 * used by importFile() internal Function, dont call directly!
	 *
	 * @param $content string
	 * @param &$we_doc we_document_webEdition
	 * @param $sourcePath string
	 * @static
	 */
	private static function importWebEditionPage(string $content, we_document_webEdition $we_doc, string $sourcePath): void{
		$data = (isset($_SESSION['prefs']['siteImportPrefs'])) ? we_unserialize($_SESSION['prefs']['siteImportPrefs']) : [];

		$valueCreateType = $data['valueCreateType'] ?? 'auto';
		$valueTemplateId = $data['valueTemplateId'] ?? 0;
		$valueUseRegex = $data['valueUseRegex'] ?? false;
		$valueFieldValues = isset($data['valueFieldValues']) ? we_unserialize($data['valueFieldValues']) : [];
		$valueDateFormat = $data['valueDateFormat'] ?? 'unix';
		$valueDateFormatField = $data['valueDateFormatField'] ?? g_l('siteimport', '[dateFormatString]');
		$valueTemplateName = $data['valueTemplateName'] ?? g_l('siteimport', '[newTemplate]');
		$valueTemplateParentID = $data['valueTemplateParentID'] ?? 0;

		$content = self::makeAbsolutPathOfContent($content, $sourcePath, $we_doc->ParentPath);

		if($valueCreateType === 'auto'){
			self::importAuto($content, $we_doc, $valueTemplateName, $valueTemplateParentID);
		} else {
			self::importSpecify($content, $we_doc, $valueTemplateId, $valueUseRegex, $valueFieldValues, $valueDateFormat, $valueDateFormatField);
		}
	}

	/**
	 * Makes a relative path from an absolute path
	 *
	 * @param	string	$docpath Absolute Path of document
	 * @param	string	$linkpath Absolute Path of link (href or src)
	 *
	 * @return         string
	 */
	public static function makeRelativePath(string $docpath, string $linkpath): string{
		$parentPath = $docpath;
		$newLinkPath = '';

		while($parentPath != substr($linkpath, 0, strlen($parentPath))){
			$parentPath = dirname($parentPath);
			$newLinkPath .= '../';
		}
		$rest = ltrim((string) substr($linkpath, strlen($parentPath)), '/');
		return $newLinkPath . $rest;
	}

	/**
	 * converts a relative path to an absolute path and returns it
	 *
	 * @param $path string path to convert
	 * @param $sourcePath string path of source file
	 * @param $parentPath string parent path
	 * @return string
	 * @static
	 */
	private static function makeAbsolutePath(string $path, string $sourcePath, string $parentPath): string{
		if(!preg_match('|^[a-z]+://|i', $path)){
			if(str_starts_with($path, '/')){
				// if href is an absolute URL convert it into a relative URL
				$path = self::makeRelativePath($sourcePath, $path);
			} elseif(str_starts_with($path, './')){
				// if href is a relative URL starting with "./" remove the "./"
				$path = (string) substr($path, 2);
			}
			// Make absolute Path out of it
			while(str_starts_with($path, '../') && $parentPath && $parentPath != '/'){
				$parentPath = dirname($parentPath);
				$path = substr($path, 3);
			}
			return rtrim($parentPath, '/') . '/' . $path;
		}
		return $path;
	}

	/**
	 * returns HTML for path information (in webEdition page settings dialog)
	 *
	 * @param $templateName string name of template
	 * @param $myid int id of template dir
	 * @return string
	 * @static
	 */
	private static function formPathHTML(string $templateName, int $myid): string{
		$path = id2path($myid, TEMPLATES_TABLE);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('TplPath')
			->setContentType([we_base_ContentTypes::FOLDER_TEMPLATE])
			->setInput('templateDirName', $path)
			->setResult('templateParentID', '0')
			->setLabel(g_l('weClass', '[dir]'))
			->setMaxResults(20)
			->setWidth(320)
			->setTable(TEMPLATES_TABLE)
			->setSelector(we_gui_suggest::DirSelector)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory',{id:document.we_form.elements.templateParentID.value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'templateParentID',targetName:'templateDirName'})"));
		$dirChooser = $weSuggest->getHTML();

		/*

		  $dirChooser = we_html_tools::formElementTable(we_html_tools::textInput($textname,$path,'',' readonly',"text",320,0),
		  g_l('weClass',"[dir]"),
		  "left",
		  "defaultfont",
		  we_html_element::hidden($idname,0),
		  $button);
		 */

		return '
<table class="default" style="margin-top:10px;">
	<tr>
		<td style="width:20px;">' . we_html_tools::formElementTable(g_l('siteimport', '[nameOfTemplate]'), we_html_tools::textInput("templateName", $templateName, 255, [
					], "text", '320px')) . '</td>
		<td style="width:20px;"></td>
		<td style="width:100px;">' . we_html_tools::formElementTable(g_l('weClass', '[extension]'), '<span class="defaultfont"><b>.tmpl</b></span>') . '</td>
	</tr>
	<tr>
		<td colspan="3">' . $dirChooser . '</td>
	</tr>
</table>';
	}

	/**
	 * converts all relative paths of a document to absolute paths and returns the converted document
	 *
	 * @param $content string document to convert
	 * @param $sourcePath string path of source file
	 * @param $parentPath string parent path
	 * @return string
	 * @static
	 */
	private static function makeAbsolutPathOfContent(string $content, string $sourcePath, string $parentPath): string{
		$sourcePath = substr($sourcePath, strlen(WE_DOCUMENT_ROOT)) ?: '';
		$sourcePath = we_base_file::addPreSlash($sourcePath);
		$regs = $regs2 = [];
		// replace hrefs
		preg_match_all('/(<[^>]+href=["\']?)([^"\' >]+)([^"\'>]?[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $orig_href){
				$new_href = self::makeAbsolutePath($orig_href, $sourcePath, $parentPath);
				if($new_href != $orig_href){
					$newTag = $regs[1][$i] . $new_href . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}
		// replace src (same as href!!)
		preg_match_all('/(<[^>]+src=["\']?)([^"\' >]+)([^"\'>]?[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $orig_href){
				$new_href = self::makeAbsolutePath($orig_href, $sourcePath, $parentPath);
				if($new_href != $orig_href){
					$newTag = $regs[1][$i] . $new_href . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}
		// url() in styles with style=''
		preg_match_all('/(<[^>]+style=")([^"]+)("[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				preg_match_all('/(url\(\'?)([^\'\)]+)(\'?\))/i', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeAbsolutePath($orig_url, $sourcePath, $parentPath);
						if($orig_url != $new_url){
							$newStyle = str_replace($orig_url, $new_url, $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}

		// url() in styles with style=''
		preg_match_all('/(<[^>]+style=\')([^\']+)(\'[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				preg_match_all('/(url\("?)([^"\)]+)("?\))/i', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeAbsolutePath($orig_url, $sourcePath, $parentPath);
						if($orig_url != $new_url){
							$newStyle = str_replace($orig_url, $new_url, $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}

		// url() in style tags
		preg_match_all('/(<style[^>]*>)(.*)(<\/style>)/isU', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				// url() in styles with style=''
				preg_match_all('/(url\([\'"]?)([^\'"\)]+)([\'"]?\))/iU', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeAbsolutePath($orig_url, $sourcePath, $parentPath);
						if($orig_url != $new_url){
							$newStyle = str_replace($regs2[0][$z], $regs2[1][$z] . $new_url . $regs2[3][$z], $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}
		return $content;
	}

	/**
	 * internal function, used by _importWebEditionPage() => don't call directly!
	 * @param $content string
	 * @param $we_doc we_document_webEdition
	 * @param $templateFilename string
	 * @param $templateParentID int
	 * @static
	 */
	private static function importAuto(string $content, we_document_webEdition $we_doc, string $templateFilename, int $templateParentID): void{

		$textareaCode = '<we:textarea name="content" wysiwyg="true" width="800" height="600" xml="true" inlineedit="true"/>';
		$titleCode = "<we:title />";
		$descriptionCode = "<we:description />";
		$keywordsCode = "<we:keywords />";

		$title = $description = $keywords = $charset = '';
		$attr = $regs = [];

		// check if we have a body start and end tag
		if(preg_match('/<body[^>]*>(.*)<\/body>/is', $content, $regs)){
			$bodyhtml = $regs[1];
			$templateCode = preg_replace('/(.*<body[^>]*>).*(<\/body>.*)/is', '${1}' . $textareaCode . '${2}', $content);
		} else {
			$bodyhtml = $content;
			$templateCode = $textareaCode;
		}

		// try to get title, description, keywords and charset
		if(preg_match('/<title[^>]*>(.*)<\/title>/is', $content, $regs)){
			$title = $regs[1];
			$templateCode = preg_replace('/<title[^>]*>.*<\/title>/is', $titleCode, $templateCode);
		}
		if(preg_match('/<meta ([^>]*)name="description"([^>]*)>/is', $content, $regs)){
			if(preg_match('/content="([^"]+)"/is', $regs[1], $attr)){
				$description = $attr[1];
			} elseif(preg_match('/content="([^"]+)"/is', $regs[2], $attr)){
				$description = $attr[1];
			}
			$templateCode = preg_replace('/<meta [^>]*name="description"[^>]*>/is', $descriptionCode, $templateCode);
		}
		if(preg_match('/<meta ([^>]*)name="keywords"([^>]*)>/is', $content, $regs)){
			if(preg_match('/content="([^"]+)"/is', $regs[1], $attr)){
				$keywords = $attr[1];
			} elseif(preg_match('/content="([^"]+)"/is', $regs[2], $attr)){
				$keywords = $attr[1];
			}
			$templateCode = preg_replace('/<meta [^>]*name="keywords"[^>]*>/is', $keywordsCode, $templateCode);
		}
		if(preg_match('/<meta ([^>]*)http-equiv="content-type"([^>]*)>/is', $content, $regs)){
			if(preg_match('/content="([^"]+)"/is', $regs[1], $attr)){
				$cs = [];
				if(preg_match('/charset=([^ "\']+)/is', $attr[1], $cs)){
					$charset = $cs[1];
				}
			} elseif(preg_match('/content="([^"]+)"/is', $regs[2], $attr)){
				if(preg_match('/charset=([^ "\']+)/is', $attr[1], $cs)){
					$charset = $cs[1];
				}
			} elseif(preg_match('_<meta [^>]*charset="([^"]*)"[^/]*>_is', $content, $regs)){
				$charset = $regs[1];
			}
			$templateCode = preg_replace('/<meta [^>]*(http-equiv="content-type"|charset=)[^>]*>/is', '<we:charset defined="' . $charset . '">' . $charset . '</we:charset>', $templateCode);
		}

		// replace external css (link rel=stylesheet)
		preg_match_all('/<link ([^>]+)>/i', $templateCode, $regs, PREG_PATTERN_ORDER);
		if($regs){
			$regs2 = [];
			foreach($regs[1] as $i => $tmp){
				preg_match_all('/([^= ]+)=[\'"]?([^\'" ]+)[\'"]?/is', $tmp, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					$attribs = array_combine($regs2[1], $regs2[2]);
					if(!empty($attribs['rel']) && $attribs['rel'] === 'stylesheet'){
						if(!empty($attribs['href'])){
							$id = path_to_id($attribs['href'], FILE_TABLE);
							$tag = '<we:css id="' . $id . '" xml="true" ' . ((!empty($attribs['media'])) ? ' pass_media="' . $attribs['media'] . '"' : '') . '/>';
							$templateCode = str_replace($regs[0][$i], $tag, $templateCode);
						}
					}
				}
			}
		}

		// replace external js scripts
		preg_match_all('_<script ([^>]+)>.*</script>_isU', $templateCode, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[1] as $i => $tmp){
				preg_match('/src=["\']?([^"\']+)["\']?/is', $tmp, $regs2);
				if($regs2){
					$id = path_to_id($regs2[1]);
					$tag = '<we:js id="' . $id . '" xml="true" />';
					$templateCode = str_replace($regs[0][$i], $tag, $templateCode);
				}
			}
		}

		// check if there is allready a template with the same content


		$newTemplateID = fI('SELECT c.DID FROM ' . CONTENT_TABLE . ' c WHERE c.Dat="' . DB_WE::$inst->escape($templateCode) . '" AND c.DocumentTable="' . stripTblPrefix(TEMPLATES_TABLE) . '" LIMIT 1');

		if(!$newTemplateID){
			// create Template


			$newTemplateFilename = $templateFilename;
			$result = DB_WE::$inst->getAllq('SELECT Filename FROM ' . TEMPLATES_TABLE . ' WHERE ParentID=' . abs($templateParentID) . ' AND Filename LIKE "' . DB_WE::$inst->escape($templateFilename) . '%"', true);
			$z = 1;
			while(in_array($newTemplateFilename, $result, true)){
				$newTemplateFilename = $templateFilename . $z;
				$z++;
			}
			$templateObject = new we_contents_template();
			$templateObject->we_new();
			$templateObject->Creation_Date = new DateTime();
			$templateObject->ID = 0;
			$templateObject->OldPath = '';
			$templateObject->Extension = '.tmpl';
			$templateObject->Filename = $newTemplateFilename;
			$templateObject->setParentID($templateParentID);
			$templateObject->OldPath = $templateObject->Path;
			$templateObject->setElementS('data', $templateCode, "txt");
			$templateObject->we_save();
			$templateObject->we_publish();
			$newTemplateID = $templateObject->ID;
		}

		$we_doc->setTemplateID($newTemplateID);
		$we_doc->setElementS('content', $bodyhtml);
		$we_doc->setElementS(we_contents_root::TITLE_FIELD, $title);
		$we_doc->setElementS(we_contents_root::KEYWORD_FIELD, $keywords);
		$we_doc->setElementS(we_contents_root::DESCRIPTION_FIELD, $description);
	}

	/**
	 * internal function, used by _importWebEditionPage() => don't call directly!
	 * @param $content string
	 * @param $we_doc we_document_webEdition
	 * @param $templateId int
	 * @param $useRegex boolean
	 * @param $fieldValues array
	 * @param $dateFormat string
	 * @param $dateFormatValue string
	 * @static
	 */
	private static function importSpecify(string $content, we_document_webEdition $we_doc, int $templateId, bool $useRegex, array $fieldValues, string $dateFormat, string $dateFormatValue): void{

		// TODO width & height of image
		// get field infos of template
		$templateFields = self::getFieldsFromTemplate($templateId);

		foreach($fieldValues as $field){
			if(!empty($field['pre']) && !empty($field['post']) && !empty($field['name'])){
				$fieldval = '';
				$field['pre'] = str_replace(["\r\n", "\r"], "\n", $field['pre']);
				$field['post'] = str_replace(["\r\n", "\n"], "\n", $field['post']);
				if(!$useRegex){
					$prepos = strpos($content, $field['pre']);
					$postpos = strpos($content, $field['post'], (int) $prepos);
					if($prepos !== false && $postpos !== false && $prepos < $postpos){
						$prepos += strlen($field['pre']);
						$fieldval = substr($content, $prepos, $postpos - $prepos) ?: '';
					}
				} else {
					$regs = [];
					if(preg_match('/' . preg_quote($field['pre'], '/') . '(.+)' . preg_quote($field['post'], '/') . '/isU', $content, $regs)){
						$fieldval = $regs[1];
					}
				}
				// only set field if field exists in template
				if(isset($templateFields[$field['name']])){

					if($templateFields[$field['name']] === "date"){ // import date fields
						switch($dateFormat){
							case 'unix':
								$fieldval = intval($fieldval);
								break;

							case 'gmt':
								$fieldval = we_import_functions::date2Timestamp(trim($fieldval), '');
								break;

							case 'own':
								$fieldval = we_import_functions::date2Timestamp(trim($fieldval), $dateFormatValue);
								break;
						}
						$we_doc->setElementI($field['name'], (int) $fieldval, 'date');
					} elseif($templateFields[$field['name']] === 'img'){ // import image fields
						if(preg_match('/<[^>]+src=["\']?([^"\' >]+)[^"\'>]?[^>]*>/i', $fieldval, $regs)){ // only if image tag has a src attribute
							$src = $regs[1];
							$we_doc->setElementI($field['name'], path_to_id($src), 'img');
						}
					} else {
						$we_doc->setElementS($field['name'], trim($fieldval));
					}
				}
			}
		}
		$we_doc->setTemplateID($templateId);
	}

	/**
	 * get an array containig id and contentType from path
	 * @param $path string
	 * @return array{int,string}
	 * @static
	 */
	private static function path_to_id_ct(string $path, string $table): array{
		if($path === '/'){
			return [0, ''];
		}
		return getHash('SELECT ID,ContentType FROM ' . DB_WE::$inst->escape($table) . ' WHERE Path="' . DB_WE::$inst->escape($path) . '"') ?: [0, ''];
	}

	/**
	 * converts an external  link (src or href) into an internal
	 * @param $href string
	 * @return string
	 * @static
	 */
	private static function makeInternalLink(string $href): string{
		[$id, $ct] = self::path_to_id_ct($href, FILE_TABLE);
		if(str_starts_with($ct, 'text/')){
			$href = we_base_link::TYPE_INT_PREFIX . $id;
		} elseif($ct == we_base_ContentTypes::IMAGE){
			if(!str_contains($href, '?')){
				$href .= '?id=' . $id;
			}
		}
		return $href;
	}

	/**
	 * converts all external links in a HTML page to internal links
	 * @param $content string
	 * @return string
	 * @static
	 */
	private static function external_to_internal(string $content): string{
		// replace hrefs
		$regs = $regs2 = [];
		preg_match_all('/(<[^>]+href=["\']?)([^"\' >]+)([^"\'>]?[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $orig_href){
				$new_href = self::makeInternalLink($orig_href);
				if($new_href != $orig_href){
					$newTag = $regs[1][$i] . $new_href . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}
		// replace src (same as href!!)
		preg_match_all('/(<[^>]+src=["\']?)([^"\' >]+)([^"\'>]?[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $orig_href){
				$new_href = self::makeInternalLink($orig_href);
				if($new_href != $orig_href){
					$newTag = $regs[1][$i] . $new_href . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}
		// url() in styles with style=''
		preg_match_all('/(<[^>]+style=")([^"]+)("[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				preg_match_all('/(url\(\'?)([^\'\)]+)(\'?\))/i', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeInternalLink($orig_url);
						if($orig_url != $new_url){
							$newStyle = str_replace($orig_url, $new_url, $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}

		// url() in styles with style=''
		preg_match_all('/(<[^>]+style=\')([^\']+)(\'[^>]*>)/i', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				preg_match_all('/(url\("?)([^"\)]+)("?\))/i', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeInternalLink($orig_url);
						if($orig_url != $new_url){
							$newStyle = str_replace($orig_url, $new_url, $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}

		// url() in style tags
		preg_match_all('/(<style[^>]*>)(.*)(<\/style>)/isU', $content, $regs, PREG_PATTERN_ORDER);
		if($regs){
			foreach($regs[2] as $i => $style){
				$newStyle = $style;
				// url() in styles with style=''
				preg_match_all('/(url\([\'"]?)([^\'"\)]+)([\'"]?\))/iU', $style, $regs2, PREG_PATTERN_ORDER);
				if($regs2){
					foreach($regs2[2] as $z => $orig_url){
						$new_url = self::makeInternalLink($orig_url);
						if($orig_url != $new_url){
							$newStyle = str_replace($regs2[0][$z], $regs2[1][$z] . $new_url . $regs2[3][$z], $newStyle);
						}
					}
				}
				if($newStyle != $style){
					$newTag = $regs[1][$i] . $newStyle . $regs[3][$i];
					$content = str_replace($regs[0][$i], $newTag, $content);
				}
			}
		}

		return $content;
	}

	/**
	 * this routine is called after normal import for each webEdition file. it is e.g. responsible for converting relative links to absolute links
	 *
	 * @param $path string
	 * @param $sourcePath string
	 * @param $destinationDirID int
	 * @return array
	 * @static
	 */
	public static function postprocessFile(string $path, string $sourcePath, int $destinationDirID): array{
		self::$DB = self::$DB ?: new DB_WE();

		// preparing Paths
		$path = str_replace('\\', '/', $path); // change windoof backslashes to slashes
		$sourcePath = str_replace('\\', '/', $sourcePath); // change windoof backslashes to slashes
		$sizeofdocroot = strlen(rtrim(WE_DOCUMENT_ROOT, '/')); // make sure that no ending slash is there
		$sizeofsourcePath = strlen(rtrim($sourcePath, '/')); // make sure that no ending slash is there
		$destinationDir = id2path($destinationDirID);
		if($destinationDir === '/'){
			$destinationDir = '';
		}
		$destinationPath = $destinationDir . substr($path, $sizeofdocroot + $sizeofsourcePath);
		$id = path_to_id($destinationPath, FILE_TABLE);
		$we_doc = new we_document_webEdition($id);

		// we need to get the name of the fields which needs to processed
		foreach($we_doc->elements as $fieldname => $element){
			switch($fieldname){
				case we_contents_root::TITLE_FIELD:
				case we_contents_root::DESCRIPTION_FIELD:
				case we_contents_root::KEYWORD_FIELD:
					break;
				default:
					switch($element['type']){
						case 'txt' :
							$we_doc->elements[$fieldname]['dat'] = self::external_to_internal((string) $element['dat']);
							break;
					}
			}
		}
		//save and publish
		if(!$we_doc->we_save()){
			return [
				'filename' => $_FILES['we_File']['name'],
				'error' => 'save_error'
			];
		}
		if(!$we_doc->we_publish()){
			return [
				'filename' => $_FILES['we_File']['name'],
				'error' => 'publish_error'
			];
		}

		return [];
	}

	/**
	 * this routine is called from task fragment class each time a document/filder is imported
	 *
	 * @param string $path
	 * @param string $contentType
	 * @param string $sourcePath
	 * @param int $destinationDirID
	 * @param string $sameName
	 * @param int[] $thumbs
	 * @param int $width
	 * @param int $height
	 * @param string $widthSelect
	 * @param string $heightSelect
	 * @param bool $keepRatio
	 * @param int $quality
	 * @param int $degrees
	 * @return array
	 * @static
	 */
	public static function importFile(string $path, string $contentType, string $sourcePath, int $destinationDirID, string $sameName, array $thumbs, int $width, int $height, string $widthSelect, string $heightSelect, bool $keepRatio, int $quality, int $degrees, bool $importMetadata = true, bool $isSearchable = true): array{
		$filesize = 0;

		// preparing Paths
		$path = str_replace('\\', '/', $path); // change windoof backslashes to slashes
		$sourcePath = str_replace('\\', '/', $sourcePath); // change windoof backslashes to slashes
		$sizeofdocroot = strlen(rtrim(WE_DOCUMENT_ROOT, '/')); // make sure that no ending slash is there
		$sizeofsourcePath = strlen(rtrim($sourcePath, '/')); // make sure that no ending slash is there
		$destinationDir = id2path($destinationDirID);
		if($destinationDir === '/'){
			$destinationDir = '';
		}
		$destinationPath = $destinationDir . '/' . we_import_functions::correctFilename(substr($path, $sizeofdocroot + $sizeofsourcePath) ?: '', true);
		$parentDirPath = dirname($destinationPath);

		$parentID = path_to_id($parentDirPath);
		$data = '';

		// initializing $we_doc
		$we_doc = we_contents_base::initDoc($contentType);
		assert($we_doc instanceof we_document_files);
		// initialize Path Information
		$we_doc->we_new();
		$we_doc->ContentType = $contentType;
		$we_doc->Path = $destinationPath;
		$we_doc->ParentID = $parentID;
		$we_doc->ParentPath = $we_doc->getParentPath();

		$Text = we_import_functions::correctFilename(basename($path));
		$regs = [];
		if(preg_match('|^(.+)(\.[^\.]+)$|', $Text, $regs)){
			$we_doc->Extension = $regs[2];
			$we_doc->Filename = $regs[1];
		} else {
			$we_doc->Extension = '';
			$we_doc->Filename = $Text;
		}

		// get Data of File
		switch($contentType){
			case we_base_ContentTypes::IMAGE:
			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:
			case we_base_ContentTypes::VIDEO:
			case we_base_ContentTypes::AUDIO:
				$filesize = !is_dir($path) && ($filesize = filesize($path)) ? $filesize : 0;
				break;
			case we_base_ContentTypes::FOLDER_DOCUMENT:
				$we_doc->Filename = $Text;
				break;
			default:
				if(!is_dir($path) && filesize($path)){
					$data = we_base_file::load($path);
				}
		}

		$id = path_to_id($we_doc->Path);

		if($id){
			switch($contentType === we_base_ContentTypes::FOLDER_DOCUMENT ? we_exim_ExIm::COLLISION_OVERWRITE : $sameName){// folders we dont have to rename => we can use the existing folder
				case we_exim_ExIm::COLLISION_OVERWRITE:
					$we_doc->initByID($id, FILE_TABLE);
					break;
				case we_exim_ExIm::COLLISION_RENAME:
					$z = 0;
					$footext = $we_doc->getText($z);
					while(fI('SELECT 1 FROM ' . FILE_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($footext) . '" AND ParentID=' . $parentID)){
						$footext = $we_doc->getText(++$z);
					}
					$we_doc->Filename = $we_doc->Filename . '_' . $z;
					$we_doc->Path = $we_doc->getPath();
					break;
				default:
					return [
						'filename' => $we_doc->Path,
						'error' => 'same_name'
					];
			}
		}

		$we_doc->IsSearchable = false;

		// initialize Content
		switch($contentType){
			case we_base_ContentTypes::WEDOCUMENT :
				assert($we_doc instanceof we_document_webEdition);
				self::importWebEditionPage($data, $we_doc, $sourcePath);
				$we_doc->IsSearchable = $isSearchable;
				break;
			case we_base_ContentTypes::FOLDER_DOCUMENT:
				break;
			case we_base_ContentTypes::IMAGE :
				assert($we_doc instanceof we_document_image);
				// getting attributes of image
				try{
					$foo = $we_doc->getimagesize($path);
					$we_doc->setElementI('width', $foo[0], 'attrib');
					$we_doc->setElementI('height', $foo[1], 'attrib');
					$we_doc->setElementI('origwidth', $foo[0], 'attrib');
					$we_doc->setElementI('origheight', $foo[1], 'attrib');
				} catch (we_exception_image $e){

				}

				$we_doc->Thumbs = $thumbs;
				$newWidth = ($width && $widthSelect === 'percent' ?
					round(($we_doc->getElementI('origwidth') / 100) * $width) :
					$width);

				$newHeight = ($height && $heightSelect === 'percent' ?
					round(($we_doc->getElementI('origheight') / 100) * $height) :
					$height);

				if(($newWidth && ($newWidth != $we_doc->getElementI('origwidth'))) || ($newHeight && ($newHeight != $we_doc->getElementI('origheight')))){
					$we_doc->resizeImage((int) $newWidth, (int) $newHeight, $quality, $keepRatio);
					/* $width = $newWidth;
					  $height = $newHeight; */
				}

				if($degrees){
					$we_doc->rotateImage(($degrees % 180 == 0) ?
							$we_doc->getElementI('origwidth') :
							$we_doc->getElementI('origheight'), ($degrees % 180 == 0) ?
							$we_doc->getElementI('origheight') :
							$we_doc->getElementI('origwidth'), $degrees, $quality);
				}
				$we_doc->DocChanged = true;
			// no break!! because we need to do the same after the following case
			case we_base_ContentTypes::APPLICATION:
			case we_base_ContentTypes::PDF:
			case we_base_ContentTypes::VIDEO:
				assert($we_doc instanceof we_document_binary);
				$we_doc->setElementI('filesize', $filesize, 'attrib');
				$we_doc->setElementS('data', $path, 'image');
				$we_doc->IsSearchable = $isSearchable;
				break;
			case we_base_ContentTypes::AUDIO:
				assert($we_doc instanceof we_document_audio);
				$we_doc->setElementI('filesize', $filesize, 'attrib');
				$we_doc->setElementS('data', $path, 'audio');
				$we_doc->IsSearchable = $isSearchable;
				break;
			case we_base_ContentTypes::HTML:
				assert($we_doc instanceof we_document_textContent);
				$we_doc->IsSearchable = $isSearchable;
			case we_base_ContentTypes::TEXT:
			case we_base_ContentTypes::JS:
			case we_base_ContentTypes::CSS:
			default:
				// set Data of File
				$we_doc->setElementS('data', $data, 'txt');
		}

		//save and publish
		if(!$we_doc->we_save()){
			return [
				'filename' => $path,
				'error' => 'save_error'
			];
		}
		if($contentType == we_base_ContentTypes::IMAGE && $importMetadata){
			assert($we_doc instanceof we_document_image);
			$we_doc->importMetaData();
			$we_doc->we_save();
		}
		if(!$we_doc->we_publish()){
			return [
				'filename' => $path,
				'error' => 'publish_error'
			];
		}
		return [];
	}

	/**
	 * this function is called right before starting to import the files
	 *
	 */
	private function fillFiles(): void{
		// directory from which we import (real path)
		// when running on windows we have to change slashes to backslashes
		$importDirectory = str_replace('/', DIRECTORY_SEPARATOR, rtrim(rtrim(WE_DOCUMENT_ROOT, '/') . $this->from, '/'));
		//$this->actualDepth = 0;
		$this->postProcess = [];
		$this->fillDirectories($importDirectory, true);
		// sort it so that webEdition files are at the end (that templates know about css and js files)

		$tmp = [
			'folder' => [],
			'other' => [],
			'doc' => [],
		];
		foreach($this->files as $e){
			switch($e['contentType']){
				case we_base_ContentTypes::FOLDER_DOCUMENT:
					$tmp['folder'][] = $e;
					break;
				case we_base_ContentTypes::WEDOCUMENT:
					$tmp['doc'][] = $e;
					break;
				default:
					$tmp['other'][] = $e;
			}
		}

		$this->files = array_merge($tmp['folder'], $tmp['other'], $tmp['doc'], $this->postProcess);
	}

	/**
	 * this function fills the $this->files and $this->_postProcess arrays
	 * @param $importDirectory string
	 */
	private function fillDirectories(string $importDirectory, bool $clean = false): void{
		if($clean){
			$this->files = [];
		}
		if($importDirectory === rtrim(WEBEDITION_PATH, '/') ||
			$importDirectory === rtrim(WE_DOCUMENT_ROOT . WE_THUMBNAIL_DIRECTORY, '/')){ // we do not import stuff from the webEdition home dir
			return;
		}

		// go throuh all files of the directory
		$d = dir($importDirectory);
		$thumbdir = trim(WE_THUMBNAIL_DIRECTORY, '/');
		$wedir = trim(WEBEDITION_DIR, '/');
		while(false !== ($entry = $d->read())){
			switch($entry){
				case $thumbdir:
				case $wedir:
				case '.':
				case '..':
					continue 2;
				default:
					if(!((strlen($entry) >= 2) && str_starts_with($entry, '._'))){
						break;
					}
			}
			// now we have to check if the file should be imported
			$PathOfEntry = $importDirectory . DIRECTORY_SEPARATOR . $entry;

			if(str_contains($PathOfEntry, trim(WEBEDITION_DIR, '/')) || is_link($PathOfEntry) ||
				(!is_dir($PathOfEntry) && ($this->maxSize && (filesize($PathOfEntry) > (abs($this->maxSize) * 1024 * 1024))))){
				continue;
			}
			$contentType = we_base_ContentTypes::getFromFilename($PathOfEntry);
			$importIt = false;

			switch($contentType){
				case we_base_ContentTypes::IMAGE:
					if($this->images){
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::HTML:
					if($this->htmlPages){
						if($this->createWePages){
							$contentType = we_base_ContentTypes::WEDOCUMENT;
							// webEdition files needs to be post processed (external links => internal links)
							$this->postProcess[] = [
								'path' => $PathOfEntry,
								'contentType' => "post/process",
								'sourceDir' => $this->from,
								'destDirID' => $this->to
							];
						}
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::VIDEO:
					if($this->video){
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::AUDIO:
					if($this->other){//FIXME: has to be audio!
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::JS:
					if($this->js){
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::TEXT:
					if($this->text){
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::CSS:
					if($this->css){
						$importIt = true;
					}
					break;
				case we_base_ContentTypes::FOLDER_DOCUMENT :
					$importIt = false;
					break;
				default:
					if($this->other){
						$importIt = true;
					}
					break;
			}

			if($importIt){
				$this->files[] = [
					'path' => $PathOfEntry,
					'contentType' => $contentType,
					'sourceDir' => $this->from,
					'destDirID' => $this->to,
					'sameName' => $this->sameName,
					'thumbs' => $this->thumbs,
					'width' => $this->width,
					'height' => $this->height,
					'widthSelect' => $this->widthSelect,
					'heightSelect' => $this->heightSelect,
					'keepRatio' => $this->keepRatio,
					'quality' => $this->quality,
					'degrees' => $this->degrees,
					'isSearchable' => $this->isSearchable,
					'importMetadata' => $this->importMetadata
				];
			}
			if($contentType === we_base_ContentTypes::FOLDER_DOCUMENT){
				if(($this->depth == -1)/* || (abs($this->depth) > $this->actualDepth) */){
					$this->files[] = [
						'path' => $PathOfEntry,
						'contentType' => $contentType,
						'sourceDir' => $this->from,
						'destDirID' => $this->to,
						'sameName' => $this->sameName,
						'thumbs' => '',
						'width' => 0,
						'height' => 0,
						'widthSelect' => '',
						'heightSelect' => '',
						'keepRatio' => false,
						'quality' => 0,
						'degrees' => 0,
						'isSearchable' => false,
						'importMetadata' => false
					];
					//$this->actualDepth++;
					$this->fillDirectories($PathOfEntry);
					//$this->actualDepth--;
				}
			}
		}
		$d->close();
	}

	/**
	 * returns hidden fields
	 * @return string
	 */
	private function getHiddensHTML(): string{
		return we_html_element::hiddens([
				'we_cmd[0]' => 'siteImport',
				'cmd' => 'buttons',
				'step' => '1']);
	}

	private function getWizFrameset(): string{
		$body = we_html_element::body(['id' => 'weMainBody']
				, we_html_element::iframe('wizbody', WEBEDITION_DIR . "we_cmd.php?we_cmd[0]=siteImport&cmd=content", 'position:absolute;top:0;bottom:40px;left:0;right:0;') .
				we_html_element::iframe('wizbusy', "we_cmd.php?we_cmd[0]=siteImport&cmd=buttons", 'position:absolute;height:40px;bottom:0;left:0;right:0;overflow: hidden') .
				we_html_element::iframe('wizcmd', "about:blank", 'position:absolute;bottom:0;height:0;left:0;right:0;overflow: hidden;')
		);

		return $this->getHtmlPage($body);
	}

	private function getHtmlPage(string $body, string $js = ''): string{
		return we_html_tools::getHtmlTop('', $js, $body);
	}

}
