<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * collection of the navigation items
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 *
 * @license		http://www.gnu.org/copyleft/gpl.html  GPL
 * @version    	SVN: $Id$
 */
class we_mail_address{
	public $name;
	public $email;

	public function __construct(string $email, string $name = ''){
		$this->email = $this->filterEmail($email);
		$this->name = $this->filterName(($name === '' || $name === $email) ? '' : $name);
	}

	/**
	 * Formats e-mail address
	 *
	 * @return string
	 */
	public function format(): string{
		if($this->name === '' || $this->name === $this->email){
			return $this->email;
		}
		$encodedName = we_mail_znd::encodeHeader($this->name);
		$format = ($encodedName === $this->name && strcspn($this->name, '()<>[]:;@\\,.') != strlen($this->name) ?
			'"%s" <%s>' :
			'%s <%s>');

		return sprintf($format, $encodedName, $this->email);
	}

	public function getFormatedMail(): string{
		return ($this->name === '' || $this->name === $this->email ?
			$this->email :
			$this->name . ' <' . $this->email . '>');
	}

	public static function parse(string $user): ?we_mail_address{
		if(!$user){
			return null;
		}
		$match = [];
		if(preg_match('/([^<]*)<(.)*>/', $user, $match)){
			return new self((string) trim($match[2]), (string) trim($match[1]));
		}
		return new self($user, '');
	}

	protected function filterEmail(string $email): string{
		return strtr($email, [
			"\r" => '',
			"\n" => '',
			"\t" => '',
			'"' => '',
			',' => '',
			'<' => '',
			'>' => '',
		]);
	}

	/**
	 * Filter of name data
	 *
	 * @param string $name
	 * @return string
	 */
	protected function filterName(string $name): string{
		return trim(strtr($name, [
			"\r" => '',
			"\n" => '',
			"\t" => '',
			'"' => "'",
			'<' => '[',
			'>' => ']',
		]));
	}

}
