<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_newsletter_view extends we_modules_view{
	const MAILS_ALL = 0;
	const MAILS_CUSTOMER = 1;
	const MAILS_EMAILS = 2;
	const MAILS_FILE = 3;

	// settings array; format settings[setting_name]=settings_value
	/** @var array<string,string> */
	var $settings = [];
	//wat page is currentlly displayed 0-properties(default);1-overview;

	/** @var array	 */
	var $customers_fields = [];

	/** @var int */
	protected $show_import_box = -1;

	/** @var int */
	protected $show_export_box = -1;

	/** @var we_newsletter_newsletter */
	var $Model;

	/** @var string */
	private $uid;

	public function __construct(){
		parent::__construct(we_modules_info::NEWSLETTER);

		$this->Model = new we_newsletter_newsletter(we_base_request::int('inid', we_base_request::int('nid', 0)));
		$this->settings = self::getSettings();
		//FIXME: add types for settings
		if(we_modules_info::isActive(we_modules_info::CUSTOMER)){
			$this->customers_fields = array_column($this->db->getAllq('SHOW FIELDS FROM ' . CUSTOMER_TABLE), 'Field');
		}
		$this->Model->Text = g_l('modules_newsletter', '[new_newsletter]');
		$this->Model->Sender = $this->settings['default_sender'];
		$this->Model->Reply = $this->settings['default_reply'];
		$this->Model->Test = $this->settings['test_account'];
		$this->Model->isEmbedImages = (bool) ($this->settings['isEmbedImages']);
	}

	/**
	 * @param array<string,int|string> $predefs
	 * @return string
	 */
	function getHiddens(array $predefs = []): string{
		return we_html_element::hiddens([
				'mod' => 'newsletter',
				'ncmd' => ($predefs['ncmd'] ?? 'new_newsletter'),
				'we_cmd[0]' => 'we_showMod',
				'nid' => (string) ($predefs['nid'] ?? $this->Model->ID),
				'pnt' => ($predefs['pnt'] ?? we_base_request::string('pnt')),
				'page' => ($predefs['page'] ?? $this->page),
				'gview' => (string) ($predefs['gview'] ?? 0),
				'hm' => (string) ($predefs['hm'] ?? 0),
				'ask' => (string) ($predefs['ask'] ?? 1),
				'test' => (string) ($predefs['test'] ?? 0)
		]);
	}

	function newsletterHiddens(): string{
		return we_html_element::hidden('ID', $this->Model->ID);
	}

	function getHiddensProperty(): string{
		$out = '';
		$counter = 0;

		foreach($this->Model->groups as $group){
			$out .= $group->getHidden($counter);
			$counter++;
		}

		return $out . we_html_element::hiddens([
				'groups' => (string) ($counter),
				'Step' => (string) ($this->Model->Step),
				'Offset' => (string) ($this->Model->Offset),
				'IsFolder' => (string) ($this->Model->IsFolder)
		]);
	}

	function getHiddensPropertyPage(): string{
		return we_html_element::hiddens([
				'Text' => $this->Model->Text,
				'Subject' => $this->Model->Subject,
				'ParentID' => (string) ($this->Model->ParentID),
				'Sender' => $this->Model->Sender,
				'Reply' => $this->Model->Reply,
				'Test' => (string) ($this->Model->Test),
				'isEmbedImages' => (string) ($this->Model->isEmbedImages)
		]);
	}

	function getHiddensMailingPage(): string{
		$out = '';
		foreach($this->Model->groups as $g => $group){
			$out .= $group->getHiddenFilter((int) $g);
		}
		return $out;
	}

	function getHiddensContentPage(): string{
		$out = '';
		foreach($this->Model->blocks as $counter => $bk){
			$out .= $bk->getHidden((int) $counter);
		}

		return $out . we_html_element::hidden('blocks', count($this->Model->blocks));
	}

	function getFields(int $id, string $table): array{
		$foo = [];

		switch(($ct = fS('SELECT ContentType FROM ' . $this->db->escape($table) . ' WHERE ID=' . $id))){
			case we_base_ContentTypes::OBJECT_FILE:
			case we_base_ContentTypes::WEDOCUMENT:
				break;
			default:
				return [];
		}

		$ent = we_base_ContentTypes::getObject($ct, $id);
		assert($ent instanceof we_document_base);

		foreach(array_keys($ent->elements) as $v){
			switch($ent->elements[$v]['type'] ?? ''){
				case we_contents_objectBase::TYPE_HREF:
					if($ct === we_base_ContentTypes::WEDOCUMENT){
						break;
					}
				case 'txt':
				case we_contents_objectBase::TYPE_INPUT:
				case we_contents_objectBase::TYPE_COUNTRY:
				case we_contents_objectBase::TYPE_LANGUAGE:
				case we_contents_objectBase::TYPE_FLOAT:
				case we_contents_objectBase::TYPE_DATE:
				case we_contents_objectBase::TYPE_TEXT:
				case we_contents_objectBase::TYPE_CHECKBOX:
				case we_contents_objectBase::TYPE_INT:
				case we_contents_objectBase::TYPE_IMG:
				case we_contents_objectBase::TYPE_LINK:
				case we_contents_objectBase::TYPE_SELECT:
					$foo[$v] = $v;
					break;
			}
		}

		return $foo;
	}

	function getJSTop(): string{
		return parent::getJSTop() .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_top.js', "parent.document.title='" . $this->getModTitle() . "'");
	}

	function getJSProperty(array $jsVars = []): string{
		return we_html_element::jsScript(JS_DIR . 'global.js', 'initWE();') .
			we_html_element::jsScript(WE_JS_MODULES_DIR . 'newsletter/newsletter_property.js', '', ['id' => 'loadVarNewsletter_property', 'data-nlView' => setDynamicVar(array_merge($jsVars, [
					'checkMail' => !empty($this->settings['reject_save_malformed']),
					'uid' => ($this->uid ?? ''),
					'testMail' => $this->Model->Test,
				]))]
		);
	}

	private function new_newsletter(): void{
		$this->Model = new we_newsletter_newsletter();
		$this->Model->Text = g_l('modules_newsletter', '[new_newsletter]');
		$this->Model->Sender = $this->settings['default_sender'];
		$this->Model->Reply = $this->settings['default_reply'];
		$this->Model->Test = $this->settings['test_account'];
		$this->Model->isEmbedImages = (bool) ($this->settings['isEmbedImages']);
	}

	private function new_newsletter_group(): void{
		$this->page = 0;
		$this->Model = new we_newsletter_newsletter();
		$this->Model->IsFolder = true;
		$this->Model->Text = g_l('modules_newsletter', '[new_newsletter_group]');
	}

	private function add_customer(): void{
		if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return;
		}

		$this->Model->groups[$ngroup]->addCustomer(we_base_request::intList('ncustomer'));
	}

	private function del_customer(): void{
		if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return;
		}

		$this->Model->groups[$ngroup]->delCustomer(we_base_request::int('ncustomer'));
	}

	private function add_file(): void{
		if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return;
		}

		$arr = $this->Model->groups[$ngroup]->getExtern();
		if(!($nfile = we_base_request::_(we_base_request::FILE, 'nfile', ''))){
			return;
		}
		$sd = str_replace('\\', '/', WE_DOCUMENT_ROOT);
		$arr[] = str_replace($sd, (str_ends_with($sd, '/') ? '/' : ''), $nfile);
		$this->Model->groups[$ngroup]->setExtern($arr);
	}

	private function del_file(): void{
		if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return;
		}

		$arr = $this->Model->groups[$ngroup]->getExtern();
		$nfile = we_base_request::_(we_base_request::FILE, 'nfile', '');
		if(!$nfile){
			return;
		}

		if(($pos = array_search($nfile, $arr, false)) === false){
			return;
		}
		unset($arr[$pos]);
		$this->Model->groups[$ngroup]->setExtern($arr);
	}

	private function save_newsletter(we_base_jsCmd $jscmd): void{
		if((we_base_request::int('nid', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
			return;
		}
		$newone = false;

		if($this->Model->filenameNotValid()){
			$jscmd->addMsg(g_l('modules_newsletter', '[we_filename_notValid]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if($this->Model->ParentID){
			$folder = fB('SELECT IsFolder FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . intval($this->Model->ParentID));
			if(empty($folder)){
				$jscmd->addMsg(g_l('modules_newsletter', '[path_nok]'), we_base_util::WE_MESSAGE_ERROR);
				return;
			}
		}

		if(isset($_REQUEST['blocks'])){
			for($i = 0; $i < $_REQUEST['blocks']; $i++){
				switch(we_base_request::int('block' . $i . '_Type')){
					case we_newsletter_block::DOCUMENT:
					case we_newsletter_block::DOCUMENT_FIELD:
						$acTable = FILE_TABLE;
						$acErrorField = g_l('modules_newsletter', '[block_document]');
						break;
					case we_newsletter_block::OBJECT:
					case we_newsletter_block::OBJECT_FIELD:
						$acTable = OBJECT_FILES_TABLE;
						$acErrorField = g_l('modules_newsletter', '[block_object]');
						break;
					default:
						$acTable = '';
						$acErrorField = '';
				}
				if($acTable){
					$folder = fB('SELECT IsFolder FROM ' . $acTable . ' WHERE ID=' . we_base_request::int('block' . $i . '_LinkID'));
					if($folder){
						$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[blockFieldError]'), ($i + 1), $acErrorField), we_base_util::WE_MESSAGE_ERROR);
						return;
					}
					if(($field = we_base_request::int('block' . $i . '_Field'))){
						$folder = fB('SELECT IsFolder FROM ' . TEMPLATES_TABLE . ' WHERE ID=' . $field);
						if($folder){
							$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[blockFieldError]'), $i, g_l('modules_newsletter', '[block_template]')), we_base_util::WE_MESSAGE_ERROR);
							return;
						}
					}
				}
			}
		}

		if(!$this->Model->ID){
			$newone = true;
		}

		if(!$newone && we_base_request::bool('ask')){
			$h = getHash('SELECT Step,Offset FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . intval($this->Model->ID), $this->db);

			if($h['Step'] != 0 || $h['Offset'] != 0){
				$jscmd->addCmd("save_newsletter_question");
				return;
			}
		}

		if(!$this->Model->Sender){
			$this->Model->Sender = $this->settings['default_sender'];
		}
		if(!$this->Model->Reply){
			$this->Model->Reply = $this->settings['default_reply'];
		}
		if(!$this->Model->Test){
			$this->Model->Test = $this->settings['test_account'];
		}
		if(!$this->Model->isEmbedImages){
			$this->Model->isEmbedImages = (bool) ($this->settings['isEmbedImages']);
		}

		$double = fI('SELECT COUNT(1) FROM ' . NEWSLETTER_TABLE . ' WHERE Path="' . $this->db->escape($this->Model->Path) . '"' . ($newone ? '' : ' AND ID!=' . $this->Model->ID));

		if(!we_base_permission::hasPerm([we_base_perm::NEWSLETTER_EDIT, we_base_perm::NEWSLETTER_NEW])){
			$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if($newone && !we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW])){
			$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!$newone && !we_base_permission::hasPerm([we_base_perm::NEWSLETTER_EDIT])){
			$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		if($double){
			$jscmd->addMsg(g_l('modules_newsletter', '[double_name]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		try{
			$this->Model->saveNewsletter((bool) ($this->settings['reject_save_malformed'] ?? true));
			if($newone){
				$jscmd->addCmd('makeTreeEntry', [
					'id' => $this->Model->ID,
					'parentid' => $this->Model->ParentID,
					'text' => $this->Model->Text,
					'open' => false,
					'contenttype' => ($this->Model->IsFolder ? we_base_ContentTypes::FOLDER_COMMON : we_base_ContentTypes::NEWSLETTER),
					'table' => NEWSLETTER_TABLE
				]);
			} else {
				$jscmd->addCmd('updateTreeEntry', [
					'id' => $this->Model->ID,
					'parentid' => $this->Model->ParentID,
					'text' => $this->Model->Text,
				]);
			}
			$jscmd->addCmd('drawTree');
			$jscmd->addCmd('unsetHot');
			$jscmd->addMsg(g_l('modules_newsletter', ($this->Model->IsFolder == 1 ? '[save_group_ok]' : '[save_ok]')), we_base_util::WE_MESSAGE_NOTICE);
		} catch (we_exception_newsletter $e){
			switch($e->getCode()){
				default:
				case we_newsletter_newsletter::MALFORMED_GROUP:
					$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_group]'), $e->getCode(), $e->getMessage()), we_base_util::WE_MESSAGE_ERROR);
					break;
				case we_newsletter_newsletter::MALFORMED_SENDER:
					$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_sender]'), $e->getMessage()), we_base_util::WE_MESSAGE_ERROR);
					break;
				case we_newsletter_newsletter::MALFORMED_REPLY:
					$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_reply]'), $e->getMessage()), we_base_util::WE_MESSAGE_ERROR);
					break;
				case we_newsletter_newsletter::MALFORMED_TEST:
					$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[malformed_mail_test]'), $e->getMessage()), we_base_util::WE_MESSAGE_ERROR);
					break;
				case we_newsletter_newsletter::SAVE_PATH_NOK:
					$jscmd->addMsg($e->getMessage(), we_base_util::WE_MESSAGE_ERROR);
					break;
			}
		}
	}

	private function delete_newsletter(we_base_jsCmd $jscmd): void{
		$nid = we_base_request::int('nid');
		if(!$nid){
			$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::bool('IsFolder') ? '[delete_group_nok]' : '[delete_nok]')), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		if(!we_base_permission::hasPerm([we_base_perm::NEWSLETTER_DELETE])){
			$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
			return;
		}
		$this->Model = new we_newsletter_newsletter($nid);

		if(!$this->Model->delete()){
			$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::bool('IsFolder') ? '[delete_group_nok]' : '[delete_nok]')), we_base_util::WE_MESSAGE_ERROR);
			return;
		}

		$this->Model = new we_newsletter_newsletter();
		$jscmd->addCmd('deleteTreeEntry', [$nid, "file"]);
		$jscmd->addMsg(g_l('modules_newsletter', (we_base_request::bool('IsFolder') ? '[delete_group_ok]' : '[delete_ok]')), we_base_util::WE_MESSAGE_NOTICE);
		$_REQUEST['home'] = 1;
		$_REQUEST['pnt'] = 'edbody';
	}

	private function import_csv(we_base_jsCmd $jscmd, int $importno): void{
		$filepath = we_base_request::_(we_base_request::FILE, 'csv_file' . $importno);
		$delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter' . $importno);
		$col = max(0, we_base_request::int('csv_col' . $importno, 1) - 1);

		$imports = [
			'hmcol' => ['import' => false, 'val' => 0],
			'salutationcol' => ['import' => false, 'val' => 0],
			'titlecol' => ['import' => false, 'val' => 0],
			'firstnamecol' => ['import' => false, 'val' => 0],
			'lastnamecol' => ['import' => false, 'val' => 0],
		];
		'@phan-var array<string,array{import:bool,val:int}> $imports';
		foreach($imports as $key => &$vals){
			$vals['val'] = we_base_request::int('csv_' . $key . $importno, 0);
			$vals['import'] = ($vals['val'] > 0);
			if($vals['val']){
				$vals['val']--;
			}
		}

		if(str_contains($filepath, '..')){
			$jscmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
		} else {
			$row = [];
			$control = [];
			$fh = @fopen(WE_DOCUMENT_ROOT . $filepath, 'rb');
			if($fh){
				$mailListArray = explode("\n", $this->Model->groups[$importno]->Emails);
				foreach($mailListArray as $line){
					[$tmp] = explode(',', $line, 2);
					$mailListArray[] = $tmp;
				}

				while(($dat = fgetcsv($fh, 1000, $delimiter))){
					'@phan-var string[] $dat';
					if(isset($control[$dat[$col]])){
						continue;
					}
					$alldat = implode('', $dat);
					if(str_replace(' ', '', $alldat) === ''){
						continue;
					}
					$mailrecip = (str_replace(' ', '', $dat[$col] ?? '') === '') ? '--- ' . g_l('modules_newsletter', '[email_missing]') . ' ---' : $dat[$col];
					if(!empty($mailrecip) && !in_array($mailrecip, $mailListArray)){
						$row[] = implode(',', [
							$mailrecip,
							( ($imports['hmcol']['import']) ? $dat[$imports['hmcol']['val']] : ''),
							( ($imports['salutationcol']['import']) ? $dat[$imports['salutationcol']['val']] : ''),
							( ($imports['titlecol']['import'] ) ? $dat[$imports['titlecol']['val']] : ''),
							( ($imports['firstnamecol']['import'] ) ? $dat[$imports['firstnamecol']['val']] : ''),
							( ($imports['lastnamecol']['import'] ) ? $dat[$imports['lastnamecol']['val']] : '')
						]);
						$control[$dat[$col]] = 1;
					}
				}
				fclose($fh);
				$this->Model->groups[$importno]->Emails .= ($this->Model->groups[$importno]->Emails ? "\n" : '') . implode("\n", $row);
			} else {
				$jscmd->addMsg(g_l('modules_newsletter', '[path_not_valid]'), we_base_util::WE_MESSAGE_ERROR);
			}
		}
	}

	private static function save_email_file(): void{
		$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', '');
		$nrid = we_base_request::int('nrid', 0);
		$email = we_base_request::_(we_base_request::EMAIL, 'email', '');
		$htmlmail = we_base_request::bool('htmlmail', false);
		$salutation = we_base_request::string('salutation', '');
		$title = we_base_request::string('title', '');
		$firstname = we_base_request::string('firstname', '');
		$lastname = we_base_request::string('lastname', '');

		$emails = ($csv_file ? we_newsletter_newsletter::getEmailsFromExternCSV([$csv_file]) : []);

		$emails[$nrid] = [$email, $htmlmail, $salutation, $title, $firstname, $lastname];
		$emails_out = '';
		foreach($emails as $mail){
			'@phan-var array $mail';
			$emails_out .= implode(',', array_slice($mail, 0, 6)) . "\n";
		}

		if($csv_file){
			we_base_file::save(WE_DOCUMENT_ROOT . $csv_file, $emails_out);
		}
	}

	private static function delete_email_file(): void{
		$nrid = we_base_request::int('nrid', -1);
		$csv_file = we_base_request::_(we_base_request::FILE, 'csv_file', '');
		$emails = ($csv_file ? we_newsletter_newsletter::getEmailsFromExternCSV($csv_file, 2) : []);

		if($nrid > -1){
			unset($emails[$nrid]);
			$emails_out = '';
			foreach($emails as $email){
				$emails_out .= implode(',', $email) . "\n";
			}

			if($csv_file){
				we_base_file::save(WE_DOCUMENT_ROOT . $csv_file, $emails_out);
			}
		}
	}

	function processCommands(we_base_jsCmd $jscmd): void{
		switch(($ncmd = we_base_request::string('ncmd'))){
			case 'new_newsletter':
				$this->new_newsletter();
				$jscmd->addCmd('loadHeadFooter', ['page' => we_base_request::int('page'), 'txt' => $this->Model->Text, 'folder' => 0]);
				break;
			case 'new_newsletter_group':
				$this->new_newsletter_group();
				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text, 'folder' => 1]);
				break;
			case 'add_customer':
				$this->add_customer();
				break;
			case 'del_customer':
				$this->del_customer();
				break;
			case 'add_file':
				$this->add_file();
				break;
			case 'del_file':
				$this->del_file();
				break;
			case 'del_all_files':
				if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
					break;
				}
				$this->Model->groups[$ngroup]->setExtern([]);
				break;
			case 'del_all_customers':
				if(($ngroup = we_base_request::int('ngroup', we_base_request::NOT_VALID_I)) === we_base_request::NOT_VALID_I){
					break;
				}
				$this->Model->groups[$ngroup]->delAllCust();
				break;
			case 'reload':
				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text, 'folder' => $this->Model->IsFolder]);
				break;
			case 'we_newsletter_edit':
				if(($nid = we_base_request::int('nid'))){
					$this->Model = new we_newsletter_newsletter($nid);
				}
				if($this->Model->IsFolder){
					$this->page = 0;
				}

				$jscmd->addCmd('loadHeadFooter', ['page' => $this->page, 'txt' => $this->Model->Text, 'folder' => $this->Model->IsFolder]);
				break;
			case 'switchPage':
				$this->page = we_base_request::int("page", $this->page);
				break;
			case 'save_newsletter':
				$this->save_newsletter($jscmd);
				break;
			case 'delete_newsletter':
				$this->delete_newsletter($jscmd);
				break;
			case 'reload_table':
				$this->page = 1;
				break;
			case 'set_import':
				$this->show_import_box = we_base_request::int('ngroup');
				break;
			case 'set_export':
				$this->show_export_box = we_base_request::int('ngroup');
				break;
			case 'reset_import':
				$this->show_import_box = -1;
				break;
			case 'reset_export':
				$this->show_export_box = -1;
				break;
			case 'addBlock':
				if(($bid = we_base_request::int('blockid', -1)) > -1){
					$this->Model->addBlock($bid + 1);
				}
				break;
			case 'delBlock':
				if(($bid = we_base_request::int('blockid', -1)) > -1){
					$this->Model->removeBlock($bid);
				}
				break;
			case 'addGroup':
				$this->Model->addGroup();
				$jscmd->addCmd('groups_changed');
				break;
			case 'delGroup':
				if(($ngroup = we_base_request::int('ngroup', -1)) > -1){
					$this->Model->removeGroup($ngroup);
					$jscmd->addCmd('groups_changed');
				}
				break;
			case 'send_test':
				if(!we_base_permission::hasPerm([we_base_perm::NEWSLETTER_SEND_TEST_EMAIL])){
					$jscmd->addMsg(g_l('modules_newsletter', '[no_perms]'), we_base_util::WE_MESSAGE_ERROR);
					return;
				}
				$this->sendTestMail(we_base_request::int('gview', 0), we_base_request::bool('hm'));
				$jscmd->addMsg(sprintf(g_l('modules_newsletter', '[test_mail_sent]'), $this->Model->Test), we_base_util::WE_MESSAGE_NOTICE);
				break;
			case 'add_filter':
				$this->Model->groups[we_base_request::int('ngroup')]->addFilter($this->customers_fields[0], we_base_request::int('noCustFilter'));
				break;
			case 'del_filter':
				$this->Model->groups[we_base_request::int('ngroup')]->delFilter(we_base_request::int('noCustFilter'));
				break;
			case 'del_all_filters':
				$this->Model->groups[we_base_request::int('ngroup')]->delallFilter();
				break;
			case 'copy_newsletter':
				if(($cid = we_base_request::int('copyid'))){
					$id = $this->Model->ID;
					$this->Model = new we_newsletter_newsletter($cid);
					$this->Model->ID = $id;
					$this->Model->Text .= '_' . g_l('modules_newsletter', '[copy]');
				}
				break;
			case 'save_settings':
				foreach(array_keys($this->settings) as $k){
					$this->settings[$k] = we_base_request::_(we_base_request::RAW, $k, 0);
				}
				$this->saveSettings();
				break;
			case 'import_csv':
				if(($importno = we_base_request::int('csv_import', -1)) > -1){
					$this->import_csv($jscmd, $importno);
				}
				break;
			case 'export_csv':
				if(($exportno = we_base_request::int('csv_export', -1)) > -1){
					$fname = rtrim(we_base_request::_(we_base_request::FILE, 'csv_dir' . $exportno), '/') . '/emails_export_' . time() . '.csv';

					we_base_file::save(WE_DOCUMENT_ROOT . $fname, $this->Model->groups[$exportno]->Emails);
					$jscmd->addCmd('export_csv_window', $fname);
				}
				break;

			case 'save_black':
				$this->saveSetting('black_list', $this->settings['black_list']);
				$jscmd->addCmd('close_dialog');
				break;

			case 'do_upload_csv':
			case 'do_upload_black':
				$group = we_base_request::int('group', 0);

				//set header we avoided when sending JSON only
				//some reaction on upload failure

				$fileUploader = new we_fileupload_resp_base();
				try{
					$tempName = $fileUploader->commitUploadedFile();
					//print next command
					$jscmd->addCmd('processAfterUpload', ['cmd' => $ncmd, 'name' => $tempName, 'group' => $group]);
				} catch (we_exception_file $e){
					$jscmd->addMsg($e->getMessage(), we_base_util::WE_MESSAGE_FRONTEND);
				}
				break;
			case 'save_email_file':
				self::save_email_file();
				break;
			case 'delete_email_file':
				self::delete_email_file();
				break;
			case 'popSend':
				$jscmd->addCmd('popSend_do_cont', [$this->Model->ID, empty(trim($this->Model->Subject)), we_base_request::bool('test')]);
				break;
			default:
		}
	}

	private function processVariablesGroups(int $groups): void{
		$this->Model->groups = [];

		if($groups == 0){
			$this->Model->addGroup();
		}

		for($i = 0; $i < $groups; $i++){
			$this->Model->addGroup();
		}

		foreach($this->Model->groups as $gkey => $gval){
			// persistens
			$gval->setFromRequest($this->Model->ID, (int) $gkey);
		}
	}

	private function processVariablesBlocks(int $blocks): void{
		$this->Model->blocks = [];

		if($blocks === 0){
			$this->Model->addBlock();
		}

		for($i = 0; $i < $blocks; $i++){
			$this->Model->addBlock();
		}

		foreach($this->Model->blocks as $skey => $sval){
			$sval->setFromRequest($this->Model->ID, (int) $skey);
		}
	}

	function processVariables(): void{
		if(($uid = we_base_request::string('wname'))){
			$this->uid = $uid;
		}

		foreach($this->Model->persistent_slots as $val => $type){
			$this->Model->$val = we_base_request::_($type, $val, $this->Model->$val);
		}

		if($this->Model->ParentID){
			$this->Model->Path = fS('SELECT Path FROM ' . NEWSLETTER_TABLE . ' WHERE ID=' . $this->Model->ParentID) . '/' . $this->Model->Text;
		} elseif(!$this->Model->filenameNotValid()){
			$this->Model->Path = '/' . $this->Model->Text;
		}

		$this->page = we_base_request::int('page', $this->page);
		if(($groups = we_base_request::int('groups', -1)) > -1){
			$this->processVariablesGroups($groups);
		}
		if(($blocks = we_base_request::int('blocks', -1)) > -1){
			$this->processVariablesBlocks($blocks);
		}
	}

	/**
	 * Newsletter printing functions
	 */
	private function initDocByObject(int $we_objectID): we_document_triggerDocument{
		$we_obj = new we_contents_objectFile($we_objectID);

		$we_doc = $this->initDoc();
		$we_doc->initByObj($we_obj);
		return $we_doc;
	}

	private function initDoc(int $id = 0): we_document_triggerDocument{
		return new we_document_triggerDocument($id);
	}

	private function getContenDocument(we_newsletter_block $block): string{
		$path = (!empty($block->getField()) ?
			TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', id2path((int) $block->getField(), TEMPLATES_TABLE)) :
			($block->getLinkID() ?
			TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', fS('SELECT t.Path FROM ' . FILE_TABLE . ' f LEFT JOIN ' . TEMPLATES_TABLE . ' t ON f.TemplateID=t.ID WHERE f.ID=' . $block->getLinkID())) :
			''));

		if($block->getLinkID() && $path){
			return $block->getLinkID() && we_base_file::isWeFile($block->getLinkID(), FILE_TABLE) ? we_getDocumentByID($block->getLinkID(), $path) : 'No such File';
		}
		return '';
	}

	private function getContenDocumentField(we_newsletter_block $block): string{
		if($block->getLinkID()){
			$we_doc = $this->initDoc($block->getLinkID());
			return (string) $we_doc->getField(['name' => $block->getField()], $we_doc->elements[$block->getField()]['type']);
		}

		return '';
	}

	private function getContenObject(we_newsletter_block $block): string{
		$path = (!empty($block->getField()) ? TEMPLATES_PATH . preg_replace('/\.tmpl$/i', '.php', id2path((int) $block->getField(), TEMPLATES_TABLE)) : '');

		if($block->getLinkID() && $path){
			return self::we_getObjectFileByID($block->getLinkID(), $path);
		}

		return '';
	}

	private function getContenObjectField(we_newsletter_block $block, bool $htmlMail = false): string{
		if($block->getLinkID()){
			$we_doc = $this->initDocByObject($block->getLinkID());

			switch(($type = $we_doc->elements[$block->getField()]['type'] ?? '')){
				case we_contents_objectBase::TYPE_INPUT:
				case we_contents_objectBase::TYPE_FLOAT:
				case we_contents_objectBase::TYPE_DATE:
				case we_contents_objectBase::TYPE_TEXT:
				case we_contents_objectBase::TYPE_SELECT:
					return (string) $we_doc->getField(['name' => $block->getField()], $we_doc->elements[$block->getField()]['type']);
				case we_contents_objectBase::TYPE_COUNTRY:
				case we_contents_objectBase::TYPE_LANGUAGE:
					if(($val = $we_doc->getElementS($block->getField()))){
						$what = $type === we_contents_objectBase::TYPE_COUNTRY ? we_base_country::TERRITORY : we_base_country::LANGUAGE;
						$langcode = array_search(we_base_WE::$Language, we_base_country::getWELangs(), true) ?: '';
						return we_base_country::getTranslation($val, $what, $langcode);
					}
					return '';
				case we_contents_objectBase::TYPE_INT:
				case we_contents_objectBase::TYPE_CHECKBOX:
					return (string) $we_doc->getFieldByVal((string) $we_doc->getElementI($block->getField()), $type, ['name' => $block->getField()]);
				case we_contents_objectBase::TYPE_IMG:
					return !$htmlMail ? '' : (string) $we_doc->getFieldByVal((string) $we_doc->getElementI($block->getField()), $type, ['name' => $block->getField()]);
				case we_contents_objectBase::TYPE_HREF:
					return !$htmlMail ? '' : we_document_base::getHrefByArray($we_doc->getElementA($block->getField()));
				case we_contents_objectBase::TYPE_LINK:
					return !$htmlMail ? '' : (string) $we_doc->getFieldByVal(we_serialize($we_doc->getElementA($block->getField())), $type);
			}
		}

		return '';
	}

	private function getContentText(we_newsletter_block $block, bool $htmlMail): string{
		$blockHtml = $block->getHTML() ? preg_replace(['/(href=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/',
				'/(src=")(\\\\*&quot;)*(.+?)(\\\\*&quot;)*(")/'], '${1}${3}${5}', stripslashes($block->getHTML())) : '';

		if($htmlMail){
			return $blockHtml ?:
				strtr($block->getSource(), [
					"\r\n" => '<br/>',
					"\r" => '<br/>',
					'&' => '&amp;',
					'<' => '&lt;',
					'>' => '&gt;',
					"\n" => '<br/>',
					"\t" => str_repeat('&nbsp;', 2),
			]);
		}
		return ($block->getSource() ?:
			(string) str_ireplace(['&nbsp;', '&lt;', '&gt;', '&quot;', '&amp;',], [' ', '<', '>', '"', '&',], preg_replace('|&nbsp;(&nbsp;)+|i', "\t", trim(strip_tags(preg_replace('|<br\s*/?\s*>|i', "\n", $blockHtml))))));
		//TODO: we should preserve img- and link-pathes: "text text linktext (path) text"
	}

	private function getContentFile(string $name): string{
		try{
			return we_base_file::load(WE_DOCUMENT_ROOT . $name);
		} catch (we_exception_file $e){
			echo g_l('modules_newsletter', '[cannot_open]') . ': ' . WE_DOCUMENT_ROOT . $name;
		}
		return '';
	}

	private function getContentUrl(string $urlText): string{
		$url = parse_url((substr(trim($urlText), 0, 4) != 'http' ? 'http://' . $urlText : $urlText));
		try{
			$content = we_http_util::getHTTP($url['host'], ($url['path'] ?? '/'), 0, defined('HTTP_USERNAME') ? HTTP_USERNAME : '', defined('HTTP_PASSWORD') ? HTTP_PASSWORD : '');
		} catch (we_exception_http $e){
			t_e($e->getMessage());
			return '';
		}
		$patterns = [
			"/<(img\s+[^>]+src\s*[\=\"|\=\'|\=\\\\|\=]*\s*)([^\'\"> ? \\\]*)([^\"\' \\\\>]*)(\s*[^>]*)>/si",
			"/<(link\s+[^>]+href\s*[\=\"|\=\'|\=\\\\|\=]*\s*)([^\'\"> ? \\\]*)([^\"\' \\\\>]*)(\s*[^>]*)>/si"
		];
		$match = [];

		foreach($patterns as $pattern){
			if(preg_match_all($pattern, $content, $match)){
				$unique = array_unique($match[2]);
				foreach($unique as $name){
					$src = parse_url($name);

					if(!isset($src['host'])){

						$path = (isset($src['path']) ?
							(dirname($src['path']) ?
							dirname($src['path']) . '/' :
							(isset($url['path']) ?
							dirname($url['path']) . '/' :
							'')) :
							'');

						$newname = $url['scheme'] . '://' . preg_replace('|/+|', '/', $url['host'] . '/' . $path . basename($name));
						$content = str_replace($name, $newname, $content);
					}
				}
			}
		}
		return $content;
	}

	function getContent(we_newsletter_block $block, int $gview = 0, bool $htmlMail = false, string $salutation = '', string $title = '', string $firstname = '', string $lastname = '', string $customerstring = ''): string{
		$groups = $block->getGroups();
		if(!(in_array($gview, $groups) || $gview == 0)){
			return '';
		}

		$content = '';

		we_newsletter_util::$MAIL = we_newsletter_base::EMAIL_REPLACE_TEXT;
		we_newsletter_util::$HTMLMAIL = $htmlMail;
		we_newsletter_util::$TITLE = $title;
		we_newsletter_util::$SALUTATION = $salutation;
		we_newsletter_util::$FIRSTNAME = $firstname;
		we_newsletter_util::$LASTNAME = $lastname;
		we_newsletter_util::$CUSTOMERID = $customerstring;

		switch($block->Type){
			case we_newsletter_block::DOCUMENT:
				$content = $this->getContenDocument($block);
				break;
			case we_newsletter_block::DOCUMENT_FIELD:
				$content = $this->getContenDocumentField($block);
				break;
			case we_newsletter_block::OBJECT:
				$content = $this->getContenObject($block);
				break;
			case we_newsletter_block::OBJECT_FIELD:
				$content = $this->getContenObjectField($block, $htmlMail);
				break;
			case we_newsletter_block::TEXT:
				$content = $this->getContentText($block, $htmlMail);
				break;
			case we_newsletter_block::FILE:
				$content = $this->getContentFile($block->getField());
				break;
			case we_newsletter_block::URL:
				$content = ($block->getField() ? $this->getContentUrl($block->getField()) : '');
				break;
			case we_newsletter_block::ATTACHMENT:
				break;
		}

		$port = (!empty($this->settings['use_port'])) ? ':' . $this->settings['use_port'] : '';
		$protocol = (!empty($this->settings['use_https_refer']) ? 'https://' : 'http://');

		if($htmlMail){
			if($block->Type === we_newsletter_block::URL){
				return $content;
			}
			$spacer = '\s*';

			$content = we_document_base::parseInternalLinks($content, 0);

			$urlReplace = we_folder_base::getUrlReplacements($this->db, false, true);
			if($urlReplace){
				$content = (string) preg_replace('-(["\'])//-', '${1}' . $protocol, preg_replace($urlReplace, array_keys($urlReplace), $content));
			}
			return (string) preg_replace(['-(<[^>]+src' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(<[^>]+href' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(<[^>]+background' . $spacer . '=' . $spacer . '[\'"]?)(/)-i',
					'-(background' . $spacer . ':' . $spacer . '[^url]*url' . $spacer . '\\([\'"]?)(/)-i',
					'+(background-image' . $spacer . ':' . $spacer . '[^url]*url' . $spacer . '\\([\'"]?)(/)+i',
					], ['${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					'${1}' . $protocol . $_SERVER['SERVER_NAME'] . $port . '${2}',
					], $content);
		}
		$urlReplace = we_folder_base::getUrlReplacements($this->db, true, true);
		if($urlReplace){
			$content = str_replace('//', $protocol, preg_replace($urlReplace, array_keys($urlReplace), $content));
		}
		$newplain = preg_replace(['|<br */? *>|', '|<title>.*</title>|i', '|&nbsp;(&nbsp;)+|i'], ["\n", '', "\t"], $content);
		if($block->Type != we_newsletter_block::TEXT){
			$newplain = strip_tags($newplain);
		}
		return (string) str_ireplace(['&nbsp;', '&lt;', '&gt;', '&quot;', '&amp;',], [' ', '<', '>', '"', '&'], $newplain);
	}

	/**
	 *
	 * @param int $group
	 * @return int[]
	 */
	function getGroupBlocks(int $group): array{
		if(!$group){
			return range(0, count($this->Model->blocks) - 1);
		}
		$content = [];
		foreach($this->Model->blocks as $kblock => $block){
			if(in_array($group, $block->getGroups())){
				$content[] = (int) $kblock;
			}
		}
		return $content;
	}

	function getGroupsForEmail(string $email): array{
		$ret = [];

		if(is_array($this->Model->groups)){
			$keys = array_keys($this->Model->groups);
			foreach($keys as $gk){
				$emails = $this->getEmails((int) $gk + 1, self::MAILS_ALL);

				if(in_array($email, $emails, true)){
					$ret[] = $gk + 1;
				}
			}
		}

		return $ret;
	}

	/**
	 *
	 * @param int $group
	 * @return string[]
	 */
	function getAttachments(int $group): array{
		$ret = $this->db->getAllq('SELECT
			CONCAT("' . $this->db->escape(WE_DOCUMENT_ROOT) . '",f.Path)
			FROM ' .
			NEWSLETTER_BLOCK_TABLE . ' b JOIN ' .
			FILE_TABLE . ' f ON (f.ID=b.LinkID)
			WHERE
			b.NewsletterID=' . $this->Model->ID . ' AND
			b.Type=' . we_newsletter_block::ATTACHMENT .
			($group ? ' AND FIND_IN_SET(' . $group . ',b.Groups)' : ''), true);
		'@phan-var string[] $ret';
		return $ret;
	}

	function sendTestMail(int $group, bool $htmlMail): void{
		$plain = $content = '';
		$inlines = [];

		$ret = $this->cacheNewsletter($this->Model->ID, false);
		$blocks = $this->getGroupBlocks($group);
		foreach($blocks as $i){
			try{
				if($htmlMail){
					$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_h_' . $i);
					$inlines = array_merge($inlines, $block['inlines'] ?? []);
					$content .= $block['default'];
					$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_p_' . $i);
					$plain .= $block['default'];
				} else {
					$block = we_cache_file::load(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache'] . '_p_' . $i);
					$content .= $block['default'];
					$plain .= $block['default'];
				}
			} catch (we_exception_cache $e){

			}
		}
		$to = we_mail_address::parse($this->Model->Test);
		if(!$to){
			throw new we_exception_mail('no recipient given');
		}
		$atts = $this->getAttachments($group);
		//$clean = $this->getCleanMail($this->newsletter->Reply);
		$phpmail = new we_mail_mail([$to], $this->Model->Subject, we_mail_address::parse($this->Model->Sender), we_mail_address::parse($this->Model->Reply), $this->Model->isEmbedImages);
		if(!$this->settings['use_base_href']){
			$phpmail->setIsUseBaseHref((bool) $this->settings['use_base_href']);
		}
		if($htmlMail){
			$phpmail->addHTMLPart($content);
		}
		$phpmail->addTextPart(trim($plain));
		foreach($atts as $att){
			$phpmail->doaddAttachment((string) $att);
		}
		//FIXME: what abpout inlines???

		$phpmail->buildMessage();
		$phpmail->send();

		we_cache_file::clean(we_cache_file::DOMAIN_NEWSLETTER . $ret['blockcache']);
	}

	function getFilterSQL(array $filter): string{
		$filterSQL = $filter['fieldname'];
		switch($filter['fieldname']){
			case 'MemberSince_Date':
			case 'LastLogin_Date':
			case 'LastAccess_Date':
				if(stristr($filter['fieldvalue'], '.')){
					$date = new DateTime($filter['fieldvalue']);
					$filter['fieldvalue'] = $date->format('Y-m-d H:i:00');
				}
		}

		switch($filter['operator']){
			case we_newsletter_newsletter::OP_EQ:
				return $filterSQL . '="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_NEQ:
				return $filterSQL . '!="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LE:
				return $filterSQL . '<"' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LEQ:
				return $filterSQL . '<="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_GE:
				return $filterSQL . '>"' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_GEQ:
				return $filterSQL . '>="' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_LIKE:
				return $filterSQL . ' LIKE "' . $filter['fieldvalue'] . '"';
			case we_newsletter_newsletter::OP_CONTAINS:
				return $filterSQL . ' LIKE "%' . $filter['fieldvalue'] . '%"';
			case we_newsletter_newsletter::OP_STARTS:
				return $filterSQL . ' LIKE "' . $filter['fieldvalue'] . '%"';
			case we_newsletter_newsletter::OP_ENDS:
				return $filterSQL . ' LIKE "%' . $filter['fieldvalue'] . '"';
			default:
				return $filterSQL;
		}
	}

	/**
	 * @param int $group
	 * @param int $select
	 * @return string[]
	 */
	function getEmails(int $group, int $select = self::MAILS_ALL): array{
		$extern = ($select === self::MAILS_ALL || $select === self::MAILS_FILE) ?
			we_newsletter_base::getEmailsFromExtern($this->Model->groups[$group - 1]->getExtern()) :
			[];

		if($select == self::MAILS_FILE){
			return $extern;
		}

		$list = ($select == self::MAILS_ALL || $select == self::MAILS_EMAILS) ?
			we_newsletter_base::getEmailsFromList($this->Model->groups[$group - 1]->Emails) :
			[];
		if($select == self::MAILS_EMAILS){
			return $list;
		}

		$customer_mail = $this->getEmailsCustomer($group);
		if($select == self::MAILS_CUSTOMER){
			return $customer_mail;
		}
		return array_merge($customer_mail, $list, $extern);
	}

	/**
	 * @param int $group
	 * @param int $select
	 * @return array<int,array{string,string,string,string,string,string}>
	 */
	function getEmailsCSV(int $group, int $select = self::MAILS_ALL): array{
		$extern = ($select === self::MAILS_ALL || $select === self::MAILS_FILE) ?
			we_newsletter_base::getEmailsFromExternCSV($this->Model->groups[$group - 1]->getExtern(), 0, $group, $this->getGroupBlocks($group)) :
			[];

		if($select == self::MAILS_FILE){
			return $extern;
		}

		$list = ($select == self::MAILS_ALL || $select == self::MAILS_EMAILS) ?
			we_newsletter_base::getEmailsFromListCSV($this->Model->groups[$group - 1]->Emails, $group, $this->getGroupBlocks($group)) :
			[];
		if($select == self::MAILS_EMAILS){
			return $list;
		}

		$customer_mail = $this->getEmailsCustomerCSV($group);
		if($select == self::MAILS_CUSTOMER){
			return $customer_mail;
		}
		return array_merge($customer_mail, $list, $extern);
	}

	/**
	 * @param int $group
	 * @return string[]
	 */
	private function getEmailsCustomer(int $group): array{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return [];
		}
		$filterarr = [];

		$filtera = $this->Model->groups[$group - 1]->getFilter();
		if($filtera){
			foreach($filtera as $k => $filter){
				$filterarr[] = ($k ? (' ' . $filter['logic'] . ' ') : ' ') . $this->getFilterSQL($filter);
			}
		}

		$filtersql = implode(' ', $filterarr);

		$customers = ($this->Model->groups[$group - 1]->isSendAll() ?
			'SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE ' . ($filtersql !== '' ? $filtersql : 1) :
			implode(',', array_map('intval', $this->Model->groups[$group - 1]->getCustomers())));

		if(!empty($customers) || !empty($filtersql)){ //Fix #10898
			$email_field = (string) $this->settings['customer_email_field'];
			$ret = $email_field ? $this->db->getAllq('SELECT ' . $email_field . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . $customers . ') AND ' . $email_field . '!="" ' . ($filtersql ? ' AND (' . $filtersql . ')' : ''), true) : [
				];
			'@phan-var string[] $ret';
			return $ret;
		}
		return [];
	}

	/**
	 * @param int $group
	 * @return array<int,array{string,string,string,string,string,string,string,int[],int,string}>
	 */
	private function getEmailsCustomerCSV(int $group): array{
		if(!we_modules_info::isActive(we_modules_info::CUSTOMER)){
			return [];
		}
		$customer_mail = $filterarr = [];

		$filtera = $this->Model->groups[$group - 1]->getFilter();
		if($filtera){
			foreach($filtera as $k => $filter){
				$filterarr[] = ($k ? (' ' . $filter['logic'] . ' ') : ' ') . $this->getFilterSQL($filter);
			}
		}

		$filtersql = implode(' ', $filterarr);

		$customers = ($this->Model->groups[$group - 1]->isSendAll() ?
			'SELECT ID FROM ' . CUSTOMER_TABLE . ' WHERE ' . ($filtersql !== '' ? $filtersql : 1) :
			implode(',', array_map('intval', $this->Model->groups[$group - 1]->getCustomers())));

		if(!empty($customers) || !empty($filtersql)){ //Fix #10898
			$email_field = (string) $this->settings['customer_email_field'];


			$default_html = fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter" AND pref_name="default_htmlmail"');
			$html_field = (string) ($this->settings['customer_html_field'] === 'ID' ? '' : $this->settings['customer_html_field']);
			$salutation_field = (string) ($this->settings['customer_salutation_field'] === 'ID' ? '' : $this->settings['customer_salutation_field']);
			$title_field = (string) ($this->settings['customer_title_field'] === 'ID' ? '' : $this->settings['customer_title_field']);
			$firstname_field = (string) $this->settings['customer_firstname_field'];
			$lastname_field = (string) $this->settings['customer_lastname_field'];

			$selectX = implode(',', array_filter(array_unique([
				$email_field,
				$html_field,
				$salutation_field,
				$title_field,
				$firstname_field,
				$lastname_field
			])));
			foreach($this->db->getAllq('SELECT ID,' . $this->db->escape($selectX) . ' FROM ' . CUSTOMER_TABLE . ' WHERE ID IN(' . $customers . ') AND ' . $email_field . '!="" ' . ($filtersql ? ' AND (' . $filtersql . ')' : '')) as $rec){

				$customer_mail[] = [
					trim($rec[$email_field] ?? ''),
					($html_field && ($tmp = trim($rec[$html_field])) ? $tmp : $default_html),
					($salutation_field ? $rec[$salutation_field] : ''),
					($title_field ? $rec[$title_field] : ''),
					$rec[$firstname_field] ?? '',
					$rec[$lastname_field] ?? '',
					(string) $group,
					$this->getGroupBlocks($group),
					$rec['ID'],
					'customer'
				];
			}
		}
		return $customer_mail;
	}

	function getEmailsNum(): int{
		$out = 0;
		$count = count($this->Model->groups);
		for($i = 0; $i < $count; $i++){
			$out += count($this->getEmails($i + 1, self::MAILS_ALL));
		}
		return $out;
	}

	/**
	 * Static function - Settings
	 */
	static function getSettings(): array{
		$db = new DB_WE();
		$domainName = str_replace('www.', '', $_SERVER['SERVER_NAME']);

		$ret = array_merge([
			'black_list' => '',
			'customer_email_field' => '',
			'customer_firstname_field' => 'Forename',
			'customer_html_field' => 'ID',
			'customer_lastname_field' => 'Surname',
			'customer_salutation_field' => '',
			'customer_title_field' => '',
			'default_htmlmail' => 0,
			'isEmbedImages' => false,
			'default_reply' => 'replay@' . $domainName,
			'default_sender' => 'mailer@' . $domainName,
			we_newsletter_newsletter::FEMALE_SALUTATION_FIELD => g_l('modules_newsletter', '[default][female]'),
			'global_mailing_list' => '',
			'log_sending' => 1,
			we_newsletter_newsletter::MALE_SALUTATION_FIELD => g_l('modules_newsletter', '[default][male]'),
			'reject_malformed' => 1,
			'reject_not_verified' => 1,
			'send_step' => 20,
			'send_wait' => 0,
			'test_account' => 'test@' . $domainName,
			'title_or_salutation' => 0,
			'use_port' => 0,
			'use_https_refer' => 0,
			'use_base_href' => 1
			], $db->getAllFirstq('SELECT pref_name,pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="newsletter"', false));
		//make sure blacklist is correct
		$ret['black_list'] = implode(',', we_base_request::filterStringList($ret['black_list']));

		return $ret;
	}

	private function saveSettings(): bool{
		$success = true;
		foreach($this->settings as $key => $value){
			if($key != 'black_list'){
				$success = $success && $this->saveSetting($key, (string) $value);
			}
		}

		return $success;
	}

	public function saveSetting(string $name, string $value): bool{
		return DB_WE::$inst->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET ' . we_database_base::arraySetter([
					'tool' => 'newsletter',
					'pref_name' => $name,
					'pref_value' => $value
		]));
	}

	function isBlack(string $email): bool{
		static $black = [];
		if(!$black){
			if(empty($this->settings['black_list'])){
				return false;
			}
			//remove whitespaces
			$black = explode(',', strtolower($this->settings['black_list']));
			foreach($black as &$b){
				$b = trim($b, " \t\n\r\n"); //intentionally duplicate \n!
			}
		}
		return in_array(trim(strtolower($email), " \t\n\r\n"), $black, true);
	}

	/**
	 * Write newsletter and mailing lists temp files
	 *
	 * @param Integer $nid
	 * @param Boolean $cachemails
	 * @return Array
	 */
	function cacheNewsletter(int $nid = 0, bool $cachemails = true): array{
		we_tag_sendMail::$active = true;
		$ret = [];
		if($nid){
			$this->Model = new we_newsletter_newsletter($nid);
		}
		$buffer = [];
		'@phan-var array<string,array{string,string,string,string,string,string,string,string[]}> $buffer';

		if($cachemails){
			// BEGIN cache emails groups
			$emailcache = we_base_file::getUniqueId();
			$groupcount = count($this->Model->groups);
			$ret['emailcache'] = $emailcache;

			for($groupid = 0; $groupid < $groupcount; $groupid++){
				$tmp = $this->getEmailsCSV($groupid + 1);
				foreach($tmp as $curEntry){
					'@phan-var array{string,string,string,string,string,string,string,string[]} $curEntry';
					if(empty($curEntry[0]) || empty($curEntry[7])){//0=mail,7=block
						continue;
					}
					$mailaddress = strtolower($curEntry[0]);
					if(!isset($buffer[$mailaddress])){
						$buffer[$mailaddress] = $curEntry;
						continue;
					}

					if(!in_array($curEntry[6], ($groups = explode(',', $buffer[$mailaddress][6])))){
						$groups[] = $curEntry[6];
						$buffer[$mailaddress][6] = implode(',', $groups);
					}
					$buffer[$mailaddress][7] = array_merge($buffer[$mailaddress][7], $curEntry[7]);
				}
			}

			//$buffer = array_values($buffer);

			$ret['ecount'] = count($buffer);

			$groups = 0;
			$go = true;
			$offset = 0;

			while($go){
				$tmp = array_slice($buffer, $offset, (int) $this->settings['send_step']);
				if(!empty($tmp)){
					$offset += (int) $this->settings['send_step'];
					$groups++;
					we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $emailcache . '_' . $groups, $tmp, we_base_util::SECONDS_DAY);
				} else {
					$go = false;
				}
			}

			$ret['gcount'] = $groups + 1;
		}

		// END cache emails groups
		// BEGIN cache newlsetter blocks
		$blockcache = we_base_file::getUniqueId();

		$ret['blockcache'] = $blockcache;

		foreach($this->Model->blocks as $blockid => $block){
			we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_p_' . $blockid, [
				'defaultC' => $this->getContent($block, 0, false, '', '', '', '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'femaleC' => $this->getContent($block, 0, false, (string) $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'maleC' => $this->getContent($block, 0, false, (string) $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_firstname_lastnameC' => $this->getContent($block, 0, false, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_lastnameC' => $this->getContent($block, 0, false, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstname_lastnameC' => $this->getContent($block, 0, false, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstnameC' => $this->getContent($block, 0, false, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'lastnameC' => $this->getContent($block, 0, false, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'default' => $this->getContent($block, 0, false, '', '', '', '', ''),
				'female' => $this->getContent($block, 0, false, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'male' => $this->getContent($block, 0, false, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_firstname_lastname' => $this->getContent($block, 0, false, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_lastname' => $this->getContent($block, 0, false, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname_lastname' => $this->getContent($block, 0, false, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname' => $this->getContent($block, 0, false, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', ''),
				'lastname' => $this->getContent($block, 0, false, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				], we_base_util::SECONDS_DAY);

			we_cache_file::save(we_cache_file::DOMAIN_NEWSLETTER . $blockcache . '_h_' . $blockid, [
				'defaultC' => $this->getContent($block, 0, true, '', '', '', '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'femaleC' => $this->getContent($block, 0, true, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'maleC' => $this->getContent($block, 0, true, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_firstname_lastnameC' => $this->getContent($block, 0, true, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'title_lastnameC' => $this->getContent($block, 0, true, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstname_lastnameC' => $this->getContent($block, 0, true, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'firstnameC' => $this->getContent($block, 0, true, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'lastnameC' => $this->getContent($block, 0, true, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, we_newsletter_base::CUSTOMERID_REPLACE_TEXT),
				'default' => $this->getContent($block, 0, true, '', '', '', '', ''),
				'female' => $this->getContent($block, 0, true, $this->settings[we_newsletter_newsletter::FEMALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'male' => $this->getContent($block, 0, true, $this->settings[we_newsletter_newsletter::MALE_SALUTATION_FIELD], we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_firstname_lastname' => $this->getContent($block, 0, true, '', we_newsletter_base::TITLE_REPLACE_TEXT, we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'title_lastname' => $this->getContent($block, 0, true, '', we_newsletter_base::TITLE_REPLACE_TEXT, '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname_lastname' => $this->getContent($block, 0, true, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				'firstname' => $this->getContent($block, 0, true, '', '', we_newsletter_base::FIRSTNAME_REPLACE_TEXT, '', ''),
				'lastname' => $this->getContent($block, 0, true, '', '', '', we_newsletter_base::LASTNAME_REPLACE_TEXT, ''),
				], we_base_util::SECONDS_DAY);
		}
		// END cache newlsetter blocks

		return $ret;
	}

	function getCleanMail(string $mail): string{
		$match = [];
		$pattern = '|[_\.0-9a-z-]+@([0-9a-z-]+\.)+[a-z]{2,6}|i';
		if(preg_match($pattern, $mail, $match)){
			return ($match[0]);
		}
		return '';
	}

	public function getShowImportBox(): int{
		return $this->show_import_box;
	}

	public function getShowExportBox(): int{
		return $this->show_export_box;
	}

	public function getHomeScreen(): string{
		$content = we_html_button::create_button('new_newsletter', "javascript:we_cmd('new_newsletter');", '', '', !we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW])) .
			'<br/>' . we_html_button::create_button('new_newsletter_group', "javascript:we_cmd('new_newsletter_group');", '', '', !we_base_permission::hasPerm([we_base_perm::NEWSLETTER_NEW]));

		return parent::getActualHomeScreen(we_modules_info::NEWSLETTER, $content, we_html_element::form(['name' => we_base_constants::WE_FORM], $this->getHiddens(['ncmd' => 'home']) . we_html_element::hidden('home', 0)));
	}

	private static function we_getObjectFileByID(int $id, string $includepath = ''): string{
		$mydoc = new we_contents_objectFile($id);
		return $mydoc->getDocumentData($includepath);
	}

	/*
	 * Grafische Aufbereitung des Versandlogs im Tab "Auswertung"
	 */

	function getHTMLReporting(): we_html_multiIconBoxContent{
		$all = $this->db->getAllq('SELECT Log,stamp,DATE_FORMAT(stamp,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS LogTime FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ' AND Log IN(\'' . we_newsletter_newsletter::LOG_START_SEND . '\', \'' . we_newsletter_newsletter::LOG_END_SEND . '\') ORDER BY stamp ASC');

		$newsletterMailOrders = [];
		$newsletterMailOrdersCnt = 0;

		foreach($all as $rec){
			switch($rec['Log']){
				case we_newsletter_newsletter::LOG_START_SEND:
					$newsletterMailOrders[++$newsletterMailOrdersCnt]['start_send'] = $rec['stamp'];
					$newsletterMailOrders[$newsletterMailOrdersCnt]['startTime'] = $rec['LogTime'];
					break;
				case we_newsletter_newsletter::LOG_END_SEND:
					$newsletterMailOrders[$newsletterMailOrdersCnt]['end_send'] = $rec['stamp'];
			}
		}

		$parts = new we_html_multiIconBoxContent();

		foreach($newsletterMailOrders as $key => $newsletterMailOrder){
			$results = $this->db->getAllFirstq('SELECT Log,COUNT(1) FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ' AND Log NOT IN (\'' . we_newsletter_newsletter::LOG_START_SEND . '\',\'' . we_newsletter_newsletter::LOG_END_SEND . '\') AND stamp BETWEEN "' . $newsletterMailOrder['start_send'] . '" AND "' . ($newsletterMailOrder['end_send'] ?? 'NOW()') . '" GROUP BY Log', false);
			'@phan-var array<string,int> $results';

			$allRecipients = (int) array_sum($results);

			/* process bar blocked by blacklist */
			$allBlockedByBlacklist = (array_key_exists(we_newsletter_newsletter::LOG_MAIL_BLACK, $results) ? $results[we_newsletter_newsletter::LOG_MAIL_BLACK] : 0);
			$percentBlockedByBlacklist = we_base_util::getPercent($allRecipients, $allBlockedByBlacklist, 2);

			$pbByB = new we_gui_progressBar($percentBlockedByBlacklist, 150, 'blacklist' . $key);

			/* process bar blocked by domain check */
			$allBlockedByDomainCheck = (array_key_exists(we_newsletter_newsletter::LOG_DOMAIN_NOK, $results) ? $results[we_newsletter_newsletter::LOG_DOMAIN_NOK] : 0);
			$percentBlockedByDomain = we_base_util::getPercent($allRecipients, $allBlockedByDomainCheck, 2);

			$pbBbD = new we_gui_progressBar($percentBlockedByDomain, 150, 'domain' . $key);


			/* process bar all clear recipients */
			$allClearRecipients = (array_key_exists(we_newsletter_newsletter::LOG_MAIL_SENT, $results) ? $results[we_newsletter_newsletter::LOG_MAIL_SENT] : 0);
			$percentClearRecipients = we_base_util::getPercent($allRecipients, $allClearRecipients, 2);

			$pbCR = new we_gui_progressBar($percentClearRecipients, 150, 'recipients' . $key);

			$table = (new we_html_table(['class' => 'defaultfont', 'style' => 'width: 588px'], 1, 5))
				->setCol(1, 0, we_html_element::span(['id' => 'blacklist_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_are_black]')))
				->setCol(1, 1, $pbByB->getHTML())
				->setCol(1, 2, we_html_element::span(['id' => 'blacklist_total', 'style' => 'color:' . (($allBlockedByBlacklist > 0) ? 'red' : 'green') . ';'], (string) $allBlockedByBlacklist), [
					'style' => 'padding: 0 5px;'])
				->setCol(1, 3, '<i class="fa fa-lg ' . ($allBlockedByBlacklist == 0 ? "fa-check fa-ok" : "fa-close fa-cancel") . '"></i>', ['style' => 'padding: 0 5px;'])
				->setCol(1, 4, (($allBlockedByBlacklist == 0) ? '' : we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:top.opener.top.we_cmd('black_list');"))), [
					'style' => 'width: 35px'])
				->setCol(2, 0, we_html_element::span(['id' => 'domain_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_nok]')))
				->setCol(2, 1, $pbBbD->getHTML())
				->setCol(2, 2, we_html_element::span(['id' => 'domain_total', 'style' => 'color:' . (($allBlockedByDomainCheck > 0) ? 'red' : 'green') . ';'], (string) ($allBlockedByDomainCheck)), [
					'style' => "padding: 0 5px;"])
				->setCol(2, 3, '<i class="fa fa-lg ' . ($allBlockedByDomainCheck == 0 ? "fa-check fa-ok" : "fa-close fa-cancel") . '"></i>', ['style' => "padding: 0 5px;"])
				->setCol(2, 4, (($allBlockedByDomainCheck == 0) ? '' : we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:top.opener.top.we_cmd('domain_check');"))), [
					'style' => 'width: 35px'])
				->setCol(3, 0, we_html_element::span(['id' => 'recipients_' . $key], g_l('modules_newsletter', '[reporting][mailing_emails_success]')))
				->setCol(3, 1, $pbCR->getHTML())
				->setCol(3, 2, we_html_element::span(['id' => 'recipients_total', 'style' => 'color:' . (($allClearRecipients <= 0) ? 'red' : 'green') . ';'], (string) ($allClearRecipients)), [
					'style' => 'padding: 0 5px;'])
				->setCol(3, 3, '<i class="fa fa-lg ' . ($allClearRecipients == $allRecipients ? "fa-check fa-ok" : "fa-exclamation-triangle fa-cancel") . '" title="' . ($allClearRecipients < $allRecipients ? g_l('modules_newsletter', '[reporting][mailing_advice_not_success]') : '') . '"></i>', [
					'style' => 'padding: 0 5px;'])
				/* total recipients */
				->setCol(3, 4, we_html_button::formatButtons(we_html_button::create_button(we_html_button::VIEW, "javascript:we_cmd('show_log','mail_sent','" . $newsletterMailOrder['start_send'] . "','" . $newsletterMailOrder['end_send'] . "')")), [
					'style' => 'width: 35px'])
				/* total recipients */
				->setCol(4, 0, we_html_element::b(g_l('modules_newsletter', '[reporting][mailing_all_emails]')))
				->setCol(4, 2, we_html_element::b((string) ($allRecipients)), ['colspan' => '2', 'style' => 'padding: 0 5px;']);

			$parts->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[reporting][mailing_send_at]') . '&nbsp;' . $newsletterMailOrder['startTime'], $table->getHTML() . we_html_element::br()));
		}

		return $parts->isEmpty() ? $parts->add(new we_html_multiIconBoxContentLine(g_l('modules_newsletter', '[reporting][mailing_not_done]'), '')) : $parts;
	}

	public function getHTMLLog(string $status, ?DateTime $start = null, ?DateTime $end = null): string{
		$all = $this->db->getAllq('SELECT Log,Param,DATE_FORMAT(stamp,"' . g_l('weEditorInfo', '[mysql_date_format]') . '") AS LogTime FROM ' . NEWSLETTER_LOG_TABLE . ' WHERE NewsletterID=' . $this->Model->ID . ($status ? ' AND Log=\'' . $status . '\'' : '') . ($start && $end ? ' AND (stamp BETWEEN \'' . $start->format(we_database_base::DATE_TIME_FORMAT) . '\' AND \'' . $end->format(we_database_base::DATE_TIME_FORMAT) . '\')' : '') . ' ORDER BY stamp DESC');
		$content = '';
		foreach($all as $rec){
			$log = g_l('modules_newsletter', '[' . $rec['Log'] . ']');
			$param = $rec['Param'];
			$content .= we_html_element::div(['class' => 'defaultfont'], $rec['LogTime'] . '&nbsp;' . ($param ? sprintf($log, $param) : $log));
		}
		return $content;
	}

}
