<?php

declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 68b2142cf321 $
 * $Author: Marc Kraemer <marc.kraemer@webedition.org> $
 * $Date: 2020-11-26 11:47 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_rpcCmd_treeLoadMainTree extends we_rpc_cmd{

	private function getItems(array $openFolders, array $parentpaths, string $wsQuery, array &$treeItems, string $table, int $ParentID, int $offset = 0, int $segment = 0, array $collectionIDs = [
			], array $collections = [
			]): void{
		if(($table === TEMPLATES_TABLE && !we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE])) ||
				($table === FILE_TABLE && !we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])) ||
				($table === VFILE_TABLE && !we_base_permission::hasPerm([we_base_perm::WORK_COLLECTIONS_CAN_SEE]))){
			return;
		}

		$DB_WE = new DB_WE();
		$docCollections = $docCollectionIDs = $objectCollections = $objCollectionIDs = [];
		if($table === VFILE_TABLE){// TODO: permision
			//FIXME: check this query
			$all = $DB_WE->getAllq('SELECT ID,remObj,remTable,position FROM ' . FILELINK_TABLE . ' WHERE type="collection" ORDER BY ID,position ASC');

			foreach($all as $rec){
				if($rec['remTable'] === stripTblPrefix(FILE_TABLE)){
					$docCollections[$rec['ID']] = $docCollections[$rec['ID']] ?? [];
					$docCollections[$rec['ID']][$rec['position']] = $rec['remObj'];
					$docCollectionIDs[] = $rec['remObj'];
				} else {
					$objectCollections[$rec['ID']] = $objectCollections[$rec['ID']] ?? [];
					$objectCollections[$rec['ID']][$rec['position']] = $rec['remObj'];
					$objCollectionIDs[] = $rec['remObj'];
				}
			}
		}

		$prevoffset = max(0, $offset - $segment);
		if($offset && $segment){
			$treeItems[] = [
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => we_tree_base::CT_UP,
				'isclassfolder' => 0,
				'isprotected' => 0,
				'table' => $table,
				'checked' => 0,
				'typ' => we_tree_base::TYP_DOTS,
				'open' => 0,
				'published' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $prevoffset
			];
		}

		$tmp = array_filter($openFolders);
		$tmp[] = $ParentID;
		$elem = 'f.ID,f.ParentID,f.Path,f.Text,f.IsFolder,f.ContentType,f.Mod_Date';
		$queryTable = $table . ' f ';
		switch($table){
			case FILE_TABLE:
				$elem .= ',f.IsProtected,f.Extension,IF(f.IsPublished=1 && f.Publish_Date<f.Mod_Date,-1,f.IsPublished) AS IsPublished';
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$elem .= ',st.DID IS NOT NULL AS inSchedule';
					$queryTable .= ' LEFT JOIN ' . SCHEDULE_TABLE . ' st ON (st.DID=ID AND st.documentTable="' . stripTblPrefix(FILE_TABLE) . '" AND st.Active=1)';
				}
				break;
			case OBJECT_FILES_TABLE:
				$elem .= ',f.IsClassFolder,IF(f.IsPublished=1 && f.Publish_Date<f.Mod_Date,-1,f.IsPublished) AS IsPublished,f.TableID';
				if(we_modules_info::isActive(we_modules_info::SCHEDULER)){
					$elem .= ',st.DID IS NOT NULL AS inSchedule';
					$queryTable .= ' LEFT JOIN ' . SCHEDULE_TABLE . ' st ON (st.DID=ID AND st.documentTable="' . stripTblPrefix(OBJECT_FILES_TABLE) . '" AND st.Active=1)';
				}
				break;
			case TEMPLATES_TABLE:
				$elem .= ',f.Extension,1 AS IsPublished';
				break;
			case VFILE_TABLE:
				$elem .= ',f.remTable,1 AS IsPublished,1 AS IsFolder';
				break;
			default:
				$elem .= ',1 AS IsPublished';
				break;
		}

		$tmpItems = [];

		$where = $collectionIDs ? ' WHERE f.ID IN(' . implode(',', $collectionIDs) . ') AND f.IsFolder=0 AND ((1' . we_users_util::makeOwnersSql() . ') ' . $wsQuery . ')' :
				' WHERE f.ID!=' . $ParentID . ' AND f.ParentID IN(' . implode(',', $tmp) . ') AND ((1' . we_users_util::makeOwnersSql() . ') ' . $wsQuery . ')';

		foreach($DB_WE->getAllq('SELECT ' . $elem . ' FROM ' . $queryTable . ' ' . $where . ' ORDER BY f.IsFolder DESC,(f.Text REGEXP "^[0-9]") DESC,ABS(REPLACE(f.Text,"info","")),f.Text' . ($segment ? ' LIMIT ' . $offset . ',' . $segment : '')) as $rec){

			$ID = (int) $rec['ID'];
			$Path = $rec['Path'];

			$tmpItems[$ID] = [
				'id' => $ID,
				'we_id' => $collectionIDs ? $ID : 0,
				'iscollectionitem' => $collectionIDs ? 1 : 0,
				'parentid' => $rec['ParentID'],
				'text' => $rec['Text'],
				'contenttype' => $rec['ContentType'],
				'isclassfolder' => (int) ($rec['IsClassFolder'] ?? 0),
				'isprotected' => (int) ($rec['IsProtected'] ?? 0),
				'table' => $table,
				'checked' => 0,
				'typ' => $rec['IsFolder'] ? we_tree_base::TYP_FOLDER : we_tree_base::TYP_ITEM,
				'open' => (in_array($ID, $openFolders) ? 1 : 0),
				'published' => $rec['IsFolder'] ? 1 : $rec['IsPublished'],
				'disabled' => (in_array($Path, $parentpaths) ? 1 : 0),
				'tooltip' => $ID,
				'inSchedule' => (int) ($rec['inSchedule'] ?? 0),
				'offset' => $offset,
				'path' => $rec['Path'],
				'tableid' => (int) ($rec['TableID'] ?? 0),
			];
		}

		if($collectionIDs){
			foreach($collections as $collectionID => $items){
				$i = 0;
				foreach($items as $itemID){
					if(isset($tmpItems[$itemID])){
						$tmpItems[$itemID]['parentid'] = $collectionID;
						$tmpItems[$itemID]['id'] = $collectionID . '_' . $i++ . '_' . $itemID;
						$treeItems[] = $tmpItems[$itemID];
					}
				}
			}
		} else {
			$treeItems = array_merge($treeItems, $tmpItems);
		}

		if($table === VFILE_TABLE){
			$docCollectionIDs = array_unique($docCollectionIDs);
			if($docCollectionIDs){
				$this->getItems($openFolders, $parentpaths, $wsQuery, $treeItems, FILE_TABLE, 0, 0, 0, $docCollectionIDs, $docCollections);
			}
			$objCollectionIDs = array_unique($objCollectionIDs);
			if($objCollectionIDs){
				$this->getItems($openFolders, $parentpaths, $wsQuery, $treeItems, OBJECT_FILES_TABLE, 0, 0, 0, $objCollectionIDs, $objectCollections);
			}
		}

		$total = fI('SELECT COUNT(1) FROM ' . $table . ' f ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && $total > $nextoffset){
			$treeItems[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => we_tree_base::CT_DOWN,
				'isclassfolder' => 0,
				'isprotected' => 0,
				'table' => $table,
				'checked' => 0,
				'typ' => we_tree_base::TYP_DOTS,
				'open' => 0,
				'published' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}
	}

	function execute(): we_rpc_response{
		$table = we_base_request::_(we_base_request::TABLE, 'we_cmd', FILE_TABLE, 1);
		$parentFolder = we_base_request::int('we_cmd', 0, 2);
		$offset = we_base_request::int('we_cmd', 0, 6);

		switch(we_base_request::string('we_cmd', '', 0)){
			case 'closeFolder':
				if(empty($_SESSION['prefs']['openFolders_' . stripTblPrefix($table)])){
					$_SESSION['prefs']['openFolders_' . stripTblPrefix($table)] = [];
					break;
				}
				$openDirs = array_flip($_SESSION['prefs']['openFolders_' . stripTblPrefix($table)]);
				unset($openDirs[$parentFolder]);
				$openDirs = array_keys($openDirs);
				$_SESSION['prefs']['openFolders_' . stripTblPrefix($table)] = $openDirs;
				break;
			case 'load':
				// when switching away from a tree to load another one, write its openFolders to session
				if(($lastOpenFolders = we_base_request::intList('we_cmd', [], 3))){
					$_SESSION['prefs']['openFolders_' . stripTblPrefix(we_base_request::_(we_base_request::TABLE, 'we_cmd', '', 4))] = $lastOpenFolders;
				}
			/* falls through */
			case 'loadFolder':
				$parentpaths = $wspaces = [];

				$ac = ($table === OBJECT_FILES_TABLE && (!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])) ?
						we_users_util::getAllowedClasses() : []);

				if(($ws = we_users_util::get_ws($table))){
					/* FIXME can we use ?!
					 * 				$folders = getAllFoldersRecursive($ws);
					  if($folders){
					  $ws_where = 'ParentID IN(' . implode(',', $folders) . ')';
					  }
					 */
					$wsPathArray = array_map('escape_sql_query', id2pathA($ws, $table));
					if($wsPathArray){
						$wspaces[] = 'Path REGEXP "^(' . implode('|', $wsPathArray) . ')/"';
					}
					foreach($wsPathArray as $path){
						$parentpaths = array_merge($parentpaths, we_tree_base::getParents($path));
					}
				}
				$parentpaths = array_unique($parentpaths);

				if($parentpaths){
					$wspaces[] = 'Path IN(' . implode(',', $parentpaths) . ')';
				}
				$wsQuery = ($ac ? ' AND TableID IN (' . implode(',', $ac) . ')' : '') .
						($wspaces ? ' AND (' . implode(' OR ', $wspaces) . ') ' : ' OR RestrictOwners=0 ' );


				$openFolders = (!empty($_SESSION['prefs']['openFolders_' . stripTblPrefix($table)]) ? $_SESSION['prefs']['openFolders_' . stripTblPrefix($table)] : []);

				if($parentFolder){
					if(!in_array($parentFolder, $openFolders)){
						$openFolders[] = $parentFolder;
						$_SESSION['prefs']['openFolders_' . stripTblPrefix($table)] = $openFolders;
					}
				}

				if(we_editor_functions::isSeeMode()){
					return $this->resp;
				}
				$jsCmd = new we_base_jsCmd();

				$Tree = new we_tree_main($jsCmd);
				$treeItems = [];
				$this->getItems($openFolders, $parentpaths, $wsQuery, $treeItems, $table, $parentFolder, $offset, $Tree->default_segment);

				switch($table){
					case FILE_TABLE:
						$this->resp->setData('treeName', g_l('global', '[documents]'));
						break;
					case TEMPLATES_TABLE:
						$this->resp->setData('treeName', g_l('global', '[templates]'));
						break;
					case OBJECT_FILES_TABLE:
						$this->resp->setData('treeName', g_l('global', '[objects]'));
						break;
					case OBJECT_TABLE:
						$this->resp->setData('treeName', g_l('modules_object', '[menu][classes]'));
						break;
					case VFILE_TABLE:
						$this->resp->setData('treeName', g_l('global', '[vfile]'));
						break;
					default:
						$this->resp->setData('treeName', '');
				}

				$this->resp->setData('parentFolder', $parentFolder);
				$this->resp->setData('offset', $offset);
				$this->resp->setData('items', $Tree->getJSLoadTree($treeItems));
				break;
			default:
				t_e('called rpcLoadMainTreeCmd with unknwon task/we_cmd[0]');
		}
		we_users_user::writePrefs((int) $_SESSION['prefs']['userID']);

		return $this->resp;
	}

}
