<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_formmail extends we_weTag_tag{
	const WE_DEFAULT_SUBJECT = 'webEdition mailform';

	/** @var array{order?:string,required?:string,subject?:string,recipient?:string,mimetype?:string,from?:string,error_page?:int,mail_error_page?:int,recipient_error_page?:int,ok_page?:int,confirm_mail?:bool,pre_confirm?:string,post_confirm?:string,we_remove?:string[],forcefrom?:string,captcha_error_page?:int,captchaname?:string}
	 */
	private static $data = [];

	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		if(!we_base_WE::$mainDoc['doc']->InWebEdition){
			self::sendMail();
		}
		return '';
	}

	protected static function print_error(string $errortext): void{
		$headline = 'Fehler / Error';
		$content = g_l('global', '[formmailerror]') . self::htmlTag('br') . '&#8226; ' . $errortext;
		if(defined('FROM_WE_SHOW_DOC')){
			echo $content;
			return;
		}
		echo we_html_tools::getHtmlTop('', '', self::htmlTag('body', ['class' => 'weEditorBody'], '', false, true) .
			we_html_tools::dialogLayout(self::htmlTag('div', ['class' => 'defaultfont lowContrast'], $content), $headline) .
			'</body>');

		exit();
	}

	protected static function checkBlocked(): bool{
//FIXME: forms can come from static content;
		/* $formBlock = */$blocked = false; //!we_tag('ifFormToken');
// check to see if we need to lock or block the formmail request
		if(FORMMAIL_LOG){
			// insert into log
			DB_WE::$inst->query('INSERT INTO ' . FORMMAIL_LOG_TABLE . ' SET ip="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '"');
			if(FORMMAIL_EMPTYLOG > -1){
				DB_WE::$inst->query('DELETE FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime<(NOW() - INTERVAL ' . intval(FORMMAIL_EMPTYLOG) . ' SECOND)');
			}

			if(FORMMAIL_BLOCK){
				// first delete all entries from blocktable which are older then now - blocktime
				DB_WE::$inst->query('DELETE FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE blockedUntil!=-1 AND blockedUntil<UNIX_TIMESTAMP()');

				// check if ip is allready blocked
				if(fB('SELECT 1 FROM ' . FORMMAIL_BLOCK_TABLE . ' WHERE ip="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '" LIMIT 1')){
					$blocked = true;
				} else {
					// ip is not blocked, so see if we need to block it
					if(fI('SELECT COUNT(1) FROM ' . FORMMAIL_LOG_TABLE . ' WHERE unixTime>(NOW()- INTERVAL ' . intval(FORMMAIL_SPAN) . ' SECOND) AND ip="' . DB_WE::$inst->escape($_SERVER['REMOTE_ADDR']) . '"') > FORMMAIL_TRIALS){
						$blocked = true;
						// insert in block table
						DB_WE::$inst->query('REPLACE INTO ' . FORMMAIL_BLOCK_TABLE . ' SET ' . we_database_base::arraySetter(['ip' => $_SERVER['REMOTE_ADDR'],
								'blockedUntil' => (FORMMAIL_BLOCKTIME == -1 ? -1 : sql_function('(UNIX_TIMESTAMP()+' . intval(FORMMAIL_BLOCKTIME) . ')'))
						]));
					}
				}
			}
		}

		$docBlock = (FORMMAIL_VIAWEDOC && we_http_util::getScriptName() == WEBEDITION_DIR . basename(__FILE__));
		$blocked = $blocked || $docBlock;

		if($blocked){
			self::print_error('Email dispatch blocked / Email Versand blockiert' . /* ($formBlock ? ' (Token)!' : '') . */ ($docBlock ? '(Doc)' : '(Log)!'));
			return false;
		}
		return true;
	}

	protected static function setData(): bool{
		$jwt = we_base_request::string('data-jwt');
		try{
			$data = we_helpers_jwt::decode($jwt, sha1(SECURITY_ENCRYPTION_KEY));
		} catch (Exception $e){
			self::print_error('Email dispatch blocked / Email Versand blockiert (mixed)!');
			return false;
		}
		$_REQUEST = array_merge($_REQUEST, $data);
		self::$data = $data;
		return true;
	}

	protected static function contains_bad_str(string $str_to_test): bool{
		static $bad_strings = [
			'content-type:',
			'mime-version:',
			'Content-Transfer-Encoding:',
			'bcc:',
			'cc:',
			'to:',
		];

		$str_to_test = trim($str_to_test);

		foreach($bad_strings as $bad_string){
			if(preg_match('|^' . preg_quote($bad_string, '|') . '|i', $str_to_test) || preg_match('|[\n\r]' . preg_quote($bad_string, "|") . '|i', $str_to_test)){
				self::print_error('Email dispatch blocked / Email Versand blockiert (str)!');
				return true;
			}
		}
		if(stristr($str_to_test, 'multipart/mixed')){
			self::print_error('Email dispatch blocked / Email Versand blockiert (mixed)!');
			return true;
		}
		return false;
	}

	protected static function replace_bad_str(string $str_to_test): string{
		static $bad_strings = [
			'#(content-type)(:)#i',
			'#(mime-version)(:)#i',
			'#(multipart/mixed)#i',
			'#(Content-Transfer-Encoding)(:)#i',
			'#(bcc)(:)#i',
			'#(cc)(:)#i',
			'#(to)(:)#i',
		];

		return preg_replace($bad_strings, '($1)$2', $str_to_test);
	}

	protected static function check_required(string $required): bool{
		if(!$required){
			return true;
		}
		$we_requiredarray = explode(',', $required);
		foreach($we_requiredarray as $cur){
			if(!we_base_request::_(we_base_request::RAW, $cur)){
				return false;
			}
		}
		return true;
	}

	protected static function error_page(): void{
		if(($errorpage = we_base_request::int('error_page')) && ($url = we_folder_base::getUrlFromID($errorpage))){
			self::redirect($url);
		} else {
			self::print_error(g_l('global', '[email_notallfields]'));
		}
	}

	protected static function ok_page(): void{
		if(($page = we_base_request::int('ok_page')) && ($url = we_folder_base::getUrlFromID($page))){
			self::redirect($url);
			exit();
		}
		$msg = self::htmlTag('div', ['class' => 'defaultfont lowContrast'], 'Vielen Dank, Ihre Formulardaten sind bei uns angekommen! / Thank you, we received your form data!');
		if(defined('FROM_WE_SHOW_DOC')){
			echo $msg;
			return;
		}
		echo we_html_tools::getHtmlTop('', '', self::htmlTag('body', ['class' => 'weEditorBody'], '', false, true) .
			we_html_tools::dialogLayout($msg, '') .
			'</body>');
		exit();
	}

	protected static function redirect(string $url, string $emosScontact = ''): void{
		if($emosScontact != ''){
			$url = $url . (str_contains($url, '?') ? '&' : '?') . 'emosScontact=' . urlencode($emosScontact);
		}
		header('Location: ' . $url);
		exit();
	}

	protected static function check_captcha(): bool{
		$name = empty(self::$data['captchaname']) ? '' : (string) self::$data['captchaname'];
		return ($name ?
			we_captcha_captcha::check($name, 'captcha', true) :
			true); // Fix: #10297
	}

	protected static function addFiles(we_mail_mail $phpmail): bool{
		$maxFilesize = we_base_request::int('MAX_FILE_SIZE', 0);

		foreach($_FILES as $file){
			if(!empty($file['tmp_name'])){
				$tmpName = (string) $file['tmp_name'];
				$tempPath = TEMP_PATH . we_base_request::filterVar($file['name'], we_base_request::FILE); //@phan-suppress-current-line PhanDeprecatedFunction
				if($maxFilesize && filesize($tmpName) > $maxFilesize){
					self::error_page();
					return false;
				}
				move_uploaded_file($tmpName, $tempPath);
				we_base_file::insertIntoCleanUp($tempPath, 3600);
				$phpmail->doaddAttachment($tempPath);
			}
		}
		return true;
	}

	public static function sendMail(): void{
		if(!self::checkBlocked() ||
			!self::setData()){
			return;
		}

		if(!self::check_captcha()){
			if(($errorpage = we_base_request::int('captcha_error_page')) && ($url = we_folder_base::getUrlFromID($errorpage))){
				self::redirect($url);
			} else {
				self::print_error(g_l('global', '[captcha_invalid]'));
				return;
			}
		}

		if(!empty(self::$data['required']) && !self::check_required((string) self::$data['required'])){
			self::error_page();
			return;
		}

		$email = we_base_request::_(we_base_request::EMAIL, 'email');

		if(!empty($_REQUEST['email'])){//fixme: note this mail can be in "abc" <cc@de.de> format
			if(!$email){
				if(($errorpage = we_base_request::int('mail_error_page')) && ($url = we_folder_base::getUrlFromID($errorpage))){
					self::redirect($url);
				} else {
					self::print_error(g_l('global', '[email_invalid]'));
					return;
				}
			}
		}

		$output = [];

		$removeArr = empty(self::$data['we_remove']) ? [] : array_map('trim', array_filter(self::$data['we_remove']));
		$we_reserved = array_merge(['from', 'we_remove', 'captchaname', 'we_mode', 'charset', 'required', 'order', 'ok_page', 'error_page', 'captcha_error_page', 'mail_error_page',
			'recipient', 'subject', 'mimetype', 'confirm_mail', 'pre_confirm', 'post_confirm', 'MAX_FILE_SIZE', session_name(), 'cookie', 'recipient_error_page', 'forcefrom',
			'securityToken', 'data-jwt',], $removeArr);

		$we_txt = '';
		$we_html = '<table>';

		if(($order = we_base_request::string('order', ''))){
			$we_orderarray = explode(',', $order);

			foreach($we_orderarray as $cur){
				if(!in_array($cur, $we_reserved, true)){
					$output[$cur] = we_base_request::_(we_base_request::RAW, $cur);
				}
			}
		} else {
			$we_orderarray = [];
		}

		if(isset($_POST)){
			foreach($_POST as $n => $v){
				if((!in_array($n, $we_reserved, true)) && (!in_array($n, $we_orderarray)) && (!is_array($v))){
					if(!(isset($_COOKIE[$n]) && $_COOKIE[$n] == $v)){//for some reason cookies are transfered as POST's, so filter them, if the data matches in case the field names are the same.
						$output[$n] = $v;
					}
				}
			}
		}

		foreach($output as $n => $v){
			if(is_array($v)){
				foreach($v as $n2 => $v2){
					if(!is_array($v2)){
						$foo = (string) (is_string($v2) ? self::replace_bad_str($v2) : $v2);
						$n = (string) (is_string($n) ? self::replace_bad_str($n) : $n);
						$n2 = (string) (is_string($n2) ? self::replace_bad_str($n2) : $n2);
						$we_txt .= $n . '[' . $n2 . ']: ' . $foo . "\n" . ($foo ? '' : "\n");
						$we_html .= '<tr><td style="text-align:right"><b>' . $n . '[' . $n2 . ']:</b></td><td>' . $foo . '</td></tr>';
					}
				}
			} else {
				$foo = self::replace_bad_str((string) $v);
				$n = self::replace_bad_str((string) $n);
				$we_txt .= $n . ': ' . $foo . "\n" . ($foo ? '' : "\n");
				$we_html .= '<tr><td style="vertical-align:top;text-align:right"><b>' . $n . ':</b></td><td>' . ($n === 'email' ? '<a href="mailto:' . $foo . '">' . $foo . '</a>' : nl2br($foo)) . '</td></tr>';
			}
		}

		$we_html .= '</table>';


		$we_html_confirm = '';
		$we_txt_confirm = '';

		$useConfirmMail = we_base_request::bool('confirm_mail');

		if($email){
			if($useConfirmMail){
				$we_html_confirm = $we_html;
				$we_txt_confirm = $we_txt;
				if(!empty(self::$data['pre_confirm'])){
					$pre = self::$data['pre_confirm'];
					$we_html_confirm = $pre . self::htmlTag('br') . $we_html_confirm;
					$we_txt_confirm = we_mail_mail::getTextContent($pre) . "\n\n" . $we_txt_confirm;
				}
				if(!empty(self::$data['post_confirm'])){
					$post = self::$data['post_confirm'];
					$we_html_confirm = $we_html_confirm . self::htmlTag('br') . $post;
					$we_txt_confirm = $we_txt_confirm . "\n\n" . we_mail_mail::getTextContent($post);
				}
			}
		}

		$from = we_base_request::_(we_base_request::EMAIL, 'from', WE_DEFAULT_EMAIL);
		$email = $email ?: $from;
		$subject = preg_replace("/(\\n+|\\r+)/", '', we_base_request::string('subject', self::WE_DEFAULT_SUBJECT));
		$recipients = implode(',', we_base_request::intList('recipient'));
		$mimetype = we_base_request::string('mimetype', '');
		$fromMail = (we_base_request::bool('forcefrom') ? $from : $email);

		$wasSent = false;

		if(!$recipients || !($recipientsList = DB_WE::$inst->getAllq('SELECT Email FROM ' . RECIPIENTS_TABLE . ' WHERE ID IN (' . $recipients . ')', true))){
			self::print_error(g_l('global', '[email_no_recipient]'));
			return;
		}
		'@phan-var string[] $recipientsList';

		if(self::contains_bad_str($email) ||
			self::contains_bad_str($from) ||
			self::contains_bad_str($subject)){
			return;
		}

		if(!we_check_email($fromMail)){
			self::print_error(g_l('global', '[email_invalid]'));
			return;
		}

		$senderForename = we_base_request::string('forename', '');
		$senderSurname = we_base_request::string('surname', '');

		$phpmail = new we_mail_mail([], $subject, new we_mail_address($fromMail, trim(implode(' ', [$senderForename, $senderSurname]))));

		if(!self::addFiles($phpmail)){
			return;
		}

		$phpmail->addAddressList($recipientsList);
		if($mimetype === we_mail_mime::TYPE_HTML){
			$phpmail->addHTMLPart($we_html);
		} else {
			$phpmail->addTextPart($we_txt);
		}
		$phpmail->buildMessage();
		if($phpmail->send()){
			$wasSent = true;
		}

		if($useConfirmMail && FORMMAIL_CONFIRM){
			if($wasSent){
				// validation
				if(!we_check_email($email) || !($to = we_mail_address::parse($email))){
					self::print_error(g_l('global', '[email_invalid]'));
					return;
				}
				$phpmail = new we_mail_mail([$to], $subject, we_mail_address::parse($from));
				if($mimetype === we_mail_mime::TYPE_HTML){
					$phpmail->addHTMLPart($we_html_confirm);
				} else {
					$phpmail->addTextPart($we_txt_confirm);
				}
				$phpmail->buildMessage();
				$phpmail->send();
			}
		}

		self::ok_page();
	}

}
