<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: adcffcbe0016 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-09-02 01:16 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifDoctype extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		self::attributeMissing($attribs, ['doctypes' => false], __CLASS__);
		$matchArr = self::getAttribute('doctypes', $attribs, [], we_base_request::STRING_LIST);
		$docAttr = self::getAttribute('doc', $attribs, 'self', we_base_request::STRING);
		$doctypeids = self::getAttribute('doctypeIDs', $attribs, [], we_base_request::INTLISTA);

		switch($docAttr){
			case 'listview':
				$doctype = (we_tag_listview::$current ? we_tag_listview::$current->f(we_listview_base::DOCTYPE_FIELD) : 0);
				break;
			default:
				$doc = self::getDocForTag($docAttr);
				$doctype = ($doc instanceof we_document_webEdition) ? $doc->DocType : 0;
		}
		if(!$doctype || (!$matchArr && !$doctypeids)){
			return false;
		}
		$or = array_filter([
			($matchArr ? 'Text IN("' . implode('","', array_map('escape_sql_query', $matchArr)) . '")' : ''),
			($doctypeids ? 'ID IN(' . implode(',', $doctypeids) . ')' : '')
		]);
		return fB('SELECT 1
FROM ' .
			DOC_TYPES_TABLE . '
WHERE
	ID=' . $doctype . ' AND (' .
			implode(' OR ', $or) .
			')'
		);
	}

}
