<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifField extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);

		switch($type){
			case 'shopCategory':
				$attribs['lvOnly'] = true;
				return we_shop_category::isCategoryMode() ? (bool) we_weTag_util::tag('ifShopCategory', $attribs) : false;
			case 'shopVat':
				if(we_shop_category::isCategoryMode()){
					$attribs['lvOnly'] = true;
					return (bool) we_weTag_util::tag('ifShopVat', $attribs);
				}
				break;
			case 'float':
			case 'int':
				$attribs['type'] = 'text'; //Bug #4815
		}

		self::attributeMissing($attribs, ['name' => false, 'match' => true], __CLASS__);
		$attribs['name'] = $attribs['_name_orig'];

		$realvalue = we_weTag_util::tag('field', $attribs);
		$match = self::getAttribute('match', $attribs, '', we_base_request::STRING);

		switch(self::getAttribute('operator', $attribs, 'equal', we_base_request::STRING)){
			default:
			case 'equal':
				return $realvalue == $match;
			case 'less':
				return intval($realvalue) < intval($match);
			case 'less|equal':
				return intval($realvalue) <= intval($match);
			case 'greater':
				return intval($realvalue) > intval($match);
			case 'greater|equal':
				return intval($realvalue) >= intval($match);
			case 'contains':
				return (str_contains((string) $realvalue, $match)); //@phan-suppress-current-line PhanPluginSuspiciousParamOrderInternal
			case 'isin':
				return (str_contains(',' . $match . ',', ',' . $realvalue . ','));
		}
	}

}
