<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_ifFieldEmpty extends we_weTag_ifTag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): bool{
		if(empty($attribs['match'])){
			self::attributeMissing($attribs, ['name' => false], __CLASS__);
		}
		self::$doc = we_base_WE::getWeDoc();

		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$orig_name = self::getAttribute('_name_orig', $attribs, '', we_base_request::STRING) ?: self::getAttribute('match', $attribs, '', we_base_request::STRING);

		if(!we_tag_listview::$current){
			echo self::parseError(g_l('parser', '[field_not_in_lv]'), __CLASS__);
			return true;
		}

		$name = (we_tag_listview::$current->f($orig_name) ? $orig_name : self::getPostName($orig_name));

		switch($type){
			case 'calendar' :
				$cal = we_tag_listview::$current->getCalendarStruct();
				if(!empty($cal)){
					if(!$cal['date'] || empty($cal['storage'])){
						return true;
					}

					switch($orig_name){
						case 'day':
							$start_date = (clone $cal['date'])->setTime(0, 0, 0);
							$end_date = (clone $cal['date'])->setTime(23, 59, 59);
							break;
						case 'month':
							$start_date = (new DateTime())->setDate($cal['date']->format('Y'), $cal['date']->format('m'), 1)->setTime(0, 0, 0);
							$end_date = (new DateTime())->setDate($cal['date']->format('Y'), $cal['date']->format('m'), $cal['numofentries'])->setTime(23, 59, 59);
							break;
						case 'year':
							$start_date = (new DateTime())->setDate($cal['date']->format('Y'), 1, 1)->setTime(0, 0, 0);
							$end_date = (new DateTime())->setDate($cal['date']->format('Y'), 12, 31)->setTime(23, 59, 59);
							break;
						default:
							return true;
					}

					foreach($cal['storage'] as $entry){
						if($entry >= $start_date && $entry <= $end_date){
							return false;
						}
					}
				}
				return true;
			case 'multiobject':
				$data = we_unserialize(we_tag_listview::$current ?
					we_tag_listview::$current->f($orig_name) :
					self::$doc->getElementS($orig_name));
				$objects = array_filter($data['objects'] ?? $data);
				return empty($objects);
			case 'object' : //Bug 3837: erstmal die Klasse rausfinden um auf den Eintrag object_X zu kommen
				if(we_tag_listview::$current instanceof we_listview_document){ // listview/document with objects included using we:object
					return empty(we_tag_listview::$current->f($name));
				}
				$name = !str_contains($orig_name, '/') ? $orig_name : substr(strrchr($orig_name, '/') ?: '', 1);
				$objectid = fI('SELECT ID FROM ' . OBJECT_TABLE . ' WHERE Text="' . DB_WE::$inst->escape($name) . '"');

				return empty(we_tag_listview::$current->f(we_listview_base::OBJECT_FIELD_PREFIX . $objectid));
			case 'checkbox':
			case 'binary':
			case 'img':
			case 'collection':
			case 'video':
				return empty(we_tag_listview::$current->f($name));
			case 'float':
				return ((float) we_tag_listview::$current->f($name)) === 0.0;
			case 'int':
				return ((int) we_tag_listview::$current->f($name)) === 0;
			case 'href' :
				if(we_tag_listview::$current instanceof we_listview_objectBase){
					if(!($hrefArr = we_tag_listview::$current->f($name) ? we_unserialize(we_tag_listview::$current->f($name)) : [])){
						return true;
					}
					$hreftmp = trim(we_document_base::getHrefByArray($hrefArr));
					/**
					 * TODO: file_exists only work when the choosen webEdition document is in the correct DOCUMENT_ROOT
					 * but within multi domains we have cross DOCUMENT_ROOT references!!!
					 */
					if(!$hreftmp || $hreftmp === '/' || ($hreftmp[0] === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp)))){
						return true;
					}
					return false;
				}

				// we must check $match . we_base_link::MAGIC_INT_LINK for block-Postfix instead of $match (which exists only for href type = ext): #6422
				$isInBlock = !( we_tag_listview::$current->f($orig_name . we_base_link::MAGIC_INT_LINK) || we_tag_listview::$current->f($orig_name) );
				$name = $isInBlock ? self::getPostName($orig_name) : $orig_name;

				$int = (we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK) == '') ? 0 : we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK);
				if($int){ // for type = href int
					$intID = (int) we_tag_listview::$current->f($name . we_base_link::MAGIC_INT_LINK_ID);
					return !($intID && id2path($intID));
				}
				$hreftmp = we_tag_listview::$current->f($name);
				if($hreftmp[0] === '/' && (!file_exists(WEBEDITION_PATH . '../' . $hreftmp))){
					return true;
				}

				break; //see return of function
			default:
				$tmp = we_unserialize(we_tag_listview::$current->f($name), '', true);
				if(is_array($tmp)){
					return empty($tmp);
				}
			//no break;
		}
		return empty(we_tag_listview::$current->f($name));
	}

}
