<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 07a9a048026d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-18 00:22 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_users_util{

	/**
	 * @param int $uid
	 * @param int[] $users
	 * @return bool
	 */
	static function isUserInUsers(int $uid, array $users): bool{ // $users can be a csv string or an array
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		$users = array_map('intval', $users);

		if(in_array($uid, $users)){
			return true;
		}

		$folders = $users ? DB_WE::$inst->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE IsFolder=1 AND ID IN(' . implode(',', $users) . ')', true) : [];
		'@phan-var int[] $folders';
		if(self::isUserInGroups($uid, $folders)){
			return true;
		}

		foreach(self::getAliases($uid) as $aid){
			if(in_array((int) $aid, $users) ||
				self::isUserInGroups((int) $aid, $folders)){
				return true;
			}
		}

		return false;
	}

	/**
	 *
	 * @param int $uid
	 * @param int[] $groupIDs
	 * @return bool
	 */
	static function isUserInGroups(int $uid, array $groupIDs): bool{
		$pid = fI('SELECT ParentID FROM ' . USER_TABLE . ' WHERE ID=' . $uid);
		if(in_array($pid, $groupIDs)){
			return true;
		}
		if($pid != 0){
			return self::isUserInGroups($pid, $groupIDs);
		}
		return false;
	}

	static function addAllUsersAndGroups(int $uid, array &$arr): void{
		$data = DB_WE::$inst->getAllFirstq('SELECT ID,IsFolder FROM ' . USER_TABLE . ' WHERE ParentID=' . $uid, false);
		'@phan-var array<int,int> $data';
		foreach($data as $id => $isFolder){
			$arr[] = $id;
			if($isFolder){
				self::addAllUsersAndGroups($id, $arr);
			}
		}
	}

	static function removeNonAsociative(array &$array): array{
		foreach(array_keys($array) as $k){
			if((string) (int) $k == $k){
				unset($array[$k]);
			}
		}

		return $array;
	}

	/**
	 *
	 * @param int[] $ids
	 * @param string $wsField
	 * @return array<int,string>
	 */
	static function getUsersForDocWorkspace(array $ids, string $wsField = 'workSpaceDoc'): array{
		$where = $ids ? 'CONCAT(",",' . $wsField . ',",") REGEXP ",(' . implode('|', $ids) . '),"' : '';
		return DB_WE::$inst->getAllFirstq('SELECT ID,username FROM ' . USER_TABLE . ' WHERE ' . ($where ?: 'FALSE'), false); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	/**
	 * @param int $id
	 * @return int[]
	 */
	static function getAliases(int $id): array{
		return DB_WE::$inst->getAllq('SELECT ID FROM ' . USER_TABLE . ' WHERE Alias=' . $id, true); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	public static function isOwner(array $csvOwners): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		$ownersArray = array_filter($csvOwners);
		$uid = we_users_user::getID();
		return (in_array($uid, $ownersArray)) || self::isUserInUsers($uid, $ownersArray);
	}

	public static function userIsOwnerCreatorOfParentDir(int $folderID, string $tab): bool{
		if(($tab !== FILE_TABLE && $tab !== OBJECT_FILES_TABLE) ||
			(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || ($folderID == 0))){
			return true;
		}
		$tmp = getHash('SELECT RestrictOwners,Owners,CreatorID FROM ' . $tab . ' WHERE ID=' . $folderID);
		if(!$tmp){
			return true;
		}
		if($tmp['RestrictOwners']){
			$ownersArr = we_base_request::filterIntList($tmp['Owners']);
			foreach($ownersArr as $uid){
				we_users_util::addAllUsersAndGroups((int) $uid, $ownersArr);
			}
			$ownersArr[] = (int) $tmp['CreatorID'];
			return (in_array(we_users_user::getID(), $ownersArr));
		}
		$pid = fI('SELECT ParentID FROM ' . $tab . ' WHERE ID=' . $folderID);
		return self::userIsOwnerCreatorOfParentDir($pid, $tab);
	}

	public static function canEditModule(string $modName): bool{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return true;
		}
		$perm = we_modules_info::getModuleField($modName, 'perm');
		assert(is_array($perm));
		return we_base_permission::hasPerm($perm);
	}

	public static function makeOwnersSql(bool $useCreatorID = true): string{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		$uid = we_users_user::getID();
		$aliases = self::getAliases($uid);
		$aliases[] = $uid;

		$groups = array_merge([$uid], we_base_util::getParentIDs(USER_TABLE, $uid));
		foreach($aliases as $id){
			$groups = array_merge($groups, we_base_util::getParentIDs(USER_TABLE, (int) $id));
		}

		return ' AND ( RestrictOwners=0 OR (RestrictOwners=1 AND (
			CONCAT(",",Owners,",") REGEXP ",(' . implode('|', array_merge($aliases, $groups)) . '),"' .
			($useCreatorID ? ' OR CreatorID IN ("' . implode('","', $aliases) . '")' : '') . '))) ';
	}

	/**
	 *
	 * @return int[]
	 */
	public static function getAllowedClasses(): array{
		if(!we_modules_info::isActive(we_modules_info::OBJECT)){
			return [];
		}
		$ofWs = self::get_ws(OBJECT_FILES_TABLE);
		//return all classes if admin, or no restrictions
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) || !$ofWs){
			return DB_WE::$inst->getAllq('SELECT ID FROM ' . OBJECT_TABLE, true); //@phan-suppress-current-line PhanPartialTypeMismatchReturn
		}
//if workspace don't exist, make sure we don't show classes
		return DB_WE::$inst->getAllq('SELECT DISTINCT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE ID IN (' . implode(',', $ofWs) . ')', true) ?: [-1]; //@phan-suppress-current-line PhanPartialTypeMismatchReturn
	}

	/**
	 *
	 * @param int $ID
	 * @param int[] $wsIDs
	 * @param string $table
	 * @param bool $norootcheck
	 * @return bool
	 */
	public static function in_workspace(int $ID, array $wsIDs, string $table = FILE_TABLE, bool $norootcheck = false): bool{
		if(empty($wsIDs) || (in_array(0, $wsIDs))){
			return true;
		}

		if($ID === 0){
			return $norootcheck;
		}
		if(in_array($ID, $wsIDs)){
			return true;
		}

		$path = id2path($ID, $table) . '/';
		$paths = id2pathA($wsIDs, $table, true);
		foreach($paths as $ws){
			if(str_starts_with($path, (string) $ws)){
				return true;
			}
		}
		return false;
	}

	public static function changeRecursive(): string{
		$ok = false;

		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', 0, 1);

			// init document
			$we_doc = we_contents_root::getSessionDocument($we_transaction);

			$childs = [];
			we_base_util::pushChilds($childs, $we_doc->ID, $we_doc->Table, false);
			if($childs){
				$ok = DB_WE::$inst->query('UPDATE ' . $we_doc->Table . ' SET ' .
					we_database_base::arraySetter(['CreatorID' => $we_doc->CreatorID,
						'Owners' => $we_doc->Owners,
						'RestrictOwners' => $we_doc->RestrictOwners,
						'OwnersReadOnly' => $we_doc->OwnersReadOnly
					]) . ' WHERE ID IN(' . implode(',', $childs) . ')');
			}
		}
		$jsCmd = new we_base_jsCmd();
		$jsCmd->addMsg(g_l('modules_users', '[grant_owners_ok]'), ($ok ? we_base_util::WE_MESSAGE_NOTICE : we_base_util::WE_MESSAGE_ERROR));

		return we_html_tools::getHtmlTop('', $jsCmd->getCmds(), we_html_element::body());
	}

	public static function get_def_ws(string $table = FILE_TABLE): int{
		if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return 0;
		}

		$ws = we_users_util::get_ws($table);
		return $ws ? (int) $ws[0] : 0;
	}

	/**
	 *
	 * @param string $table
	 * @return int[]
	 */
	public static function get_ws(string $table = FILE_TABLE): array{
		if(isset($_SESSION) && isset($_SESSION['perms'])){
			if(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
				return [];
			}
			if($_SESSION['user']['workSpace'] && !empty($_SESSION['user']['workSpace'][$table])){
				return $_SESSION['user']['workSpace'][$table];
			}
		}
		return [];
	}

	public static function get_ws_paths(string $table = FILE_TABLE, bool $withParents = false): string{ //FIXME: save path-array, so we don't have to query all paths on every access
		$wspaces = $parents = [];
		$ws = self::get_ws($table);
		if(!$ws){
			return '1';
		}
		/* FIXME can we use ?!
		 * 				$folders = getAllFoldersRecursive($ws,NAVIGATION_TABLE);
		  if($folders){
		  $ws_where = 'ParentID IN(' . implode(',', $folders) . ')';
		  }
		 */

		$wsPathArray = id2pathA($ws, $table);
		if($wsPathArray){
			$wspaces[] = 'Path REGEXP "^(' . implode('|', array_map('escape_sql_query', $wsPathArray)) . ')/"';
		}
		foreach($wsPathArray as $path){
			$parents = $withParents ? array_merge($parents, we_tree_base::getParents((string) $path)) : [];
		}
		$parentpaths = array_unique($parents);
		if($parentpaths){
			$wspaces[] = 'Path IN(' . implode(',', $parentpaths) . ')';
		}

		$wspaces[] = ' ID IN(' . implode(',', $ws) . ')';
		return implode('OR', $wspaces);
	}

}
