<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * Class which contains all functions for the
 * rebuild dialog and the rebuild function
 * @static
 */
abstract class we_versions_wizard{
	const DELETE_VERSIONS = 'delete_versions';
	const RESET_VERSIONS = 'reset_versions';

	/**
	 * returns HTML for the Body Frame
	 *
	 * @return string
	 */
	static function getBody(): string{
		switch(we_base_request::int('step')){
			default:
			case 0:
				return self::getPage(self::getStep0());
			case 1:
				return self::getPage(self::getStep1());
			case 2:
				return self::getPage(self::getStep2());
			case 3:
				return self::getPage(self::getStep3());
		}
	}

	/**
	 * returns HTML for the Frame with the progress bar
	 *
	 * @return string
	 */
	static function getBusy(): string{
		$dc = we_base_request::bool("dc");

		$WE_PB = new we_gui_progressBar(0, ($dc ? 490 : 200));
		$WE_PB->addText("", we_gui_progressBar::TOP, "pb1");


		$cancelButton = we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();");
		$refreshButton = we_html_button::create_button(we_html_button::REFRESH, "javascript:WE().util.dialogTop(window).wizcmd.location.reload();", '', '', false, false);

		$nextbutdisabled = !(we_base_permission::hasPerm([we_base_perm::REBUILD_ALL, we_base_perm::REBUILD_FILTERD, we_base_perm::REBUILD_OBJECTS, we_base_perm::REBUILD_INDEX,
				we_base_perm::REBUILD_THUMBS, we_base_perm::REBUILD_META]));

		if($dc){
			$pb = we_html_tools::dialogLayout($WE_PB->getHTML(), g_l('rebuild', '[rebuild]'), $refreshButton . $cancelButton);
		} else {
			$content2 = we_html_element::span(['id' => 'prev', 'style' => "padding-left:10px;text-align:right"], we_html_button::create_button(we_html_button::BACK, "javascript:WE().util.dialogTop(window).wizbody.handle_event('previous');", '', '', true, false)) .
				we_html_element::span(['id' => 'next', 'style' => "padding-left:10px;text-align:right"], we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).wizbody.handle_event('next');", '', '', $nextbutdisabled, false)) .
				we_html_element::span(['id' => 'refresh', 'style' => "display:none; padding-left:10px;text-align:right"], $refreshButton) .
				we_html_element::span(['id' => 'cancel', 'style' => "padding-left:10px;text-align:right"], $cancelButton);

			$content = (new we_html_table(['class' => 'default', 'width' => "100%"], 1, 2))
				->setCol(0, 0, $WE_PB->getHTML('', 'display:none;'), ['id' => 'progr', 'style' => 'text-align:left'])
				->setCol(0, 1, $content2, ['style' => 'text-align:right']);
			$pb = $content->getHtml();
		}

		return we_html_tools::getHtmlTop('', we_gui_progressBar::getJSCode() .
				we_html_element::jsScript(JS_DIR . 'nextButtons.js'), we_html_element::body(
					['class' => ($dc ? 'weDialogBody' : 'editfooter'),], $pb)
		);
	}

	/**
	 * returns HTML for the Cmd Frame
	 *
	 * @return string for now it is an empty page
	 */
	static function getCmd(): string{
		return self::getPage(['', '']);
	}

	/**
	 * returns the HTML for the First Step (0) of the wizard
	 *
	 * @return array
	 */
	private static function getStep0(): array{
		$versionCts = we_versions_version::getContentTypesVersioning();
		$type = we_base_request::string('type', self::DELETE_VERSIONS);

		$version_delete = [
			'delete_date' => (string) we_base_request::_(we_base_request::RAW, 'delete_date', ''),
			'delete_hours' => (string) we_base_request::int('delete_hours'),
			'delete_minutes' => (string) we_base_request::int('delete_minutes'),
			'delete_seconds' => (string) we_base_request::int('delete_seconds'),
		];
		$version_reset = [
			'reset_date' => (string) we_base_request::_(we_base_request::RAW, 'reset_date', ''),
			'reset_hours' => (string) we_base_request::int('reset_hours'),
			'reset_minutes' => (string) we_base_request::int('reset_minutes'),
			'reset_seconds' => (string) we_base_request::int('reset_seconds'),
		];

		foreach($versionCts as $k){
			$version_delete[$k] = we_base_request::bool('version_delete_' . $k);
			$version_reset[$k] = we_base_request::bool('version_reset_' . $k);
		}

		$def = (we_base_request::string('type') === 'reset_versions');
		$version_reset['reset_doPublish'] = we_base_request::bool('reset_doPublish', $def);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::DELETE_VERSIONS, $type, "type", g_l('versions', '[delete_versions]'), true, "defaultfont", "", false, g_l('versions', '[txt_delete_versions]'), 0, 495)))
			->add(new we_html_multiIconBoxContentLine('', we_html_forms::radiobuttonV(self::RESET_VERSIONS, $type, 'type', g_l('versions', '[reset_versions]'), true, "defaultfont", "", false, g_l('versions', '[txt_reset_versions]'), 0, 495)));


		$js = '
window.onload = function(){
	WE().util.dialogTop(window).focus();
}
function handle_event(what){
	f = document.we_form;
	switch(what){
	case "previous":
		break;
	case "next":
		selectedValue="";
		for(var i=0;i<f.type.length;i++){
			if(f.type[i].checked){
				selectedValue = f.type[i].value;
			}
		}
		goTo(selectedValue);
		break;
	}
}
function goTo(where){
	f = document.we_form;
	switch(where){
	case "rebuild_thumbnails":
	case "delete_versions":
		f.target="wizbody";
		break;
	case "rebuild_objects":
	case "rebuild_index":
	case "rebuild_navigation":
		set_button_state(1);
		f.target="wizcmd";
		f.step.value="2";
		break;
	}
	f.submit();
}
function set_button_state(alldis) {
	if(WE().util.dialogTop(window).wizbusy){
		WE().util.dialogTop(window).wizbusy.back_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "back", "disabled");
		if(alldis){
			WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "disabled");
			WE().util.dialogTop(window).wizbusy.showRefreshButton();
		}else{
			WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
		}
	}else{
		window.setTimeout(set_button_state,300,(alldis ? 1 : 0));
	}
}
set_button_state(false);';

		$hiddenFields = we_html_element::hiddens($version_delete) .
			we_html_element::hiddens($version_reset);

		return [$js,
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('versions', '[versioning]'))->get() .
			$hiddenFields .
			we_html_element::hiddens([
				'fr' => 'body',
				'we_cmd[0]' => 'we_versions_wizard',
				'step' => '1'])
		];
	}

	/**
	 * returns the HTML for the Second Step (1) of the wizard
	 *
	 * @return array
	 */
	private static function getStep1(): array{
		switch(we_base_request::string("type", self::DELETE_VERSIONS)){
			case self::DELETE_VERSIONS:
				return self::getDelete1();
			case self::RESET_VERSIONS:
				return self::getReset1();
		}
		return [];
	}

	private static function getDelete1(): array{
		$versionCts = we_versions_version::getContentTypesVersioning();
		$type = we_base_request::string("type", self::DELETE_VERSIONS);

		//	$versions_delete_all = we_base_request::bool("version_delete_all");
		$version_delete_date = we_base_request::_(we_base_request::RAW, "delete_date", "");
		$version_delete_hours = we_base_request::int("delete_hours", 0);
		$version_delete_minutes = we_base_request::int("delete_minutes", 0);
		$version_delete_seconds = we_base_request::int("delete_seconds", 0);

		$content = '';
		foreach($versionCts as $k){
			$txt = (string) $k;
			$val = $name = 'version_delete_' . $txt;
			$checked = we_base_request::_(we_base_request::RAW, $txt, 0);
			if($txt === 'all'){
				$jvs = "checkAll(this);";
				$content .= we_html_forms::checkbox($val, $checked, $name, g_l('versions', '[versions_all]'), false, "defaultfont", $jvs) . "<br/>";
			} else {
				$jvs = "checkAllRevert(this);";
				$content .= we_html_forms::checkbox($val, $checked, $name, g_l('contentTypes', '[' . $txt . ']'), false, "defaultfont", $jvs) . "<br/>";
			}
		}
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('versions', '[ct_delete_text]'), we_html_tools::TYPE_INFO, 520)))
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('versions', '[ContentType]'), $content))
			->space(we_html_multiIconBox::SPACE_BIG)
			->noline()
		);


		$versions_delete_date = we_html_tools::getNewDateInput('delete_date', new DateTime($version_delete_date), we_html_tools::FORMAT_DATE);

		$reset_hours = new we_html_select(['id' => "delete_hours",
			'name' => "delete_hours",
			'style' => '',
			'class' => "weSelect",
			'onchange' => ''
		]);

		for($x = 0; $x <= 23; $x++){
			$reset_hours->addOption((string) ($x), sprintf("%'.02d", $x));
		}

		$reset_hours->selectOption($version_delete_hours);

		$reset_minutes = new we_html_select(['id' => "delete_minutes",
			'name' => "delete_minutes",
			'style' => '',
			'class' => "weSelect",
			'onchange' => ''
		]);
		$reset_seconds = new we_html_select([
			'id' => 'delete_seconds',
			'name' => 'delete_seconds',
			'style' => '',
			'class' => "weSelect",
			'onchange' => ''
		]);

		for($x = 0; $x <= 59; $x++){
			$reset_minutes->addOption((string) ($x), sprintf("%'.02d", $x));
			$reset_seconds->addOption((string) ($x), sprintf("%'.02d", $x));
		}

		$reset_minutes->selectOption($version_delete_minutes);
		$reset_seconds->selectOption($version_delete_seconds);

		$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('versions', '[date_delete_text]'), we_html_tools::TYPE_INFO, 520)))
				->noline()
		);

		$clearDate = we_html_button::create_button(we_html_button::RESET, "javascript:document.getElementById('delete_date').value='';", '', '', false, false);


		$parts->add((new we_html_multiIconBoxContentLine(g_l('versions', '[time]'), '<div style="padding-bottom:3px;">' . g_l('versions', '[day]') . ':</div><div style="float:left;">' . $versions_delete_date . '</div><div style="float:left;margin: 0 0 10px 10px;">' . $clearDate . "</div><br style='clear:left;' /><div style='padding-bottom:3px;'>" . g_l('versions', '[clocktime]') . ":</div>" . $reset_hours->getHtml() . " h : " . $reset_minutes->getHtml() . " m: " . $reset_seconds->getHtml() . " s"))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
		);

		//js
		$jsCheckboxCheckAll = $jsCheckboxCtIf = '';

		//$jsCheckboxArgs = '';
		foreach($versionCts as $k){
			if($k != "all"){
				$jsCheckboxCheckAll .= 'document.getElementById("version_delete_' . $k . '").checked = checked;';
			}
			$jsCheckboxCtIf .= (empty($jsCheckboxCtIf) ? '' : ' && ') . 'document.getElementById("version_delete_' . $k . '").checked==0';
			//$jsCheckboxArgs .= 'args += "&ct[' . $k . ']="+encodeURI(document.getElementById("version_delete_' . $k . '").checked);';
		}

		$nextButton = we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).wizbody.handle_event(\"next\");", '', '', false, false);

		$js = '
window.onload = function(){
	WE().util.dialogTop(window).focus();
}
function handle_event(what){
	f = document.we_form;
	switch(what){
		case "previous":
			f.step.value=0
			f.target="wizbody";
			f.submit();
			break;
		case "next":
			var date = document.getElementById("delete_date").value;
			var hour = document.getElementById("delete_hours").value;
			var minutes = document.getElementById("delete_minutes").value;
			var seconds = document.getElementById("delete_seconds").value;
			if(' . $jsCheckboxCtIf . ') {
				WE().util.showMessage(WE().consts.g_l.alert.notCheckedContentType,WE().consts.message.WE_MESSAGE_NOTICE, window);
			}else {
				selectedValue="";
				for(var i=0;i<f.type.length;i++){
					if(f.type[i].checked){;
						selectedValue = f.type[i].value;
					}
				}
				goTo(selectedValue);
			}
		break;
	}
}

function checkAll(val) {
	if(val.checked) {
		checked = 1;
	}else {
		checked = 0;
	}' . $jsCheckboxCheckAll . ';

}

function checkAllRevert() {//FIXME:unused (box doesnt exist?)
	var checkbox = document.getElementById("version_delete_all");
	checkbox.checked = false;
}

function goTo(where){
	f = document.we_form;
	switch(where){
		case "delete_versions":
			f.target="wizbody";
			break;
	}
	f.submit();
}

function set_button_state(alldis) {
					if(WE().util.dialogTop(window).wizbusy){
						WE().util.dialogTop(window).wizbusy.back_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "back", "enabled");
		if(alldis){
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							WE().util.dialogTop(window).wizbusy.showRefreshButton();
		}else{
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							var nextBut = WE().util.dialogTop(window).wizbusy.document.getElementById(\'next\');
				nextBut.innerHTML = \'' . $nextButton . '\';
		}
	}else{
		window.setTimeout(set_button_state,300,(alldis ? 1 : 0));
	}
}
set_button_state(false);';


		return [$js,
				(new we_html_multiIconBox($parts))->marginLeft()
				->headline(g_l('versions', '[delete_versions]') . ' - ' . g_l('versions', '[step]') . ' 1 ' . g_l('versions', '[of]') . ' 2')
				->get() .
			we_html_element::hiddens(['fr' => 'body',
				'type' => $type,
				'we_cmd[0]' => 'we_versions_wizard',
				'step' => '2'])
		];
	}

	private static function getReset1(): array{
		$versionCts = we_versions_version::getContentTypesVersioning();
		$type = we_base_request::_(we_base_request::RAW, 'type', self::RESET_VERSIONS);

		$version_reset_date = we_base_request::_(we_base_request::RAW, 'reset_date', '');
		$version_reset_hours = we_base_request::int('reset_hours', 0);
		$version_reset_minutes = we_base_request::int('reset_minutes', 0);
		$version_reset_seconds = we_base_request::int('reset_seconds', 0);
		$version_reset_doPublish = we_base_request::bool('reset_doPublish');


		$content = '';
		foreach($versionCts as $k){
			$txt = (string) $k;
			$name = 'version_reset_' . $txt;
			$val = 'version_reset_' . $txt;
			$checked = we_base_request::_(we_base_request::RAW, $txt, 0);
			if($txt === "all"){
				$jvs = "checkAll(this);";
				$content .= we_html_forms::checkbox($val, $checked, $name, g_l('versions', '[versions_all]'), false, 'defaultfont', $jvs) . '<br/>';
			} else {
				$jvs = "checkAllRevert(this);";
				$content .= we_html_forms::checkbox($val, $checked, $name, g_l('contentTypes', '[' . $txt . ']'), false, 'defaultfont', $jvs) . '<br/>';
			}
		}

		$versions_reset_date = we_html_tools::getNewDateInput('reset_date', new DateTime($version_reset_date), we_html_tools::FORMAT_DATE);
		$doPublish = we_html_forms::checkbox((string) $version_reset_doPublish, $version_reset_doPublish, "reset_doPublish", g_l('versions', '[publishIfReset]'));

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('versions', '[ct_reset_text]'), we_html_tools::TYPE_INFO, 520)))
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine(g_l('versions', '[ContentType]'), $content))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('versions', '[doPublish_text]'), we_html_tools::TYPE_INFO, 520)))
				->noline()
			)
			->add((new we_html_multiIconBoxContentLine('', $doPublish))
			->noline()
		);

		$reset_hours = new we_html_select([
			'id' => "reset_hours",
			'name' => "reset_hours",
			'style' => '',
			'class' => "weSelect",
			'onchange' => ''
		]);

		for($x = 0; $x <= 23; $x++){
			$reset_hours->addOption((string) $x, sprintf('%02d', $x));
		}

		$reset_hours->selectOption($version_reset_hours);

		$reset_minutes = new we_html_select([
			'id' => 'reset_minutes',
			'name' => 'reset_minutes',
			'style' => '',
			'class' => 'weSelect',
			'onchange' => ''
		]);

		for($x = 0; $x <= 59; $x++){
			$reset_minutes->addOption((string) $x, sprintf('%02d', $x));
		}

		$reset_minutes->selectOption($version_reset_minutes);

		$reset_seconds = new we_html_select([
			'id' => 'reset_seconds',
			'name' => 'reset_seconds',
			'style' => '',
			'class' => 'weSelect',
			'onchange' => ''
		]);

		for($x = 0; $x <= 59; $x++){
			$reset_seconds->addOption((string) $x, sprintf('%02d', $x));
		}

		$reset_seconds->selectOption($version_reset_seconds);

		$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('versions', '[date_reset_text]'), we_html_tools::TYPE_INFO, 520)))
				->noline()
		);

		$clearDate = we_html_button::create_button(we_html_button::RESET, "javascript:document.getElementById('reset_date').value='';", '', '', false, false);

		$parts->add((new we_html_multiIconBoxContentLine(g_l('versions', '[time]'), "<div style='padding-bottom:3px;'>Tag:</div><div style='float:left;'>" . $versions_reset_date . "</div><div style='float:left;margin:0 0 10px 10px;'>" . $clearDate . "</div><br style='clear:left;' /><div style='padding-bottom:3px;'>Uhrzeit:</div>" . $reset_hours->getHtml() . " h : " . $reset_minutes->getHtml() . " m: " . $reset_seconds->getHtml() . " s "))
				->noline()
				->space(we_html_multiIconBox::SPACE_BIG)
		);

		//js
		$jsCheckboxCheckAll = '';
		$jsCheckboxCtIf = [];

		//$jsCheckboxArgs = '';
		foreach($versionCts as $k){
			if($k != "all"){
				$jsCheckboxCheckAll .= 'document.getElementById("version_reset_' . $k . '").checked = checked;';
			}
			$jsCheckboxCtIf[] = 'document.getElementById("version_reset_' . $k . '").checked==0';
//			$jsCheckboxArgs .= 'args += "&ct[' . $k . ']="+encodeURI(document.getElementById("version_reset_' . $k . '").checked);';
		}

		$nextButton = we_html_button::create_button(we_html_button::NEXT, "javascript:WE().util.dialogTop(window).wizbody.handle_event(\"next\");", '', "", false, false);

		$js = 'window.onload = function(){
					WE().util.dialogTop(window).focus();
				}
				function handle_event(what){
					f = document.we_form;
					switch(what){
						case "previous":
							f.step.value=0
							f.target="wizbody";
							f.submit();
							break;
						case "next":
							var date = document.getElementById("reset_date").value;
							var hour = document.getElementById("reset_hours").value;
							var minutes = document.getElementById("reset_minutes").value;
							var seconds = document.getElementById("reset_seconds").value;
							if(' . implode('&&', $jsCheckboxCtIf) . ') {
								WE().util.showMessage(WE().consts.g_l.alert.notCheckedContentType,WE().consts.message.WE_MESSAGE_NOTICE, window);
							}
							else if(date=="") {
							WE().util.showMessage(WE().consts.g_l.alert.notCheckedDate,WE().consts.message.WE_MESSAGE_NOTICE, window);
							}
							else {
								selectedValue="";
								for(var i=0;i<f.type.length;i++){
									if(f.type[i].checked){;
										selectedValue = f.type[i].value;
									}
								}
								goTo(selectedValue);
							}
						break;
					}
				}

				function checkAll(val) {
		            	if(val.checked) {
		            		checked = 1;
		            	}else {
		            		checked = 0;
		            	}
						' . $jsCheckboxCheckAll . ';

					}

	            	function checkAllRevert() {//FIXME:unused (box doesnt exist?)
	            		var checkbox = document.getElementById("version_reset_all");
						checkbox.checked = false;
	            	}


				function goTo(where){
					f = document.we_form;
					switch(where){
						case "reset_versions":
							f.target="wizbody";
							break;
					}
					f.submit();
				}

				function set_button_state(alldis) {
					if(WE().util.dialogTop(window).wizbusy){
						WE().util.dialogTop(window).wizbusy.back_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "back", "enabled");
						if(alldis){
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							WE().util.dialogTop(window).wizbusy.showRefreshButton();
						}else{
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							var nextBut = WE().util.dialogTop(window).wizbusy.document.getElementById("next");
				  			nextBut.innerHTML = \'' . $nextButton . '\';
						}
					}else{
						window.setTimeout(set_button_state,300,(alldis ? 1 : 0));
					}
				}
				set_button_state(false);';

		return [$js,
				(new we_html_multiIconBox($parts))
				->marginLeft()->
				headline(g_l('versions', '[reset_versions]') . " - " . g_l('versions', '[step]') . " 1 " . g_l('versions', '[of]') . " 2")
				->get() .
			we_html_element::hiddens(['fr' => 'body',
				'type' => (string) ($type),
				'we_cmd[0]' => 'we_versions_wizard',
				'step' => '2'])
		];
	}

	/**
	 * returns the HTML for the Third Step (2) of the wizard. - Here the real work (loop) is done - it should be displayed in the cmd frame
	 *
	 * @return array
	 */
	private static function getStep2(): array{
		$type = we_base_request::string("type", self::DELETE_VERSIONS);

		switch($type){
			case self::DELETE_VERSIONS:
				return self::getDelete2();
			case self::RESET_VERSIONS:
				return self::getReset2();
		}
		return [];
	}

	private static function getStep3(): array{
//		$versionCts = we_versions_version::getContentTypesVersioning();
		$type = we_base_request::string('type', self::DELETE_VERSIONS);

		/* 		$version_delete = [];
		  $version_reset = [];

		  foreach($versionCts as $k){
		  $version_delete[$k] = we_base_request::bool('version_delete_' . $k);
		  $version_reset[$k] = we_base_request::bool('version_reset_' . $k);
		  }

		  $version_delete['delete_date'] = we_base_request::_(we_base_request::RAW, 'delete_date', "");
		  $version_delete['delete_hours'] = we_base_request::int('delete_hours', 0);
		  $version_delete['delete_minutes'] = we_base_request::int('delete_minutes', 0);
		  $version_delete['delete_seconds'] = we_base_request::int('delete_seconds', 0);

		  $version_reset['reset_date'] = we_base_request::_(we_base_request::RAW, 'reset_date', '');
		  $version_reset['reset_hours'] = we_base_request::int('reset_hours', 0);
		  $version_reset['reset_minutes'] = we_base_request::int('reset_minutes', 0);
		  $version_reset['reset_seconds'] = we_base_request::int('reset_seconds', 0);
		  $version_reset['reset_doPublish'] = we_base_request::bool('reset_doPublish', (we_base_request::string('type') === 'reset_versions'));
		 */

		$taskname = md5(session_id() . '_version_wizard');
		$currentTask = we_base_request::bool('doFragments'); //true if data was already set
		$taskFilename = WE_FRAGMENT_PATH . $taskname;

		$js = '';
		if(!(file_exists($taskFilename) && $currentTask)){
			switch($type){
				case self::DELETE_VERSIONS:
					$data = we_versions_fragment::getDocuments($type/* , $version_delete */);
					break;
				case self::RESET_VERSIONS:
					$data = we_versions_fragment::getDocuments($type/* , $version_reset */);
					break;
			}
			if(!empty($data)){
				$frag = new we_versions_fragment($taskname, 5, [], $data);
				$frag->run();
				return [];
			}
			return [$js . 'WE().util.showMessage(WE().consts.g_l.versions.deleteNothingFound,WE().consts.message.WE_MESSAGE_NOTICE, window);WE().util.dialogTop(window).wizbusy.showPrevNextButton();',
				""
			];
		}
		$frag = new we_versions_fragment($taskname, 5);
		$frag->run();
		return [];
	}

	/**
	 * returns Array with javascript (array[0]) and HTML Content (array[1]) for the rebuild document page
	 *
	 * @return array
	 */
	static function getDelete2(): array{
		$versionCts = we_versions_version::getContentTypesVersioning();

		$type = we_base_request::string('type', self::DELETE_VERSIONS);

		$version_delete = [
			'delete_date' => we_base_request::_(we_base_request::RAW, 'delete_date', ''),
			'delete_hours' => we_base_request::int('delete_hours', 0),
			'delete_minutes' => we_base_request::int('delete_minutes', 0),
			'delete_seconds' => we_base_request::int('delete_seconds', 0),
		];

		foreach($versionCts as $k){
			$version_delete[$k] = we_base_request::bool('version_delete_' . $k);
		}

		$timestampWhere = 1;
		if($version_delete['delete_date'] != ''){
			$date = new DateTime(we_base_request::string('delete_date'));
			$timestampWhere = ' logDate<"' . $date->format($date->format(we_database_base::DATE_TIME_FORMAT)) . '"';
		} else {
			$date = null;
		}

		$parts = new we_html_multiIconBoxContent();

		$whereCtA = [];
		foreach($version_delete as $k => $v){
			switch($k){
				case 'all':
				case 'delete_date':
				case 'delete_hours':
				case 'delete_minutes':
				case 'delete_seconds':
					break;
				default:
					if($v){
						$whereCtA[] = '"' . $k . '"';
					}
			}
		}
		$whereCt = ($whereCtA ? ' ContentType IN (' . implode(',', $whereCtA) . ')' : '1');

		$cont = $docIds = [];
		$_SESSION['weS']['versions']['deleteWizardWhere'] = $whereCt . ' AND ' . $timestampWhere;
		$_SESSION['weS']['versions']['logDeleteIds'] = [];
		foreach(DB_WE::$inst->getAllq('SELECT ID,documentID,documentTable,Text AS text,Path AS path,ContentType,binaryPath,logDate,version FROM ' . VERSIONS_TABLE . ' WHERE ' . $whereCt . ' AND ' . $timestampWhere . ' ORDER BY ID') as $item){
			if(!in_array($item['documentID'], $docIds)){
				$docIds[$item['documentID']] = [
					'Path' => $item['path'],
					'ContentType' => $item['ContentType']
				];
			}

			$cont[] = $item;
			$_SESSION['weS']['versions']['logDeleteIds'][$item['ID']] = [
				'Text' => $item['text'],
				'ContentType' => $item['ContentType'],
				'Path' => $item['path'],
				'Version' => $item['version'],
				'documentID' => $item['documentID'],
			];
			if($item['binaryPath']){
				$_SESSION['weS']['versions']['deleteWizardbinaryPath'][] = $item['binaryPath'];
			}
		}

		$out = '<div style="width:520px;">' .
			g_l('versions', '[step2_txt1]') .
			($date ?
			sprintf(g_l('versions', '[step2_txt2_delete]'), $date->format(g_l('date', '[format][default]'))) :
			'') .
			g_l('versions', '[step2_txt3]') .
			'</div>
<div style="background-color:#fff;width:520px;margin-top:20px;">
	<table style="width:100%">
		<tr class="defaultfont" style="height:30px;">
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[_id]') . '</th>
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[path]') . '</th>
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[ContentType]') . '</th>
		</tr>';

		foreach($docIds as $k => $v){
			$out .= '
<tr class="defaultfont">
	<td style="text-align:center">' . $k . '</td>
	<td style="text-align:center">' . we_base_util::shortenPath($v['Path'], 55) . '</td>
	<td style="text-align:center">' . $v['ContentType'] . '</td>
</tr>';
		}
		$out .= '</table>
			</div>';

		$parts->add(new we_html_multiIconBoxContentLine('', $out));

		$hiddenFields = we_html_element::hiddens($version_delete);

		return [self::getPage2Js(!empty($cont), 'delete'),
				(new we_html_multiIconBox($parts))
				->marginLeft()
				->headline(g_l('versions', '[delete_versions]') . ' - ' . g_l('versions', '[step]') . ' 2 ' . g_l('versions', '[of]') . ' 2')
				->get() .
			$hiddenFields .
			we_html_element::hiddens([
				'fr' => 'body',
				'type' => $type,
				'we_cmd[0]' => 'we_versions_wizard',
				'step' => '3']
		)];
	}

	static function getReset2(): array{
		$type = we_base_request::_(we_base_request::RAW, "type", self::RESET_VERSIONS);

		$_SESSION['weS']['versions']['logResetIds'] = [];

		$version_reset = [
			'reset_date' => we_base_request::string('reset_date', ''),
			'reset_hours' => we_base_request::int('reset_hours', 0),
			'reset_minutes' => we_base_request::int('reset_minutes', 0),
			'reset_seconds' => we_base_request::int('reset_seconds', 0),
		];

		foreach(we_versions_version::getContentTypesVersioning() as $k){
			$version_reset[$k] = we_base_request::bool('version_reset_' . $k);
		}

		$def = (we_base_request::string('type') === 'reset_versions');
		$version_reset['reset_doPublish'] = we_base_request::bool('reset_doPublish', $def);

		$date = ($version_reset['reset_date'] ?
			new DateTime($version_reset['reset_date']) :
			new DateTime());

		$w = [];
		foreach($version_reset as $k => $v){
			switch($k){
				case 'all':
				case 'reset_date':
				case 'reset_hours':
				case 'reset_minutes':
				case 'reset_seconds':
				case 'reset_doPublish':
					break;
				case we_base_ContentTypes::WEDOCUMENT:
				case we_base_ContentTypes::HTML:
				case we_base_ContentTypes::OBJECT_FILE:
					if($v){
						$w[] = '(ContentType="' . $k . '" AND status="published" )';
					}
					break;
				default:
					if($v){
						$w[] = ' ContentType="' . $k . '" ';
					}
					break;
			}
		}

		$cont = $docIds = [];

		$_SESSION['weS']['versions']['query'] = 'SELECT ID,documentID,documentTable,CONCAT(Filename,COALESCE(Extension,"")) AS Text,Path,ContentType,logDate,MAX(version) as version FROM ' . VERSIONS_TABLE . ' WHERE logDate<="' . $date->format(we_database_base::DATE_TIME_FORMAT) . '"' . ($w ? ' AND (' . implode(' OR ', $w) . ') ' : '') . ' GROUP BY documentTable,documentID ORDER BY version DESC';
		$all = DB_WE::$inst->getAllq($_SESSION['weS']['versions']['query']);
		foreach($all as $rec){
			if(!in_array($rec['documentID'], $docIds)){
				$docIds[$rec['documentID']]['Path'] = $rec['Path'];
				$docIds[$rec['documentID']]['ContentType'] = $rec['ContentType'];
			}

			$cont[] = [
				'id' => $rec['ID'],
				'documentID' => $rec['documentID'],
				'version' => $rec['version'],
				'text' => $rec['Text'],
				'path' => $rec['Path'],
				'table' => $rec['documentTable'],
				'contentType' => $rec['ContentType'],
				'logDate' => $rec['logDate']
			];
		}

//FIXME: date format should be obtained from g_l
		$out = '
<div style="width:520px;">' . sprintf(g_l('versions', '[step2_txt_reset]'), $date->format(g_l('date', '[format][default]'))) . '</div>
<div style="background-color:#fff;width:520px;margin-top:20px;">
	<table style="width:100%">
	<tr class="defaultfont" style="height:30px;">
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[_id]') . '</th>
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[path]') . '</th>
		<th style="border-bottom:1px solid #B7B5B6;">' . g_l('versions', '[ContentType]') . '</th>
	</tr>';

		foreach($docIds as $k => $v){
			$out .= '
<tr class="defaultfont">
	<td style="text-align:center">' . $k . '</td>
	<td style="text-align:center">' . we_base_util::shortenPath($v['Path'], 55) . '</td>
	<td style="text-align:center">' . $v['ContentType'] . '</td>
</tr>';
		}
		$out .= '</table>
			</div>';

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine('', $out));

		$hiddenFields = we_html_element::hiddens($version_reset);

		return [self::getPage2Js(!empty($cont), 'reset'),
				(new we_html_multiIconBox($parts))
				->marginLeft()
				->headline(g_l('versions', '[reset_versions]') . " - " . g_l('versions', '[step]') . " 2 " . g_l('versions', '[of]') . " 2")
				->get() .
			$hiddenFields .
			we_html_element::hiddens(['fr' => "body",
				'type' => (string) ($type),
				'we_cmd[0]' => 'we_versions_wizard',
				'step' => '3'])
		];
	}

	/**
	 * returns HTML for the frameset
	 *
	 * @return string
	 */
	static function getFrameset(): string{
		$query = [];
		if(($btype = we_base_request::string('btype', we_base_request::NOT_VALID_S)) !== we_base_request::NOT_VALID_S){
			$query['btype'] = $btype;
		}
		if(($type = we_base_request::string('type'))){
			$query['type'] = $type;
		}
		if(($tid = we_base_request::int('templateID', -1)) > -1){
			$query['templateID'] = $tid;
		}
		if(($step = we_base_request::int('step', -1)) > -1){
			$query['step'] = $step;
		}
		if(($text = we_base_request::string('responseText', we_base_request::NOT_VALID_S)) !== we_base_request::NOT_VALID_S){
			$query['responseText'] = $text;
		}

		$taskname = md5(session_id() . '_version_wizard');
		$taskFilename = WE_FRAGMENT_PATH . '/' . $taskname;
		if(file_exists($taskFilename)){
			we_base_file::delete($taskFilename);
		}

		if($query){
			$query['we_cmd'][0] = 'we_versions_wizard';
			$query['fr'] = 'body';
			//maybe restore of a given version?
			$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => "wizcmd.location='" . WEBEDITION_DIR . 'we_cmd.php?' . http_build_query($query) . "';"]
					, we_html_element::iframe('wizbusy', WEBEDITION_DIR . 'we_cmd.php?' . http_build_query(['we_cmd' => [0 => 'we_versions_wizard'], 'fr' => 'busy',
							'dc' => 1]), 'position:absolute;top:0;bottom:0;left:0;right:0;overflow: hidden;') .
					we_html_element::iframe('wizcmd', "about:blank", 'position:absolute;height:0;bottom:0;left:0;right:0;overflow: hidden;')
			);
		} else {
			$body = we_html_element::body(['class' => 'weDialogBody']
					, we_html_element::iframe('wizbody', WEBEDITION_DIR . 'we_cmd.php?' . http_build_query(['we_cmd' => [0 => 'we_versions_wizard'], 'fr' => 'body']), 'position:absolute;top:0;bottom:40px;left:0;right:0;') .
					we_html_element::iframe('wizbusy', WEBEDITION_DIR . 'we_cmd.php?' . http_build_query(['we_cmd' => [0 => 'we_versions_wizard'], 'fr' => 'busy']), 'position:absolute;bottom:0;height:40px;left:0;right:0;overflow: hidden;') .
					we_html_element::iframe('wizcmd', WEBEDITION_DIR . 'we_cmd.php?' . http_build_query(['we_cmd' => [0 => 'we_versions_wizard'], 'fr' => 'cmd']), 'position:absolute;height:0;bottom:0;left:0;right:0;overflow: hidden;')
			);
		}

		return we_html_tools::getHtmlTop(g_l('versions', '[versions_wizard]'), '', $body);
	}

	/**
	 * returns Javascript for step 2 (1)
	 *
	 * @return string
	 * @param bool $cont
	 * @param string $action
	 */
	static function getPage2Js(bool $cont, string $action): string{
		$disabled = !$cont;
		//reset
		$act = ($action === 'delete' ? 0 : 1);

		$nextButton = we_html_button::create_button('go', 'javascript:WE().util.dialogTop(window).wizbody.handle_event("next");', '', '', $disabled, false);
		$publish = we_base_request::bool('reset_doPublish');
		$we_transaction = we_base_WE::$transaction;
		return '
WE().util.loadConsts(document, "g_l.versions");

window.onload = function(){
	WE().util.dialogTop(window).focus();
}
function handle_event(what){
	f = document.we_form;
	switch(what){
		case "previous":
			f.step.value=1
			f.target="wizbody";
			f.submit();
			break;
		case "next":
				selectedValue="";
				for(var i=0;i<f.type.length;i++){
					if(f.type[i].checked){;
						selectedValue = f.type[i].value;
					}
				}
				goTo(selectedValue);

		break;
	}
}

function ajaxCallbackDeleteVersionsWizard (responseText){
	if(responseText != "") {
		WE().util.dialogTop(window).wizbusy.document.getElementById("progr").innerHTML = responseText;
		WE().util.showMessage(WE().consts.g_l.versions.deleteDateVersionsOK,WE().consts.message.WE_MESSAGE_NOTICE, window);
		// reload current document => reload all open Editors on demand

		var _usedEditors =  WE().layout.weEditorFrameController.getEditorsInUse();
		for (frameId in _usedEditors) {

			if ( _usedEditors[frameId].getEditorIsActive() ) { // reload active editor
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
				_usedEditors[frameId].setEditorIsActive(true);

			} else {
				_usedEditors[frameId].setEditorReloadAllNeeded(true);
			}
		}

		//reload tree
		WE().util.dialogTop(window).opener.we_cmd("load", WE().util.dialogTop(window).opener.top.treeData.table ,0);
		WE().util.dialogTop(window).doClose();
	}
}

function ajaxCallbackResetVersionsWizard(responseText) {
	if(responseText != "") {
		WE().util.dialogTop(window).wizbusy.document.getElementById("progr").innerHTML = responseText;
		WE().util.showMessage(WE().consts.g_l.versions.resetAllVersionsOK,WE().consts.message.WE_MESSAGE_NOTICE, window);
		WE().util.dialogTop(window).doClose();
	}
}

function goTo(where){

	if(' . $act . ') {
	var f = document.we_form;
		switch(where){
			case "delete_versions":
				f.target="wizbody";
				break;
		}
		f.submit();

//						WE().util.dialogTop(window).wizbusy.document.getElementById("progr").style.display = "block";
/*	WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?cmd=ResetVersionsWizard","publish=' . (string) $publish . '&we_transaction=' . $we_transaction . '&cns=versionlist", ajaxCallbackResetVersionsWizard,"html");
*/
//
	}	else {
		WE().util.dialogTop(window).wizbusy.document.getElementById("progr").style.display = "block";
		WE().util.rpc(WE().consts.dirs.WEBEDITION_DIR + "rpc.php?mod=version&cmd=versionDeleteWizard", "",ajaxCallbackDeleteVersionsWizard,"html");
	}
}

function set_button_state(alldis) {
					if(WE().util.dialogTop(window).wizbusy){
						WE().util.dialogTop(window).wizbusy.back_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "back", "enabled");
		if(alldis){
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							WE().util.dialogTop(window).wizbusy.showRefreshButton();
		}else{
							WE().util.dialogTop(window).wizbusy.next_enabled = WE().layout.button.switch_button_state(WE().util.dialogTop(window).wizbusy.document, "next", "enabled");
							var nextBut = WE().util.dialogTop(window).wizbusy.document.getElementById(\'next\');
				nextBut.innerHTML = \'' . $nextButton . '\';
		}
	}else{
		window.setTimeout(set_button_state,300,(alldis ? 1 : 0));
	}
}
set_button_state(false);';
	}

	/**
	 * returns Javascript for step 2 (1)
	 *
	 * @return string
	 * @param array{string,string} $contents first element (array[0]) must be a javascript, second element (array[1]) must be the Body HTML
	 */
	static function getPage(array $contents): string{
		if(!count($contents)){
			return '';
		}

		return we_html_tools::getHtmlTop('', ($contents[0] ? we_html_element::jsElement($contents[0]) : ""), we_html_element::body(
					['class' => 'weDialogBody']
					, we_html_element::form(['name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php'
						], $contents[1]))
		);
	}

	public static function showFrameset(): void{
		switch(we_base_request::string("fr", '')){
			case 'body':
				echo self::getBody();
				break;
			case 'busy':
				echo self::getBusy();
				break;
			case 'cmd':
				echo self::getCmd();
				break;
			default:
				echo self::getFrameset();
		}
	}

}
